!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Calculates coordinates, metric elements and curvatures using fast Fourier transform on regular grid.

!latex \item The coordinates are constructed by an interpolation of the Fourier harmonics that describe the interface geometry.

!latex \item Note that if \verb+Lgeometry.le.3+, the $Z$ function is not required.

!latex \end{enumerate} \subsubsection{logic overview}

!latex The input parameter \verb+Lcurv+ controls the operation:

!latex \begin{enumerate}

!latex \item if \verb+Lcurv=0+: The coordinates are returned.
!latex \item if \verb+Lcurv=1+: The coordinates and the metric elements are returned. This is required for the computation of volume integrals.
!latex \item if \verb+Lcurv=2+: The coordinates and the metric elements and the curvatures are returned. The curvatures are required to compute the current.
!latex \item if \verb+Lcurv=3+: The coordinates and the metrics are returned, and the derivatives of the coordinates and metrics with respect to interface geometry are returned;
!latex                          normalization is included.
!latex \item if \verb+Lcurv=4+: The coordinates and the metrics are returned, and the derivatives of the coordinates and metrics with respect to interface geometry are returned.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine co01aa( lvol, lss, Lcurv, Ntz )
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  use constants, only : zero, half, one, two

  use numerical, only : sqrtmachprec, vsmall

  use fileunits, only : ounit

  use inputlist, only : Wco01aa, Itoroidal, Nvol, Ni, Lgeometry

  use cputiming, only : Tco01aa

  use allglobal, only : myid, cpus, mn, im, in, interfacelabel, ss, &
                        iRbc, iZbs, Rmn, Zmn, SumNi, pi2nfp,        &
                        costeta,                                    &
                        Nt, Nz, isr, trigm, trign, trigwk,          &
                        efmn, ofmn, cfmn, sfmn, ijreal, ijimag,     &
                        sgmn,                                       & ! this is nowhere used;
                        Rij, Zij,                                   &
                        cosj, sinj,                                 &
                          sg,   gij,                                &
                        dxsg, dxgij, dAd,                           &
                        Lcoordinatesingularity
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  LOCALS

  INTEGER, intent(in) :: lvol  ! labels volume;
  REAL   , intent(in) :: lss   ! radial coordinate;
  INTEGER, intent(in) :: Lcurv ! flag indicating that derivatives are to be calculated;
  INTEGER, intent(in) :: Ntz   ! resolution parameter; this is really global but included as argument to simplify allocations;
  
  INTEGER             :: imn, ii, ij, ik, ie, if, ig, ifail, igij, lNi
  REAL                :: hmm, isshm

  REAL                :: xx, ds, ixx, oxx

  REAL                :: dXdXj(0:3,1:Ntz)
  
  BEGIN(co01aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  FATALMESS(co01aa,lvol.lt.1.or.lvol.gt.Nvol,invalid interface label)

  FATALMESS(co01aa,lss.lt.interfacelabel(lvol-1)-vsmall .or. lss.gt.interfacelabel(lvol)+vsmall,radial coordinate is outside identified volume) ! 10 Oct 12;

  FATALMESS(co01aa,Lcurv.lt.0.or.Lcurv.gt.4,invalid input value for Lcurv) ! 10 Oct 12;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  Rij(0:9,1:Ntz) = zero ; sg(0:3,1:Ntz) = zero ; gij(1:6,0:3,1:Ntz) = zero ! initialize summation variables; these are returned through global;
  Zij(0:9,1:Ntz) = zero
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  Rmn(0:SumNi,0:2,1:mn) = zero ! initialize summation variables; recall that these are global; under construction ??? ;
  Zmn(0:SumNi,0:2,1:mn) = zero ! initialize summation variables; recall that these are global; under construction ??? ;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  do imn = 1, mn ! loop over Fourier modes; perform interpolation;
   
!  if( imask(imn).le.0 ) cycle ! under construction;
   
!latex \end{enumerate} \subsubsection{coordinate interpolation} \begin{enumerate}
   
!latex \item The coordinate interpolation is a linear interpolation between the interfaces:
!latex \be \begin{array}{ccccccc} R_j &=& R_{l-1,j} + \left(R_{l,j}-R_{l-1,j}\right) ( s - s_{l-1} ) / ( s_{l}-s_{l-1}),\\
!latex                            Z_j &=& Z_{l-1,j} + \left(Z_{l,j}-Z_{l-1,j}\right) ( s - s_{l-1} ) / ( s_{l}-s_{l-1}),
!latex \end{array} \label{eq:coordinateinterpolationannulus} \ee
!latex {\em except} in the innermost volume, where regularization factors must be included:
!latex \be \begin{array}{ccccccc} R_j(s) &=& R_{j,1} \; s^{m_j/2} \; / \; s_1^{m_j/2},\\
!latex                            Z_j(s) &=& Z_{j,1} \; s^{m_j/2} \; / \; s_1^{m_j/2}.
!latex \end{array} \ee
   
   xx = lss                  - interfacelabel(lvol-1)
   ds = interfacelabel(lvol) - interfacelabel(lvol-1)
   
   Rmn(0,0,imn) = iRbc(lvol-1,imn) + ( iRbc(lvol,imn) - iRbc(lvol-1,imn) ) * xx / ds 
   Zmn(0,0,imn) = iZbs(lvol-1,imn) + ( iZbs(lvol,imn) - iZbs(lvol-1,imn) ) * xx / ds 
   
   if( Lcurv.ge.1 ) then
    Rmn(0,1,imn) =                   ( iRbc(lvol,imn) - iRbc(lvol-1,imn) )      / ds
    Zmn(0,1,imn) =                   ( iZbs(lvol,imn) - iZbs(lvol-1,imn) )      / ds 
   endif
   
   if( Lcurv.ge.2 ) then
    Rmn(0,2,imn) =                     zero                                 
    Zmn(0,2,imn) =                     zero                                 
   endif
   
!  if( lvol.eq.1 .and. ( Lgeometry.eq.2 .or. Lgeometry.eq.3 .or. Lgeometry.eq.5 .or. abs(Lgeometry).eq.6 ) ) then ! 10 Oct 12;
   if( Lcoordinatesingularity ) then  ! 10 Oct 12;
    
    select case( Lgeometry )
    case(   2 ) ; hmm = half + im(imn) * half
    case(   3 ) ; hmm = half + im(imn) * half
    case(   5 ) ; hmm =        im(imn) * half
!   case(-6,6 ) ; hmm =        im(imn) * half ! 10 Oct 12;
    case(   6 ) ; hmm =        im(imn) * half ! 10 Oct 12;
    case default
     FATALMESS(co01aa,.true.,invalid Lgeometry : hmm not set) 
    end select
    
    isshm = interfacelabel(1)**hmm
    
    Rmn(0,0,imn) = ( iRbc(1,imn) / isshm ) * lss**(hmm    )
    Zmn(0,0,imn) = ( iZbs(1,imn) / isshm ) * lss**(hmm    )
    
    if( Lcurv.ge.1 ) then
     FATALMESS(co01aa,lss.lt.vsmall .and. hmm-one.lt.zero,error in dx1**dx2 where dx1=0 and dx2 < 0)
     Rmn(0,1,imn) = ( iRbc(1,imn) / isshm ) * lss**(hmm-one) * hmm
     Zmn(0,1,imn) = ( iZbs(1,imn) / isshm ) * lss**(hmm-one) * hmm
    endif
    
    if( Lcurv.ge.2 ) then
     FATALMESS(co01aa,lss.lt.vsmall .and. hmm-two.lt.zero,error in dx1**dx2 where dx1=0 and dx2 < 0)
     Rmn(0,2,imn) = ( iRbc(1,imn) / isshm ) * lss**(hmm-two) * hmm * (hmm-one)
     Zmn(0,2,imn) = ( iZbs(1,imn) / isshm ) * lss**(hmm-two) * hmm * (hmm-one)
    endif
    
   endif ! end of if( Lcoordinatesingularity );
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
  enddo ! end of do imn = 1, mn; the interpolated Fourier harmonics are now constructed; now perform fft to reconstruct geometry and derivatives;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!#ifdef DEBUG ! 23 Oct 12;
!  write(ounit,'("co01aa : " 10x " : Rbc=",99es13.5)')Rbc(0,0,1:mn) ! 23 Oct 12;
!  write(ounit,'("co01aa : " 10x " : Zbs=",99es13.5)')Zbs(0,0,1:mn) ! 23 Oct 12;
!#endif ! 23 Oct 12;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \end{enumerate} \subsubsection{coordinate functions} \begin{enumerate}

!latex \item If \verb+Lcurv.le.0+, then the following are constructed:
!latex \begin{enumerate}
!latex \item the coordinate functions,
!latex
!latex $R \equiv $\verb+Rij(0,:)+
!latex
!latex $Z \equiv $\verb+Zij(0,:)+
!latex \end{enumerate}
  
  ofmn(1:mn)=zero ; cfmn(1:mn)=zero
  call invfft( mn, im, in, Rmn(0,0,:), ofmn(1:mn), cfmn(1:mn), Zmn(0,0,:), Nt, Nz, Rij(0,:), Zij(0,:), isr, trigm, trign, trigwk ) ! maps to real space;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  if( Lcurv.eq.0 ) goto 9999 ! only the coordinates are required;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!latex \end{enumerate} \subsubsection{Jacobian and metrics} \begin{enumerate}

!latex \item if \verb+Lcurv.ge.1+, then the following are constructed:
!latex \begin{enumerate}
!latex \item the coordinate derivatives, 
!latex
!latex $R_\s\equiv$\verb+Rij(1,:)+
!latex 
!latex $R_\t\equiv$\verb+Rij(2,:)+
!latex 
!latex $R_\z\equiv$\verb+Rij(3,:)+
!latex \item the Jacobian, 
!latex
!latex $\sqrt g\equiv$\verb+sg(0,:)+
!latex \item the `lower' metric elements, 
!latex 
!latex $g_{\s\s}\equiv$\verb+gij(1,0,:)+
!latex 
!latex $g_{\s\t}\equiv$\verb+gij(2,0,:)+
!latex 
!latex $g_{\s\z}\equiv$\verb+gij(3,0,:)+
!latex 
!latex $g_{\t\t}\equiv$\verb+gij(4,0,:)+
!latex 
!latex $g_{\t\z}\equiv$\verb+gij(5,0,:)+
!latex 
!latex $g_{\z\z}\equiv$\verb+gij(6,0,:)+
!latex
!latex \end{enumerate}
  
  ofmn(1:mn) = zero ; cfmn(1:mn) = zero
  call invfft( mn, im, in, Rmn(0,1,:),     ofmn(1:mn),     cfmn(1:mn), Zmn(0,1,:), Nt, Nz, Rij(1,:), Zij(1,:), isr, trigm, trign, trigwk ) ! map to real space;
  
  efmn(1:mn) = zero ; sfmn(1:mn) = zero
  call invfft( mn, im, in, efmn(1:mn), -im*Rmn(0,0,:), +im*Zmn(0,0,:), sfmn(1:mn), Nt, Nz, Rij(2,:), Zij(2,:), isr, trigm, trign, trigwk ) ! map to real space;
  
  efmn(1:mn) = zero ; sfmn(1:mn) = zero
  call invfft( mn, im, in, efmn(1:mn), +in*Rmn(0,0,:), -in*Zmn(0,0,1:mn), sfmn(:), Nt, Nz, Rij(3,:), Zij(3,:), isr, trigm, trign, trigwk ) ! map to real space;
  
!latex \item The Jacobian and metrics depend on the interpretation of the geometry, which is controlled by \verb+Lgeometry+ as follows:
!latex \begin{enumerate}

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  select case( Lgeometry ) 
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
!latex \item \verb+Lgeometry.eq.1+: Cartesian geometry:
!latex \begin{itemize}
!latex \item Position is given 
!latex \be {\bf x} = \t \; \hat {\bf i} + \z \; \hat {\bf j}+ R(\s,\t,\z) \; \hat {\bf k}.\ee
!latex \item The Jacobian is \be \sqrt g = R_s.\ee
!latex \end{itemize}
   
  case( 1 ) ! Cartesian;
   
   sg(0,:) =            Rij(1,:)
   
   gij(1,0,:) = Rij(1,:) * Rij(1,:)
   gij(2,0,:) = Rij(1,:) * Rij(2,:)
   gij(3,0,:) = Rij(1,:) * Rij(3,:)
   gij(4,0,:) = Rij(2,:) * Rij(2,:) + one
   gij(5,0,:) = Rij(2,:) * Rij(3,:)
   gij(6,0,:) = Rij(3,:) * Rij(3,:) + one

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
!latex \item \verb+Lgeometry.eq.2+: Cylindrical geometry:
!latex \begin{itemize}
!latex \item Position is given 
!latex \be {\bf x} = R(\s,\t,\z) \cos\t \; {\bf i} + R(\s,\t,\z)\sin\t \; {\bf j} + \z \; {\bf k},\ee
!latex \item The Jacobian is \be \sqrt g = R R_s. \ee
!latex \item The `lower' metric coefficients, $g_{ij}$, are given by
!latex \be g_{\s\s} &=& R_{\s}R_{\s}       \\
!latex     g_{\s\t} &=& R_{\s}R_{\t}       \\
!latex     g_{\s\z} &=& R_{\s}R_{\z}       \\
!latex     g_{\t\t} &=& R_{\t}R_{\t} + R R \\
!latex     g_{\t\z} &=& R_{\t}R_{\z}       \\
!latex     g_{\z\z} &=& R_{\z}R_{\z} + 1
!latex \ee
!latex \end{itemize}

  case( 2 ) ! cylindrical; have not yet scaled radial coordinate correctly near the origin;
   
   sg(0,:) = Rij(0,:) * Rij(1,:)
   
   gij(1,0,:) = Rij(1,:) * Rij(1,:)
   gij(2,0,:) = Rij(1,:) * Rij(2,:)
   gij(3,0,:) = Rij(1,:) * Rij(3,:)
   gij(4,0,:) = Rij(2,:) * Rij(2,:) + Rij(0,:)**2
   gij(5,0,:) = Rij(2,:) * Rij(3,:)
   gij(6,0,:) = Rij(3,:) * Rij(3,:) + one
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
  case( 3 ) ! toroidal;     
   
   sg(0,:) = Rij(0,:) * Rij(1,:) * ( iRbc(0,1) + Rij(0,:) * costeta(:) ) 
   
   gij(1,0,:) = Rij(1,:) * Rij(1,:)
   gij(2,0,:) = Rij(1,:) * Rij(2,:)
   gij(3,0,:) = Rij(1,:) * Rij(3,:)
   gij(4,0,:) = Rij(2,:) * Rij(2,:) + Rij(0,:)**2
   gij(5,0,:) = Rij(2,:) * Rij(3,:)
   gij(6,0,:) = Rij(3,:) * Rij(3,:) + ( iRbc(0,1) + Rij(0,:) * costeta(:) )**2
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
!  case( 4 ) ! Cartesian;
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
!latex \item \verb+Lgeometry=5+: Cylindrical geometry:
!latex \begin{itemize}
!latex \item Position is given 
!latex \be {\bf x} = R(\s,\t,\z) \; {\bf i} + Z(\s,\t,\z) \; {\bf j} + \z \; {\bf k},\ee
!latex \item The Jacobian is \be \sqrt g = ( R_s Z_\theta - R_\theta Z_s ). \ee
!latex \end{itemize}

  case( 5 ) ! cylindrical
   
   sg(0,:) =              Rij(1,:)*Zij(2,:) - Rij(2,:)*Zij(1,:)
   
   gij(1,0,:) = Rij(1,:) * Rij(1,:) + Zij(1,:) * Zij(1,:)
   gij(2,0,:) = Rij(1,:) * Rij(2,:) + Zij(1,:) * Zij(2,:)
   gij(3,0,:) = Rij(1,:) * Rij(3,:) + Zij(1,:) * Zij(3,:)
   gij(4,0,:) = Rij(2,:) * Rij(2,:) + Zij(2,:) * Zij(2,:)
   gij(5,0,:) = Rij(2,:) * Rij(3,:) + Zij(2,:) * Zij(3,:)
   gij(6,0,:) = Rij(3,:) * Rij(3,:) + Zij(3,:) * Zij(3,:) + one
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item \verb+Lgeometry.eq.6+ Toroidal geometry:
!latex \begin{itemize}
!latex \item Position is given \be {\bf x} = R(\s,\t,\z) \hat R + Z(\s,\t,\z) \hat Z,\ee
!latex where $\hat R \equiv \cos \zeta \; {\bf i} + \sin \zeta \; {\bf j}$ and $\zeta \equiv \phi$.
!latex \item The Jacobian is \be \sqrt g = R ( R_s Z_\theta - R_\theta Z_s ). \ee
!latex \item The `lower' metric coefficients, $g_{ij}$, are given by
!latex \be g_{\s\s} &=& R_{\s}R_{\s} + Z_{\s}Z_{\s},     \\
!latex     g_{\s\t} &=& R_{\s}R_{\t} + Z_{\s}Z_{\t},     \\
!latex     g_{\s\z} &=& R_{\s}R_{\z} + Z_{\s}Z_{\z},     \\
!latex     g_{\t\t} &=& R_{\t}R_{\t} + Z_{\t}Z_{\t},     \\
!latex     g_{\t\z} &=& R_{\t}R_{\z} + Z_{\t}Z_{\z},     \\
!latex     g_{\z\z} &=& R_{\z}R_{\z} + Z_{\z}Z_{\z} + R^2.
!latex \ee
!latex \end{itemize}

  case( 6 ) ! toroidal;
   
   sg(0,:) = Itoroidal * Rij(0,:) * ( Rij(2,:)*Zij(1,:) - Rij(1,:)*Zij(2,:) )
   
   gij(1,0,:) = Rij(1,:) * Rij(1,:) + Zij(1,:) * Zij(1,:)
   gij(2,0,:) = Rij(1,:) * Rij(2,:) + Zij(1,:) * Zij(2,:)
   gij(3,0,:) = Rij(1,:) * Rij(3,:) + Zij(1,:) * Zij(3,:)
   gij(4,0,:) = Rij(2,:) * Rij(2,:) + Zij(2,:) * Zij(2,:)
   gij(5,0,:) = Rij(2,:) * Rij(3,:) + Zij(2,:) * Zij(3,:)
   gij(6,0,:) = Rij(3,:) * Rij(3,:) + Zij(3,:) * Zij(3,:) + Rij(0,:)**2
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
  case default
   
   FATALMESS(co01aa,.true.,invalid Lgeometry)
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
  end select
  
!latex \end{enumerate}
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  ijreal(1:Ntz) =       sg(0,1:Ntz) ! THIS IS NOWHERE USED;
  ijimag(1:Ntz) = one / sg(0,1:Ntz) ! THIS IS NOWHERE USED;
  
  call tfft( Nt,Nz,ijreal(1:Ntz),ijimag(1:Ntz),isr,trigm,trign,trigwk,mn,im(1:mn),in(1:mn),efmn(1:mn),ofmn(1:mn),sgmn(1:mn),sfmn(1:mn),ifail ) ! THIS IS NOWHERE USED;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  if( Lcurv.le.1 ) goto 9999
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!latex \end{enumerate} \subsubsection{curvatures} \begin{enumerate}

!latex \item if \verb+Lcurv.eq.2+, then the following are constructed:
!latex \begin{enumerate}
!latex \item the coordinate function second derivatives, 
!latex
!latex $R_{\s\s}\equiv$\verb+Rij(4,:)+
!latex
!latex $R_{\s\t}\equiv$\verb+Rij(5,:)+
!latex
!latex $R_{\s\z}\equiv$\verb+Rij(6,:)+
!latex
!latex $R_{\t\t}\equiv$\verb+Rij(7,:)+
!latex
!latex $R_{\t\z}\equiv$\verb+Rij(8,:)+
!latex
!latex $R_{\z\z}\equiv$\verb+Rij(9,:)+
!latex 
!latex \item the Jacobian derivatives, 
!latex
!latex $\partial_\s \sqrt g\equiv$\verb+sg(1,:)+
!latex
!latex $\partial_\t \sqrt g\equiv$\verb+sg(2,:)+
!latex
!latex $\partial_\z \sqrt g\equiv$\verb+sg(3,:)+
!latex \item the `lower' metric element derivatives,
!latex 
!latex $\partial_\s g_{\s\s}\equiv$\verb+gij(1,1,:)+
!latex 
!latex $\partial_\t g_{\s\s}\equiv$\verb+gij(1,2,:)+
!latex 
!latex $\partial_\z g_{\s\s}\equiv$\verb+gij(1,3,:)+
!latex \end{enumerate}

  if( Lcurv.eq.2 ) then
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
   ofmn(1:mn)=zero ; cfmn(1:mn)=zero
   call invfft( mn, im, in,        Rmn(0,2,:),   ofmn(1:mn),   cfmn(1:mn),        Zmn(0,2,:), Nt, Nz, Rij(4,:), Zij(4,:), isr, trigm, trign, trigwk )
   
   efmn(1:mn)=zero ; sfmn(1:mn)=zero
   call invfft( mn, im, in,      efmn(1:mn), -im*Rmn(0,1,:), +im*Zmn(0,1,:),      sfmn(1:mn), Nt, Nz, Rij(5,:), Zij(5,:), isr, trigm, trign, trigwk )
   
   efmn(1:mn)=zero ; sfmn(1:mn)=zero
   call invfft( mn, im, in,      efmn(1:mn), +in*Rmn(0,1,:), -in*Zmn(0,1,:),      sfmn(1:mn), Nt, Nz, Rij(6,:), Zij(6,:), isr, trigm, trign, trigwk )
   
   ofmn(1:mn)=zero ; cfmn(1:mn)=zero
   call invfft( mn, im, in, -im*im*Rmn(0,0,:),   ofmn(1:mn),   cfmn(1:mn), -im*im*Zmn(0,0,:), Nt, Nz, Rij(7,:), Zij(7,:), isr, trigm, trign, trigwk )
   
   ofmn(1:mn)=zero ; cfmn(1:mn)=zero
   call invfft( mn, im, in,  im*in*Rmn(0,0,:),   ofmn(1:mn),   cfmn(1:mn),  im*in*Zmn(0,0,:), Nt, Nz, Rij(8,:), Zij(8,:), isr, trigm, trign, trigwk )
   
   ofmn(1:mn)=zero ; cfmn(1:mn)=zero
   call invfft( mn, im, in, -in*in*Rmn(0,0,:),   ofmn(1:mn),   cfmn(1:mn), -in*in*Zmn(0,0,:), Nt, Nz, Rij(9,:), Zij(9,:), isr, trigm, trign, trigwk )
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
   select case( Lgeometry )
    
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
    
!  case(   1) ! Cartesian;
    
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
    
   case(   2) ! cylindrical;
    
    sg(1,:) = Rij(1,:) * Rij(1,:) + Rij(0,:) * Rij(4,:)
    sg(2,:) = Rij(2,:) * Rij(1,:) + Rij(0,:) * Rij(5,:)
    sg(3,:) = Rij(3,:) * Rij(1,:) + Rij(0,:) * Rij(6,:)
    
    gij(1,1,:) = Rij(4,:) * Rij(1,:) + Rij(1,:) * Rij(4,:)
    gij(1,2,:) = Rij(5,:) * Rij(1,:) + Rij(1,:) * Rij(5,:)
    gij(1,3,:) = Rij(6,:) * Rij(1,:) + Rij(1,:) * Rij(6,:)
    
    gij(2,1,:) = Rij(4,:) * Rij(2,:) + Rij(1,:) * Rij(5,:)
    gij(2,2,:) = Rij(5,:) * Rij(2,:) + Rij(1,:) * Rij(7,:)
    gij(2,3,:) = Rij(6,:) * Rij(2,:) + Rij(1,:) * Rij(8,:)
    
    gij(3,1,:) = Rij(4,:) * Rij(3,:) + Rij(1,:) * Rij(6,:)
    gij(3,2,:) = Rij(5,:) * Rij(3,:) + Rij(1,:) * Rij(8,:)
    gij(3,3,:) = Rij(6,:) * Rij(3,:) + Rij(1,:) * Rij(9,:)
    
    gij(4,1,:) = Rij(5,:) * Rij(2,:) + Rij(2,:) * Rij(5,:) + Rij(1,:) * Rij(0,:) + Rij(0,:) * Rij(1,:) 
    gij(4,2,:) = Rij(7,:) * Rij(2,:) + Rij(2,:) * Rij(7,:) + Rij(2,:) * Rij(0,:) + Rij(0,:) * Rij(2,:) 
    gij(4,3,:) = Rij(8,:) * Rij(2,:) + Rij(2,:) * Rij(8,:) + Rij(3,:) * Rij(0,:) + Rij(0,:) * Rij(3,:) 
    
    gij(5,1,:) = Rij(5,:) * Rij(3,:) + Rij(2,:) * Rij(6,:)
    gij(5,2,:) = Rij(7,:) * Rij(3,:) + Rij(2,:) * Rij(8,:)
    gij(5,3,:) = Rij(8,:) * Rij(3,:) + Rij(2,:) * Rij(9,:)
    
    gij(6,1,:) = Rij(6,:) * Rij(3,:) + Rij(3,:) * Rij(6,:)
    gij(6,2,:) = Rij(8,:) * Rij(3,:) + Rij(3,:) * Rij(8,:)
    gij(6,3,:) = Rij(9,:) * Rij(3,:) + Rij(3,:) * Rij(9,:)
    
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
    
!  case(   3) ! toroidal;
    
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
    
!  case(   4) ! Cartesian;
    
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
    
   case(   5) ! cylindrical;
    
    sg(1,:) =                                                                   ( Rij(4,:)*Zij(2,:) - Rij(5,:)*Zij(1,:) + Rij(1,:)*Zij(5,:) - Rij(2,:)*Zij(4,:) )
    sg(2,:) =                                                                   ( Rij(5,:)*Zij(2,:) - Rij(7,:)*Zij(1,:) + Rij(1,:)*Zij(7,:) - Rij(2,:)*Zij(5,:) )  
    sg(3,:) =                                                                   ( Rij(6,:)*Zij(2,:) - Rij(8,:)*Zij(1,:) + Rij(1,:)*Zij(8,:) - Rij(2,:)*Zij(6,:) )  
    
    do ii = 1, 3
     do ij = ii, 3 ; call gi00aa(ii,ij,ig)
      
      do ik = 1, 3 ; call gi00aa(ii,ik,ie) ; call gi00aa(ij,ik,if)
       
       gij(ig,ik,:) = Rij(3+ie,:)*Rij(ij,:) + Rij(ii,:)*Rij(3+if,:) + Zij(3+ie,:)*Zij(ij,:) + Zij(ii,:)*Zij(3+if,:)
       
      enddo
      
     enddo
    enddo
    
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
    
!  case(-6,6) ! toroidal; ! 10 Oct 12;
   case(   6) ! toroidal; ! 10 Oct 12;

    sg(1,:) = Rij(1,:) * ( Rij(2,:)*Zij(1,:) - Rij(1,:)*Zij(2,:) ) + Rij(0,:) * ( Rij(5,:)*Zij(1,:) - Rij(4,:)*Zij(2,:) + Rij(2,:)*Zij(4,:) - Rij(1,:)*Zij(5,:) )
    sg(2,:) = Rij(2,:) * ( Rij(2,:)*Zij(1,:) - Rij(1,:)*Zij(2,:) ) + Rij(0,:) * ( Rij(7,:)*Zij(1,:) - Rij(5,:)*Zij(2,:) + Rij(2,:)*Zij(5,:) - Rij(1,:)*Zij(7,:) )
    sg(3,:) = Rij(3,:) * ( Rij(2,:)*Zij(1,:) - Rij(1,:)*Zij(2,:) ) + Rij(0,:) * ( Rij(8,:)*Zij(1,:) - Rij(6,:)*Zij(2,:) + Rij(2,:)*Zij(6,:) - Rij(1,:)*Zij(8,:) )
    
    sg(1:3,1:Ntz) = Itoroidal * sg(1:3,1:Ntz)

    do ii = 1, 3
     do ij = ii, 3 ; call gi00aa(ii,ij,ig)
      
      do ik = 1, 3 ; call gi00aa(ii,ik,ie) ; call gi00aa(ij,ik,if)
       
       gij(ig,ik,:) = Rij(3+ie,:)*Rij(ij,:) + Rij(ii,:)*Rij(3+if,:) + Zij(3+ie,:)*Zij(ij,:) + Zij(ii,:)*Zij(3+if,:)
       
       if( ii.eq.3 .and. ij.eq.3 ) gij(ig,ik,:) = gij(ig,ik,:) + two * Rij(ik,:) * Rij(0,:)
       
      enddo
      
     enddo
    enddo
    
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
    
   case default
    
    FATALMESS(co01aa,.true.,have not yet implemented curvatures for this value of Lgeometry)
    
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
   end select ! end of select case ( Lgeometry ) ;
  
  endif ! end of if( Lcurv.eq.2 ) ; 

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!


!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \end{enumerate} \subsubsection{derivatives of Jacobian and metric elements with respect to interface geometry} \begin{enumerate}
!latex \item if \verb+Lcurv.eq.3+ then:
!latex \item In order to determine how the Beltrami fields change when the interface geometry, described by the $R_{l,j}$ and $Z_{l,j}$ is changed,
!latex       it is required to determine how the metric elements and Jacobian depend on the interface geometry.
!latex \item From \Eqn{coordinateinterpolationannulus}, the derivatives in the annular regions are
!latex \be \begin{array}{cccccccccccccccccccccccc}
!latex     \ds \frac{\partial}{\partial R_{l,j}}R      &=& \ds \frac{(s-s_{l-1})}{\Delta s} &\cos \alpha_j &      &,\\
!latex     \ds \frac{\partial}{\partial R_{l,j}}R_{\s} &=& \ds \frac{   1       }{\Delta s} &\cos \alpha_j &      &,\\
!latex     \ds \frac{\partial}{\partial R_{l,j}}R_{\t} &=& \ds \frac{(s-s_{l-1})}{\Delta s} &\sin \alpha_j &(-m_j)&,\\
!latex     \ds \frac{\partial}{\partial R_{l,j}}R_{\z} &=& \ds \frac{(s-s_{l-1})}{\Delta s} &\sin \alpha_j &(+n_j)&,\\
!latex     \ds \frac{\partial}{\partial Z_{l,j}}Z      &=& \ds \frac{(s-s_{l-1})}{\Delta s} &\sin \alpha_j &      &,\\
!latex     \ds \frac{\partial}{\partial Z_{l,j}}Z_{\s} &=& \ds \frac{   1       }{\Delta s} &\sin \alpha_j &      &,\\
!latex     \ds \frac{\partial}{\partial Z_{l,j}}Z_{\t} &=& \ds \frac{(s-s_{l-1})}{\Delta s} &\cos \alpha_j &(+m_j)&,\\
!latex     \ds \frac{\partial}{\partial Z_{l,j}}Z_{\z} &=& \ds \frac{(s-s_{l-1})}{\Delta s} &\cos \alpha_j &(-n_j)&,
!latex \end{array}\ee
!latex \be \begin{array}{cccccccccccccccccccccccc}
!latex     \ds \frac{\partial}{\partial R_{l-1,j}}R      &=& \ds \frac{(s_{l}-s)}{\Delta s} &\cos \alpha_j &      &,\\
!latex     \ds \frac{\partial}{\partial R_{l-1,j}}R_{\s} &=& \ds \frac{   -1    }{\Delta s} &\cos \alpha_j &      &,\\
!latex     \ds \frac{\partial}{\partial R_{l-1,j}}R_{\t} &=& \ds \frac{(s_{l}-s)}{\Delta s} &\sin \alpha_j &(-m_j)&,\\
!latex     \ds \frac{\partial}{\partial R_{l-1,j}}R_{\z} &=& \ds \frac{(s_{l}-s)}{\Delta s} &\sin \alpha_j &(+n_j)&,\\
!latex     \ds \frac{\partial}{\partial Z_{l-1,j}}Z      &=& \ds \frac{(s_{l}-s)}{\Delta s} &\sin \alpha_j &      &,\\
!latex     \ds \frac{\partial}{\partial Z_{l-1,j}}Z_{\s} &=& \ds \frac{   -1    }{\Delta s} &\sin \alpha_j &      &,\\
!latex     \ds \frac{\partial}{\partial Z_{l-1,j}}Z_{\t} &=& \ds \frac{(s_{l}-s)}{\Delta s} &\cos \alpha_j &(+m_j)&,\\
!latex     \ds \frac{\partial}{\partial Z_{l-1,j}}Z_{\z} &=& \ds \frac{(s_{l}-s)}{\Delta s} &\cos \alpha_j &(-n_j)&,
!latex \end{array}\ee
!latex where $\Delta s \equiv s_{l}-s_{l-1}$.
!latex \item Similar expressions may be written for the innermost simple torus where the regularization factors must be included.

  if( Lcurv.eq.3 .or. Lcurv.eq.4 ) then ! will return derivatives of metric elements with respect to interface geometry;
   
   ij = dAd%imn ! identifies Fourier harmonic; shorthand;
   
   ixx = xx ; lNi =  1
   oxx = xx

   select case( Lgeometry )
    
   case( 1 ) ! Cartesian;
    
    if( dAd%innout.eq.0 .and. dAd%irz.eq.0 ) then ! perturb inner interface Rj;
     dXdXj(0,:) =           ( ds - ixx ) * cosj(ij,:)       / ds
     dXdXj(1,:) =                -         cosj(ij,:) * lNi / ds
     dXdXj(2,:) = -im(ij) * ( ds - ixx ) * sinj(ij,:)       / ds
     dXdXj(3,:) =  in(ij) * ( ds - ixx ) * sinj(ij,:)       / ds
    endif
    
    if( dAd%innout.eq.1 .and. dAd%irz.eq.0 ) then ! perturb outer interface Rj;     
     dXdXj(0,:) =                  oxx   * cosj(ij,:)       / ds
     dXdXj(1,:) =                          cosj(ij,:) * lNi / ds
     dXdXj(2,:) = -im(ij) *        oxx   * sinj(ij,:)       / ds
     dXdXj(3,:) =  in(ij) *        oxx   * sinj(ij,:)       / ds
    endif
   
    dxgij(1,:) = dXdXj(1,:) * Rij(1,:) + Rij(1,:) * dXdXj(1,:)
    dxgij(2,:) = dXdXj(1,:) * Rij(2,:) + Rij(1,:) * dXdXj(2,:)
    dxgij(3,:) = dXdXj(1,:) * Rij(3,:) + Rij(1,:) * dXdXj(3,:)
    dxgij(4,:) = dXdXj(2,:) * Rij(2,:) + Rij(2,:) * dXdXj(2,:)
    dxgij(5,:) = dXdXj(2,:) * Rij(3,:) + Rij(2,:) * dXdXj(3,:)
    dxgij(6,:) = dXdXj(3,:) * Rij(3,:) + Rij(3,:) * dXdXj(3,:)
    
    dxsg(:) = dXdXj(1,:) ! will need this for derivatives in fc02aa;
    
   case( 6 ) ! Toroidal;
    
    if( lvol.eq.1 ) then ! need to include regularity factor on coordinates;
     
     hmm = im(ij) * half ; isshm = interfacelabel(1)**hmm
     
     if( dAd%innout.eq.0 .and. dAd%irz.eq.0 ) then ! perturb inner interface Rj;
      dXdXj(0,:) =  zero
      dXdXj(1,:) =  zero
      dXdXj(2,:) =  zero
      dXdXj(3,:) =  zero
     endif
     
     if( dAd%innout.eq.0 .and. dAd%irz.eq.1 ) then ! perturb inner interface Zj;
      dXdXj(0,:) =  zero
      dXdXj(1,:) =  zero
      dXdXj(2,:) =  zero
      dXdXj(3,:) =  zero
     endif
     
     if( dAd%innout.eq.1 .and. dAd%irz.eq.0 ) then ! perturb outer interface Rj;
      dXdXj(0,:) =           lss**(hmm    )       * cosj(ij,:) / isshm
      dXdXj(1,:) =           lss**(hmm-one) * hmm * cosj(ij,:) / isshm
      dXdXj(2,:) = -im(ij) * lss**(hmm    )       * sinj(ij,:) / isshm
      dXdXj(3,:) =  in(ij) * lss**(hmm    )       * sinj(ij,:) / isshm
     endif
     
     if( dAd%innout.eq.1 .and. dAd%irz.eq.1 ) then ! perturb outer interface Zj;
      dXdXj(0,:) =           lss**(hmm    )       * sinj(ij,:) / isshm
      dXdXj(1,:) =           lss**(hmm-one) * hmm * sinj(ij,:) / isshm
      dXdXj(2,:) =  im(ij) * lss**(hmm    )       * cosj(ij,:) / isshm
      dXdXj(3,:) = -in(ij) * lss**(hmm    )       * cosj(ij,:) / isshm
     endif
     
    else ! matches if( lvol.eq.1 ) then ;
     
     if( dAd%innout.eq.0 .and. dAd%irz.eq.0 ) then ! perturb inner interface Rj;
      dXdXj(0,:) =           ( ds - ixx )  * cosj(ij,:) / ds
      dXdXj(1,:) =                -          cosj(ij,:) / ds
      dXdXj(2,:) = -im(ij) * ( ds - ixx )  * sinj(ij,:) / ds
      dXdXj(3,:) =  in(ij) * ( ds - ixx )  * sinj(ij,:) / ds
     endif
     
     if( dAd%innout.eq.0 .and. dAd%irz.eq.1 ) then ! perturb inner interface Zj;
      dXdXj(0,:) =            ( ds - ixx ) * sinj(ij,:) / ds
      dXdXj(1,:) =                 -         sinj(ij,:) / ds
      dXdXj(2,:) =  im(ij) *  ( ds - ixx ) * cosj(ij,:) / ds
      dXdXj(3,:) = -in(ij) *  ( ds - ixx ) * cosj(ij,:) / ds
     endif
     
     if( dAd%innout.eq.1 .and. dAd%irz.eq.0 ) then ! perturb outer interface Rj;
      dXdXj(0,:) =                  oxx    * cosj(ij,:) / ds
      dXdXj(1,:) =                           cosj(ij,:) / ds
      dXdXj(2,:) = -im(ij) *        oxx    * sinj(ij,:) / ds
      dXdXj(3,:) =  in(ij) *        oxx    * sinj(ij,:) / ds
     endif

     if( dAd%innout.eq.1 .and. dAd%irz.eq.1 ) then ! perturb outer interface Zj;
      dXdXj(0,:) =                   oxx   * sinj(ij,:) / ds
      dXdXj(1,:) =                           sinj(ij,:) / ds
      dXdXj(2,:) =  im(ij) *         oxx   * cosj(ij,:) / ds
      dXdXj(3,:) = -in(ij) *         oxx   * cosj(ij,:) / ds
     endif
     
    endif ! end of if( lvol.eq.1 ) ;
   
    if( dAd%irz.eq.0 ) then
     
     dxgij(1,:) = dXdXj(1,:) * Rij(1,:) + Rij(1,:) * dXdXj(1,:)
     dxgij(2,:) = dXdXj(1,:) * Rij(2,:) + Rij(1,:) * dXdXj(2,:)
     dxgij(3,:) = dXdXj(1,:) * Rij(3,:) + Rij(1,:) * dXdXj(3,:)
     dxgij(4,:) = dXdXj(2,:) * Rij(2,:) + Rij(2,:) * dXdXj(2,:)
     dxgij(5,:) = dXdXj(2,:) * Rij(3,:) + Rij(2,:) * dXdXj(3,:)
     dxgij(6,:) = dXdXj(3,:) * Rij(3,:) + Rij(3,:) * dXdXj(3,:) + two * Rij(0,:) * dXdXj(0,:)

     dxsg(:) = Itoroidal * ( dXdXj(0,:) * ( Rij(2,:) * Zij(1,:) - Rij(1,:) * Zij(2,:) ) + Rij(0,:) * ( dXdXj(2,:) * Zij(1,:) - dXdXj(1,:) * Zij(2,:) ) )
     
    endif
   
    if( dAd%irz.eq.1 ) then
     
     dxgij(1,:) = dXdXj(1,:) * Zij(1,:) + Zij(1,:) * dXdXj(1,:)
     dxgij(2,:) = dXdXj(1,:) * Zij(2,:) + Zij(1,:) * dXdXj(2,:)
     dxgij(3,:) = dXdXj(1,:) * Zij(3,:) + Zij(1,:) * dXdXj(3,:)
     dxgij(4,:) = dXdXj(2,:) * Zij(2,:) + Zij(2,:) * dXdXj(2,:)
     dxgij(5,:) = dXdXj(2,:) * Zij(3,:) + Zij(2,:) * dXdXj(3,:)
     dxgij(6,:) = dXdXj(3,:) * Zij(3,:) + Zij(3,:) * dXdXj(3,:)

     dxsg(:) = Itoroidal * (                                                              Rij(0,:) * ( dXdXj(1,:) * Rij(2,:) - dXdXj(2,:) * Rij(1,:) ) )
     
    endif

   case default
    
    FATALMESS(co01aa,.true.,have not yet implemented metric coordinate derivatives for this value of Lgeometry)
    
   end select ! end of select case( Lgeometry ) ;
   
   if( Lcurv.eq.3 ) then ! normalize for me00ab;
    do igij = 1, 6 ; dxgij(igij,:) = ( dxgij(igij,:) - gij(igij,0,:) * dxsg(:) / sg(0,:) ) / sg(0,:)
    enddo ! end of do igij; 
   endif ! end of if( Lcurv.eq.3 ) ;

  endif ! end of if( Lcurv.eq.3 .or. Lcurv.eq.4 ) ;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  RETURN(co01aa)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
end subroutine co01aa

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!l!tex \subsubheading{boundary layer} ! 10 Oct 12;
!l!tex \item It is not required to differentiate all the metric cooefficients: how the Beltrami fields vary when the geometry is varied is essentially a boundary problem.
!l!tex \item Near the inner boundary, the coordinates can be assumed to vary according to
!l!tex \be R = R_{l-1} + \frac{(s-s_{l-1})}{(s_{l,1}-s_{l-1})}(R_{l,1}-R_{l-1}),
!l!tex \ee
!l!tex where $R_{l,1}$ is considered fixed, and we derive
!l!tex \be
!l!tex \frac{\partial R}{\partial R_{l-1}} = \frac{\Delta s - N(s-s_{l,N-1})}{\Delta s}.
!l!tex \ee
!l!tex \item Near the outer boundary, the coordinates can be assumed to vary according to
!l!tex \be R = R_{l,N-1} + \frac{(s-s_{l,N-1})}{(s_{l}-s_{l,N-1})}(R_{l}-R_{l,N-1}),
!l!tex \ee
!l!tex where $R_{l,N-1}$ is considered fixed, and we derive
!l!tex \be
!l!tex \frac{\partial R}{\partial R_{l}} = \frac{N(s-s_{l,N-1})}{\Delta s}.
!l!tex \ee

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
