!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Given vector position, returns force and derivative matrix.

!latex \item The force vector, ${\bf F}({\bf x})$, is a combination of the pressure-imbalance Fourier harmonics, $[[p+B^2/2]]_{m,n}$,
!latex       and the spectral condensation constraints, $\partial M/\partial \lambda_j$.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine fc02aa( Ngdof, position, force, Lderiv )
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  use constants, only : zero, half, one, two, pi2, pi
  
  use numerical, only : 
  
  use fileunits, only : ounit
  
  use inputlist, only : Wfc02aa, ext, Nvol, Nofe, Lgeometry, gamma, adiabatic, pscale, mu, Itoroidal, Energy, Lvacuum
  
  use cputiming, only : Tfc02aa
  
  use allglobal, only : ncpu, myid, cpus, pi2nfp,               &
                        Lcoordinatesingularity,                 &
                        Louterboundarynormal,                   &
                        Iquad,                                  & ! convenience; provided to ma00aa as argument to avoid allocations;
                        iRbc, iZbs,                             & ! Fourier harmonics of geometry; vector of independent variables, position, is "unpacked" into iRbc,iZbs;
                        NBdof,                                  & ! shorthand;
                        mn, im, in, Ntz, ijreal, ijimag,        &
                        efmn, ofmn, cfmn, sfmn,                 & ! FFT workspace;
                        trigm, trign, trigwk, isr, Nt, Nz,      & ! FFT workspace;
                        cosj, sinj,                             & ! FFT workspace;
                        interfacelabel,                         &
                        dAd,                                    &
                        iMB, MB, MR, duMB, duMR, dxMB, dxMR,    & ! Beltrami matrix and right-hand-side; derivatives of Beltrami matrix wrt interface Fourier harmonics;
                        mpnq,                                   & 
                        Bomn, Iomn,                             &
                        DenergyDrz, DanalyticDrz,               & ! 
                                    DanalyDrzDrz,               & ! 
                        DforcebalDrz, DspectralDrz,             &
                        LforcebalDrz,                           &
                        dpflux,                                 & ! poloidal flux; needed to re-constitute magnetic vector potential;
                        lvolume, lBBintegral, lABintegral,      &
                        dvolume, dBBintegral, dABintegral,      &
                        doAtmn, doAzmn,                         & ! interpolated vector potential; constructed in sa00aa;
                          sg,   gij, Rij, Zij,                  & ! Jacobian and metrics; computed in co01aa;
                        dxsg, dxgij,                            & ! derivatives of Jacobian and metrics; computed in co01aa;
                        LbeltramiNewton, LbeltramiSeQuad,       &
                        nflux, Lmp00ac
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  LOCALS
  
  INTEGER, intent(in)  :: Ngdof               ! dimensions;
  REAL,    intent(in)  :: position(1:Ngdof)   ! degrees-of-freedom = internal geometry;
  REAL,    intent(out) :: force(1:Ngdof)      ! force;
  LOGICAL, intent(in)  :: Lderiv              ! indicates whether derivatives are to be calculated;

! LOGICAL              :: Lcompare = .false. ! for comparing Cholesky decomposition approach to direct solver;
! REAL   , allocatable :: workvectora(:) ! workspace for F04ASF;
! REAL   , allocatable :: workvectorb(:) ! workspace for F04ASF;
  
  INTEGER              :: imn, mmi, nni, jmn, mmj, nnj, kmn, mmk, nnk, idof, jdof

  INTEGER              :: lvol, innout, irz, jrz, iocons, NN, Ldo, Lcurv, IB, IBB, ifail, minant, IX, IR, IA, IC

  INTEGER              :: if04asf ! construct solution using dense matrix solver; assumes positive definite matrix;
  INTEGER              :: if03aef ! construct Cholesky decomposition of matrix;
  
  LOGICAL              :: LCholeskyOK
  REAL                 :: errorvalue = 9.9E+09

  REAL                 :: lss, lmu, dpf, cpul, deter, lfactor

 !REAL   , allocatable :: matrix(:,:) ! workspace arrays;
  REAL   , allocatable :: vector(:,:) ! workspace arrays;

  REAL   , allocatable :: osolution(:) ! original ( unperturbed ) solution;
  REAL   , allocatable :: psolution(:) !              perturbed   solution;

  REAL   , allocatable :: dAt(:,:), dAz(:,:), Bsqd(:), dBsqd(:), Roij(:), Zoij(:), II(:), dIdR(:,:), dIdZ(:,:)

 !REAL   , allocatable :: cholesky(:,:), reciprocals(:)
  REAL   , allocatable :: reciprocals(:)

  CHARACTER            :: packorunpack
  
  BEGIN(fc02aa)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
 !FATALMESS(fc02aa, LbeltramiSeQuad .or. LbeltramiNewton, needs attention )

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!latex \item The vector, ${\bf x}$, represents the geometrical degrees of freedom of the internal interfaces.
!latex       This vector is `unpacked' and the \verb+iRbc+, \verb+iZbs+ arrays are assigned.

  packorunpack = 'U' ; WCALL(fc02aa,gf00aa,( Ngdof, position(1:Ngdof), Nvol, mn, iRbc(1:Nvol-1,1:mn), iZbs(1:Nvol-1,1:mn), packorunpack ))
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  RALLOCATE(dAt,(-1:0,1:Ntz)) ! these don't depend on Beltrami resolution; 
  RALLOCATE(dAz,(-1:0,1:Ntz))

  RALLOCATE( Bsqd,(1:Ntz))
  RALLOCATE(dBsqd,(1:Ntz))

  RALLOCATE(Roij,(1:Ntz)) ! NOT USED:
  RALLOCATE(Zoij,(1:Ntz)) ! NOT USED;

  RALLOCATE(II,(1:Ntz)) ! spectral constraints;

  RALLOCATE(dIdR,(1:mn,1:Ntz)) ! derivatives of spectral constraints;
  RALLOCATE(dIdZ,(1:mn,1:Ntz)) ! derivatives of spectral constraints;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  do lvol = 1, Nvol ! loop over volumes;
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

   if( myid.ne.modulo(lvol-1,ncpu) ) cycle ! perform in parallel;
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

   if( Lgeometry.eq.1 .or. Lgeometry.eq.4 .or. lvol.gt.1 ) then ; Lcoordinatesingularity = .false. ! either Cartesian geometry or annular volume; used elsewhere;
   else                                                         ; Lcoordinatesingularity = .true.
   endif

   if( Lvacuum.ne.0 .and. lvol.eq.Nvol                   ) then ; Louterboundarynormal   = .true.
   else                                                         ; Louterboundarynormal   = .false.
   endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
   NN = NBdof(lvol) ! shorthand;
   
!  LALLOCATE( iMB,(    0:NN,0:NN)) ! this is just book-keeping; simple way to record non-zero elements of Beltrami linear matrix for sparse methods;   
   RALLOCATE(  MB,(    0:NN,0:NN)) ! matrix          of linear system;
   RALLOCATE(  MR,(    0:NN     )) ! right hand side of linear system;
!  RALLOCATE(duMB,(1:2,0:NN,0:NN)) ! matrix          of linear system; derivative wrt mu & dpflux;
!  RALLOCATE(duMR,(1:2,0:NN     )) ! right hand side of linear system; derivative wrt mu & dpflux;   
   if( Lderiv) then
   RALLOCATE(dxMB,(    0:NN,0:NN)) ! matrix          of linear system; dense representation; only required if Lderiv;
   RALLOCATE(dxMR,(         0:NN)) ! right hand side of linear system;                     ; only required if Lderiv;
   endif
  !RALLOCATE(matrix,(1:NN,1:NN))
   RALLOCATE(vector,(1:NN,1: 1))

   RALLOCATE(osolution,(1:NN)) ! this will store original  Beltrami field;
   RALLOCATE(psolution,(1:NN)) ! this will store perturbed Beltrami field;

!  if( Lcompare ) then 
!  RALLOCATE(workvectora,(1:NN))
!  RALLOCATE(workvectorb,(1:NN))
!  endif

  !RALLOCATE(cholesky,(1:NN,1:NN))
   RALLOCATE(reciprocals,(1:NN))

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

   dAd%L = .true. ; dAd%innout = -1 ; dAd%irz = -1 ; dAd%imn = -1 ! will allocate, but not compute, derivatives;

   WCALL(fc02aa,ma00ab,( 'A', lvol )) ! allocate volume integrated metric elements;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

   dAd%L = .false. ; dAd%innout = -1 ; dAd%irz = -1 ; dAd%imn = -1

   WCALL(fc02aa,ma00aa,( Iquad, lvol )) ! compute volume integrals of metric elements;

   WCALL(fc02aa,ma02aa,( lvol, NN )) ! calls mp00ac to compute the Beltrami fields (perhaps iteratively to satisfy transform/helicity constraints);
      
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
!   if( Lcompare ) then ! construct unperturbed solution using standard solver; used to confirm sparse method is accurate;
!    
!    cpul = MPI_WTIME()
!    
!    matrix(1:NN,1:NN) = MB(1:NN,1:NN) ; vector(1:NN,1) = MR(1:NN) ! last Beltrami matrix and r.h.s. should be consistent with present value of \mu;
!    
!    if04asf = 1 ; IA = NN
!    call F04ASF( matrix(1:NN,1:NN), IA, vector(1:NN,1), NN, osolution(1:NN), workvectora(1:NN), workvectorb(1:NN), if04asf ) ! uses iterative refinement;
!    
!    cput = MPI_WTIME()
!    select case( if04asf ) !                                                             12345678901234567
!    case(  0  )  ;               write(ounit,0800)cput-cpus,myid,lvol,if04asf,cput-cpul,"success ;        "
!    case(  1  )  ;               write(ounit,0800)cput-cpus,myid,lvol,if04asf,cput-cpul,"not + definite ; "
!    case(  2  )  ;               write(ounit,0800)cput-cpus,myid,lvol,if04asf,cput-cpul,"ill conditioned ;"
!    case(  3  )  ;               write(ounit,0800)cput-cpus,myid,lvol,if04asf,cput-cpul,"input error ;    "
!    case default ;               write(ounit,0800)cput-cpus,myid,lvol,if04asf,cput-cpul,"invalid ifail ;  "
!    end select
!
!    workvectora(1:NN) = osolution(1:NN) ! to compare with "exact" solution constructed above;
!    
!   endif ! end of if( Lcompare ) ;
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
! compute Cholesky decompostion of Beltrami-linear matrix;
   
   LCholeskyOK = .false. 
   
   if( Lderiv ) then ! only need the Cholesky decomposition if multiple (derivative) matrix calculations are required;
    
    cpul = MPI_WTIME()
    
! note that MB is only meaningful if Lbeltramilinear: see ma02aa/mp00ac/ma01ag;

!   cput = MPI_WTIME()
!   write(ounit,'("fc02aa : "f10.2" : myid="i3" ; lvol="i3" ; |MB|="es13.5" ;")')cput-cpus,myid,lvol,sum(abs(MB(1:NN,1:NN)))
    
!   cholesky(1:NN,1:NN) = MB(1:NN,1:NN) ! HEREAFTER MB IS CHOLESKY;
    
    if03aef = 1 ; IA = NN
    call F03AEF( NN, MB(1:NN,1:NN), IA, reciprocals(1:NN), deter, minant, if03aef ) ! Note: matrix is replaced by Cholesky decomposition;
    
    cput = MPI_WTIME() 
    select case( if03aef ) !                                                                          1234567890123456
    case(  0  )  ; if( Wfc02aa ) write(ounit,0810)cput-cpus,myid,lvol,if03aef,deter,minant,cput-cpul,"success ;       "
    case(  1  )  ;               write(ounit,0810)cput-cpus,myid,lvol,if03aef,deter,minant,cput-cpul,"not + definite ;"
    case(  2  )  ;               write(ounit,0810)cput-cpus,myid,lvol,if03aef,deter,minant,cput-cpul,"input error ;   "
    case default ;               write(ounit,0810)cput-cpus,myid,lvol,if03aef,deter,minant,cput-cpul,"invalid ifail ; "
    end select
    
0810 format("fc02aa : "f10.2" : myid="i3" ; lvol="i3" ; Cholesky factorization ; if03aef="i2" ; det="f8.4 " x 2.0^"i9" ; time="f9.6" ; "a16)
    
    if( if03aef.eq.0 ) LCholeskyOK = .true.
   
   endif
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
   if( LCholeskyOK   ) then ! Cholesky decomposition succeeded; construct unperturbed solution using Cholesky decomposition;
    
    IR = 1 ; IA = NN ; IB = NN ; IX = NN
    vector(1:NN,1) = MR(1:NN)
    call F04AGF( NN, IR, MB(1:NN,1:NN), IA, reciprocals(1:NN), vector(1:NN,1:IR), IB, osolution(1:NN), IX ) ! this has already been constructed in mp00ac;
   
    Ldo = 0 ; packorunpack = 'U' ; dpf = dpflux(lvol)
    WCALL(fc02aa,up00aa,( packorunpack, lvol, NN, osolution(1:NN), dpf, Ldo )) ! unpacking (original) vector potential; already unpacked by mp00ac;
 
   else ! solution has already been unpacked in ma02aa;
   
    osolution(1:NN) = zero ! backup plan;
    
   endif
   
   cput = MPI_WTIME()
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
! comparing solutions from dense (exact) linear solver with the Cholesky solution;
   
!  if( Lcompare ) write(ounit,0820)cput-cpus,myid,lvol,maxval(abs(workvectora(1:NN)-osolution(1:NN))),cput-cpul
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
   WCALL(fc02aa,vo00aa,( lvol ))      ! computes volume integral               ; returns lvolume, etc through global;

   Ldo = 0
   WCALL(fc02aa,fu00aa,( lvol, Ldo )) ! computes volume integral of B.B and A.B; returns lBBintegral, etc. through global; requires solution in unpacked format;

!  if( .not.LCholeskyOK ) lBBintegral(lvol) = errorvalue

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
   do innout = 0, 1 ! construct field magnitude on inner and outer interfaces; inside do lvol;
    
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
    
    if( lvol.eq.1    .and. innout.eq.0 ) cycle ! fixed inner boundary / coordinate axis;
    if( lvol.eq.Nvol .and. innout.eq.1 ) cycle ! fixed outer boundary                  ;
    
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
    
    lss = interfacelabel(lvol-1+innout) ! local radial coordinate;
    
    Lcurv = 1
    WCALL(fc02aa,co01aa,( lvol, lss, Lcurv, Ntz )) ! get coordinates and derivatives wrt Rj, Zj, at specific radial location;
    
    Ldo = 0
    WCALL(fc02aa,sa00aa,( lvol, lss, Ldo )) ! get (unperturbed) vector potential and derivatives at specific radial location; requires solution in unpacked format;
    
    ofmn(1:mn) = zero ; sfmn(1:mn) = zero
    call invfft( mn, im, in, doAtmn( 0,1,1:mn), ofmn(1:mn), doAzmn( 0,1,1:mn), sfmn(1:mn), Nt, Nz, dAt( 0,1:Ntz), dAz( 0,1:Ntz), isr, trigm, trign, trigwk )
    
!   if( .not.LCholeskyOK ) corrupted;
    
    Bsqd(:) = ( dAz(0,:)*dAz(0,:)*gij(4,0,:) - two * dAz(0,:)*dAt(0,:)*gij(5,0,:) + dAt(0,:)*dAt(0,:)*gij(6,0,:) ) / sg(0,:)**2
    
    ijreal(1:Ntz) = adiabatic(lvol) * pscale / lvolume(lvol)**gamma + half * Bsqd(1:Ntz) ; ijimag(1:Ntz) = zero ! force; includes the pressure term;
    
!   if( Lgeometry.le.3 ) then ;                                  II(1:Ntz) = zero                                     !           angle constraints not required;
!   else                      ; WCALL(fc02aa,sw02aa,( Ntz, lvol, II(1:Ntz), mn, dIdR(1:mn,1:Ntz), dIdZ(1:mn,1:Ntz) )) ! calculate angle constraints             ;
!   endif
    
!   II(1:Ntz) = zero
   !if( Lgeometry.ge.4 .and. innout.eq.1 ) then ; WCALL(fc02aa,sw02aa,( Ntz, lvol, II(1:Ntz), mn, dIdR(1:mn,1:Ntz), dIdZ(1:mn,1:Ntz) )) ! calculate angle constraints ;
   !else                                        ;                                  II(1:Ntz) = zero
   !endif
!   ijimag(1:Ntz) = II(1:Ntz)
    
    ifail = 0 ; call tfft( Nt, Nz, ijreal(:), ijimag(:), isr, trigm, trign, trigwk, mn, im, in, efmn(1:mn), ofmn(1:mn), cfmn(1:mn), sfmn(1:mn), ifail )
    
    ;                                      Bomn(lvol,innout,1:mn) = efmn(1:mn)             ! this is required for fv00aa;
!   ;                                      Bomn(lvol,innout,1:mn) = efmn(1:mn)*imask(1:mn) ! this is required for fv00aa;
!   if( Lgeometry.ge.4 .and. innout.eq.1 ) Iomn(lvol       ,1:mn) = sfmn(1:mn)*imask(1:mn) ! this is required for fv00aa;
    
     if( Lgeometry.ge.4 .and. innout.eq.1 ) then ! include spectral constraints; includes normalization; computed using slow transform;
      WCALL(fc02aa,sw03aa,( mn, iRbc(lvol,1:mn), iZbs(lvol,1:mn), Iomn(lvol,1:mn) ))
      ;                                                           Iomn(lvol,1:mn) = Iomn(lvol,1:mn)
!     ;                                                           Iomn(lvol,1:mn) = Iomn(lvol,1:mn)*imask(1:mn)
     endif
    
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
    
    if( .not.Lderiv ) then
     
!latex \subsubheading{energy functional gradient}
     
!latex \item Consider 
!latex       \be \delta F = - \int_{\cal I} [[p+B^2/2]] \; \boldxi \cdot {\bf dS} = - \int \!\!\!\!\! \int [[p+B^2/2]] \; \boldxi \cdot {\bf e}_\t \times {\bf e}_\z \; d\t d\z.
!latex       \ee
    
     select case( Lgeometry ) !latex \begin{enumerate}
      
!latex \item \verb+Lgeometry.eq.1+ Cartesian geometry: 
!latex       \be {\bf x} = \t \; \hat {\bf i} + \z \; \hat {\bf j}+ R \; \hat {\bf k}\ee
!latex       \be {\bf e}_\theta \times {\bf e}_\zeta = \hat {\bf i} \times \hat {\bf j} + R_\z \; \hat {\bf i} \times \hat {\bf k} + R_\t \; \hat {\bf k} \times \hat {\bf j}\ee
!latex       \be \delta {\bf x} \cdot {\bf e}_\theta \times {\bf e}_\zeta = \delta R\ee
!latex       \be \frac{\partial F}{\partial R_j} = [[p+B^2/2]]_j\ee
     
!   case( 1 ) 
     
!   lthEnergy(lvol,innout,1   ) = Bdmn(innout,1   ) * pi2 * pi2nfp ! I am not sure why the pi2 * pi2nfp factor is required; need to check construction of \int A.B dv;
!   lthEnergy(lvol,innout,2:mn) = Bdmn(innout,2:mn) * pi  * pi2nfp
     
!latex \item \verb+Lgeometry.eq.2+ cylindrical:
!latex       \be{\bf x} = R \cos\t \, {\bf \hat i} + R \sin\t \, {\bf \hat j} + \z \, {\bf \hat k}\ee
!latex       \be {\bf e}_\theta \times {\bf e}_\zeta = - R Z_\theta \hat R + (Z_\theta R_\zeta - R_\theta Z_\zeta) \hat \phi + R R_\theta \hat z\ee
!latex       \be \delta {\bf x}\cdot {\bf e}_\theta \times {\bf e}_\zeta =  \delta R \; R \ee
!latex       \be \frac{\partial F}{\partial R_j} &=& - \left( [[ p + B^2/2 ]] \; R \right)_j
!latex       \ee
   
! case( 2 ) ! THIS NEEDS CHECKING;
   
!  ijreal(1:Ntz) = - ( adiabatic(lvol) * pscale / lvolume(lvol)**gamma + half * Bsqd(1:Ntz) ) * Rij(0,:)
!  ijimag(1:Ntz) =   zero
   
!  call tfft( Nt,Nz,ijreal(1:Ntz),ijimag(1:Ntz),isr,trigm,trign,trigwk,mn,im(1:mn),in(1:mn),efmn(1:mn),ofmn(1:mn),cfmn(1:mn),sfmn(1:mn),ifail )
   
!   lthEnergy(lvol,innout,   1        ) = efmn(1   ) * pi2 * pi2nfp ! I am not sure why the pi2 * pi2nfp factor is required; need to check construction of \int A.B dv;
!   lthEnergy(lvol,innout,   2:mn     ) = efmn(2:mn) * pi  * pi2nfp
   
!latex \item \verb+Lgeometry.eq.5+ cylindrical:
!latex       \be{\bf x} = R \, {\bf \hat i} + Z \, {\bf \hat j} + \zeta \, {\bf \hat k}\ee
!latex       \be {\bf e}_\theta \times {\bf e}_\zeta = Z_\theta {\bf \hat i} - R_\theta {\bf \hat j} \ee
!latex       \be \delta {\bf x}\cdot {\bf e}_\theta \times {\bf e}_\zeta = +\delta R\; Z_\t - \delta Z\; R_\t\ee
!latex       \be \frac{\partial F}{\partial R_j} &=& + \left( [[ p + B^2/2 ]] \; Z_\t \right)_j\\
!latex           \frac{\partial F}{\partial Z_j} &=& - \left( [[ p + B^2/2 ]] \; R_\t \right)_j.
!latex       \ee
   
! case (5)
   
!  ijreal(1:Ntz) = + ( adiabatic(lvol) * pscale / lvolume(lvol)**gamma + half * Bsqd(1:Ntz) ) * Zij(2,:)
!  ijimag(1:Ntz) = - ( adiabatic(lvol) * pscale / lvolume(lvol)**gamma + half * Bsqd(1:Ntz) ) * Rij(2,:)
     
!  call tfft( Nt,Nz,ijreal(1:Ntz),ijimag(1:Ntz),isr,trigm,trign,trigwk,mn,im(1:mn),in(1:mn),efmn(1:mn),ofmn(1:mn),cfmn(1:mn),sfmn(1:mn),ifail )

!   lthEnergy(lvol,innout,   1        ) = efmn(1   ) * pi2 * pi2nfp ! I am not sure why the pi2 * pi2nfp factor is required; need to check construction of \int A.B dv;
!   lthEnergy(lvol,innout,   2:mn     ) = efmn(2:mn) * pi  * pi2nfp
!   lthEnergy(lvol,innout,mn+1:mn+mn-1) = sfmn(2:mn) * pi  * pi2nfp
   
!latex \item \verb+Lgeometry.eq.6+ toroidal geometry:
!latex       \be{\bf x} = R \, {\bf \hat R} + Z \, {\bf \hat k}\ee
!latex       \be {\bf e}_\theta \times {\bf e}_\zeta = - R \, Z_\theta \, \hat R + (Z_\theta \,R_\zeta - R_\theta \,Z_\zeta) \hat \phi + R \,R_\theta \,\hat z\ee
!latex       \be \delta {\bf x}\cdot {\bf e}_\theta \times {\bf e}_\zeta = - \delta R \; R \; Z_\t + \delta Z \; R \; R_\t\ee
!latex       \be \frac{\partial F}{\partial R_j} &=& - \left( [[ p + B^2/2 ]] \; R \; Z_\t \right)_j\\
!latex           \frac{\partial F}{\partial Z_j} &=& + \left( [[ p + B^2/2 ]] \; R \; R_\t \right)_j.
!latex       \ee
      
     case( 6 )
     
! inside do lvol; inside do innout;

      ijreal(1:Ntz) = ( adiabatic(lvol) * pscale / lvolume(lvol)**gamma + half * Bsqd(1:Ntz) ) * Rij(0,:) * Zij(2,:)
      ijimag(1:Ntz) = ( adiabatic(lvol) * pscale / lvolume(lvol)**gamma + half * Bsqd(1:Ntz) ) * Rij(0,:) * Rij(2,:)
      
      call tfft( Nt,Nz,ijreal(1:Ntz),ijimag(1:Ntz),isr,trigm,trign,trigwk,mn,im(1:mn),in(1:mn),efmn(1:mn),ofmn(1:mn),cfmn(1:mn),sfmn(1:mn),ifail )
      
      idof = 0 ! local to interface;

      do imn = 1, mn
       
       lfactor = pi * pi2nfp * nflux(lvol-1+innout)**(im(imn)*half) * (-1)**(innout-1) * ( -Itoroidal ) ! only depends on lvol, innout, imn
       
       do irz = 0, 1
        
!       if( Lgeometry.le.3 .and. irz.eq.1 ) cycle ! redundant; this is inside select case( Lgeometry ); included for consistency with other routines;
        
        if( imn.eq.1 .and. irz.eq.1 ) cycle ! no dependence on Z_{m,n} \sin( m\t-n\z) for m=0, n=0;
        
        idof = idof + 1       
        
!       if( imask(imn).le.0 ) cycle
        
        if( irz.eq.0 .and. imn.eq.1 ) DanalyticDrz(lvol,innout,idof) = - efmn(imn) * two * lfactor ! R;
        if( irz.eq.0 .and. imn.gt.1 ) DanalyticDrz(lvol,innout,idof) = - efmn(imn)       * lfactor ! R;
!       if( irz.eq.1 .and. imn.eq.1 ) DanalyticDrz(lvol,innout,idof) =   sfmn(imn) * two * lfactor ! Z; this mode is always irrelevant;
        if( irz.eq.1 .and. imn.gt.1 ) DanalyticDrz(lvol,innout,idof) =   sfmn(imn)       * lfactor ! Z;
        
       enddo ! end of do irz;
       
      enddo ! end of do imn;

     case default
      
      FATALMESS(fc02aa,.true.,invalid Lgeometry)
      
     end select !end of select case( Lgeometry ) ; !latex \end{enumerate}
     
    endif ! end of if( .not.Lderiv )
    
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
    
   enddo ! end of do innout = 0, 1;
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

! still inside do lvol = 1, Nvol loop;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
   if( Lderiv ) then

    FATALMESS(fc02aa, LbeltramiSeQuad .or. LbeltramiNewton, I think that the derivatives have not been calculated )
    
    dAd%L = .true. ! will need derivatives;
    
    idof = 0 ! labels degree of freedom; local to interface;
    
    do imn = 1, mn ! loop over deformations in Fourier harmonics;; inside do lvol;
     
     dAd%imn = imn ! controls construction of derivatives in subroutines called below;
     
     do irz = 0, 1 ! loop over deformations in R and Z; inside do lvol; inside do imn;
      
      if( imn.eq.1 .and. irz.eq.1 ) cycle ! no dependence on Z_{m=0,n=0} sin( m \t - n \z);
      
      if( irz.eq.1 .and. Lgeometry.le.3 ) cycle ! no dependence on Z;
      
      dAd%irz = irz ! controls construction of derivatives;
      
      idof = idof + 1
      
!     if( imask(imn).le.0 ) cycle ! this is not a selected degree of freedom;
      
      if( Wfc02aa ) then
       if( LforcebalDrz(lvol,idof) ) then ; write(ounit,1003) myid, lvol, imn, im(imn), in(imn), irz, idof, LforcebalDrz(lvol,idof), " skip  "
       else                               ; write(ounit,1003) myid, lvol, imn, im(imn), in(imn), irz, idof, LforcebalDrz(lvol,idof), "compute"
       endif
      endif
      
1003   format("fc02aa : " 10x " : myid="i3" ; lvol="i3" ; imn="i3" ; im="i3" ; in="i3" ; irz="i2" ; idof="i4" ; LforcebalDrz="L2" ; "a7";")
      
      if( LforcebalDrz(lvol,idof) ) cycle

      do innout = 0, 1 ! loop over deformations to inner and outer interface; inside do lvol; inside do imn; inside do irz;
       
       if( lvol.eq.1    .and. innout.eq.0 ) cycle ! no degrees of freedom at coordinate axis / fixed inner boundary;
       if( lvol.eq.Nvol .and. innout.eq.1 ) cycle ! no degress of freedom                      fixed outer boundary;
       
       dAd%innout = innout
       
       if( LCholeskyOK ) then ! Cholesky decomposition succeeded;
        
        WCALL(fc02aa,ma00aa,( Iquad, lvol )) ! compute volume integrals of metric elements; differentiated;
        
        LALLOCATE( iMB,(    0:NN,0:NN)) ! this is just book-keeping; simple way to record non-zero elements of Beltrami linear matrix for sparse methods;   
        RALLOCATE(duMB,(1:2,0:NN,0:NN)) ! matrix          of linear system;
        RALLOCATE(duMR,(1:2,0:NN     )) ! right hand side of linear system;
        
        lmu = mu(lvol) ; dpf = dpflux(lvol) ! these have been already updated by call to ma02aa/mp00ac;
        
        WCALL(fc02aa,ma01ag,( lvol, mn, Nofe, lmu, dpf )) ! returns Beltrami matrices; differentiated;
        
        DEALLOCATE( iMB)
        DEALLOCATE(duMB)
        DEALLOCATE(duMR)
        
        vector(1:NN,1) = dxMR(1:NN) - matmul( dxMB(1:NN,1:NN), osolution(1:NN) ) ! construct ``perturbed'' right-hand-side;
        
        IR = 1
        call F04AGF( NN, IR, MB(1:NN,1:NN), IA, reciprocals(1:NN), vector(1:NN,1:IR), IB, psolution(1:NN), IX ) ! computed pertubed vector potential;
        
        Ldo = -1 ; packorunpack = 'u' ; dpf = dpflux(lvol) ! TAKE CARE: does perturbed vector potential include the flux?  NO;
        
        WCALL(fc02aa,up00aa,( packorunpack, lvol, NN, psolution(1:NN), dpf, Ldo )) ! unpacking perturbed vector potential; packed into doAt(Ldo,lvol,. . .)
        
       else ! matches if( LCholeskyOK ) then ! Cholesky decomposition succeeded;
        
        psolution(1:NN) = zero ! backup plan; Cholesky decomposition failed;
        
       endif ! end of if( LCholeskyOK );
       
       
       WCALL(fc02aa,vo00aa,( lvol )) ! compute derivative of volume;
       
       Ldo = 0
       WCALL(fc02aa,fu00aa,( lvol, Ldo )) ! compute derivative of integrals; returns dBBintegral etc through global;
       
       if( .not.LCholeskyOK ) dBBintegral = zero
       
       DenergyDrz(lvol,innout,idof) = dBBintegral * nflux(lvol-1+innout)**(im(imn)*half) ! derivative of \int [ p/(gamma-1) + B.B/2] dv wrt geometry;
       
       do iocons = 0, 1 ! loop over inner and outer interface; inside do lvol; inside do imn; inside do irz; inside do innout;
        
        if( Wfc02aa ) write(ounit,'("fc02aa : "10x" : lvol="i2" ; innout="i2" ; irz="i2" ; imn="i3" ; iocons="i2" ; ")')lvol,innout,irz,imn,iocons       
        
        if( lvol.eq.1    .and. iocons.eq.0 ) cycle ! coordinate axis / fixed inner boundary;
        if( lvol.eq.Nvol .and. iocons.eq.1 ) cycle !                   fixed outer boundary;
        
        lss = interfacelabel(lvol-1+iocons)
        
        Lcurv = 4
        WCALL(fc02aa,co01aa,( lvol, lss, Lcurv, Ntz )) ! get coordinates and derivatives wrt Rj, Zj, at specific radial location;
        
        Ldo = 0
        WCALL(fc02aa,sa00aa,( lvol, lss, Ldo )) ! get (unperturbed) vector potential and derivatives at specific radial location;
        
        ofmn(1:mn) = zero ; sfmn(1:mn) = zero
        call invfft( mn, im, in, doAtmn( 0,1,1:mn), ofmn(1:mn), doAzmn( 0,1,1:mn), sfmn(1:mn), Nt, Nz, dAt( 0,1:Ntz), dAz( 0,1:Ntz), isr, trigm, trign, trigwk )
        
        Bsqd(:) = ( dAz(0,:)*dAz(0,:)*gij(4,0,:) - two * dAz(0,:)*dAt(0,:)*gij(5,0,:) + dAt(0,:)*dAt(0,:)*gij(6,0,:) ) / sg(0,:)**2
        
        Ldo = -1
        WCALL(fc02aa,sa00aa,( lvol, lss, Ldo )) ! get perturbed vector potential and derivatives at specific radial location;
        
        ofmn(1:mn) = zero ; sfmn(1:mn) = zero
        call invfft( mn, im, in, doAtmn(-1,1,1:mn), ofmn(1:mn), doAzmn(-1,1,1:mn), sfmn(1:mn), Nt, Nz, dAt(-1,1:Ntz), dAz(-1,1:Ntz), isr, trigm, trign, trigwk )
        
        dBsqd(1:Ntz) = (     ( dAz(-1,:)*dAz( 0,:)*gij(4,0,:) + dAz( 0,:)*dAz(-1,:)*gij(4,0,:) + dAz( 0,:)*dAz( 0,:)*dxgij(4,:) )                &
                     - two * ( dAz(-1,:)*dAt( 0,:)*gij(5,0,:) + dAz( 0,:)*dAt(-1,:)*gij(5,0,:) + dAz( 0,:)*dAt( 0,:)*dxgij(5,:) )                &
                     +       ( dAt(-1,:)*dAt( 0,:)*gij(6,0,:) + dAt( 0,:)*dAt(-1,:)*gij(6,0,:) + dAt( 0,:)*dAt( 0,:)*dxgij(6,:) ) ) / sg(0,:)**2 &
                     - two * Bsqd(:)*dxsg(:) / sg(0,:)
        
        ijreal(1:Ntz) = - adiabatic(lvol) * pscale * gamma * dvolume / lvolume(lvol)**(one+gamma) + half * dBsqd(1:Ntz) ! include pressure term;
        
        ifail = 0 ; ijimag(1:Ntz) = zero
        call tfft( Nt, Nz, ijreal(1:Ntz), ijimag(1:Ntz), isr, trigm, trign, trigwk, mn, im, in, efmn(1:mn), ofmn(1:mn), cfmn(1:mn), sfmn(1:mn), ifail )
        
        if( .not.LCholeskyOK ) efmn(1:mn) = zero
        
        DforcebalDrz(lvol,innout,idof,iocons,1:mn) = efmn(1:mn) * nflux(lvol-1+innout)**(im(imn)*half) ! incorporate psifactor;

#ifdef SECONDDERIVATIVES

! inside do lvol; inside do imn; inside do irz; inside do innout; inside do iocons;
        
        select case( irz )
        case( 0 ) ! compute derivative wrt R;
         if( innout.eq.iocons ) then 
          ijreal(1:Ntz) = ( - adiabatic(lvol) * pscale * gamma * dvolume / lvolume(lvol)**(one+gamma) + half * dBsqd(1:Ntz) ) * Rij(0,:)    * Zij(2,:) &
                        + (   adiabatic(lvol) * pscale                   / lvolume(lvol)**(    gamma) + half *  Bsqd(1:Ntz) ) * cosj(imn,:) * Zij(2,:) 

          ijimag(1:Ntz) = ( - adiabatic(lvol) * pscale * gamma * dvolume / lvolume(lvol)**(one+gamma) + half * dBsqd(1:Ntz) ) * Rij(0,:)    * Rij(2,:) &
                        + (   adiabatic(lvol) * pscale                   / lvolume(lvol)**(    gamma) + half *  Bsqd(1:Ntz) ) * cosj(imn,:) * Rij(2,:) &
                        + (   adiabatic(lvol) * pscale                   / lvolume(lvol)**(    gamma) + half *  Bsqd(1:Ntz) ) * Rij(0,:)    * sinj(imn,:) * ( - im(imn) )
         else
          ijreal(1:Ntz) = ( - adiabatic(lvol) * pscale * gamma * dvolume / lvolume(lvol)**(one+gamma) + half * dBsqd(1:Ntz) ) * Rij(0,:)    * Zij(2,:)

          ijimag(1:Ntz) = ( - adiabatic(lvol) * pscale * gamma * dvolume / lvolume(lvol)**(one+gamma) + half * dBsqd(1:Ntz) ) * Rij(0,:)    * Rij(2,:)
         endif
        case( 1 ) ! compute derivative wrt Z;
         if( innout.eq.iocons ) then
          ijreal(1:Ntz) = ( - adiabatic(lvol) * pscale * gamma * dvolume / lvolume(lvol)**(one+gamma) + half * dBsqd(1:Ntz) ) * Rij(0,:)    * Zij(2,:) &
                        + (   adiabatic(lvol) * pscale                   / lvolume(lvol)**(    gamma) + half *  Bsqd(1:Ntz) ) * Rij(0,:)    * cosj(imn,:) * (   im(imn) )

          ijimag(1:Ntz) = ( - adiabatic(lvol) * pscale * gamma * dvolume / lvolume(lvol)**(one+gamma) + half * dBsqd(1:Ntz) ) * Rij(0,:)    * Rij(2,:)
         else
          ijreal(1:Ntz) = ( - adiabatic(lvol) * pscale * gamma * dvolume / lvolume(lvol)**(one+gamma) + half * dBsqd(1:Ntz) ) * Rij(0,:)    * Zij(2,:)

          ijimag(1:Ntz) = ( - adiabatic(lvol) * pscale * gamma * dvolume / lvolume(lvol)**(one+gamma) + half * dBsqd(1:Ntz) ) * Rij(0,:)    * Rij(2,:)
         endif
        case default
         FATALMESS(fc02aa,.true.,must be R or Z)
        end select
        
        call tfft( Nt,Nz,ijreal(1:Ntz),ijimag(1:Ntz),isr,trigm,trign,trigwk,mn,im(1:mn),in(1:mn),efmn(1:mn),ofmn(1:mn),cfmn(1:mn),sfmn(1:mn),ifail )       
        
        jdof = 0
        do jmn = 1, mn
         lfactor = pi * pi2nfp * nflux(lvol-1+innout)**(im(jmn)*half) * (-1)**(innout-1) * ( -Itoroidal ) !
         do jrz = 0, 1
          if( Lgeometry.le.3 .and. jrz.eq.1 ) cycle ! no dependence on Z;
          if( jmn.eq.1 .and. jrz.eq.1 ) cycle
          jdof = jdof + 1
!         if( imask(jmn).le.0 ) cycle
          if( jrz.eq.0 .and. jmn.eq.1 ) DanalyDrzDrz(lvol,iocons,jdof,innout,idof) = - efmn(jmn) * two * lfactor ! R;
          if( jrz.eq.0 .and. jmn.gt.1 ) DanalyDrzDrz(lvol,iocons,jdof,innout,idof) = - efmn(jmn)       * lfactor ! R;
!         if( jrz.eq.1 .and. jmn.eq.1 ) DanalyDrzDrz(lvol,iocons,jdof,innout,idof) =   sfmn(jmn) * two * lfactor ! Z; this mode is always irrelevant;
          if( jrz.eq.1 .and. jmn.gt.1 ) DanalyDrzDrz(lvol,iocons,jdof,innout,idof) =   sfmn(jmn)       * lfactor ! Z;
         enddo ! end of do jrz;
        enddo ! end of do jmn;

#endif

!latex \item The derivatives of the spectral constraints, $I =R_\t X + Z_\t Y$,  are derived using $\sin(\alpha+\beta)=\left[\sin(\alpha+\beta)+\sin(\alpha-\beta)\right]/2$
!latex       to give
!latex       \be \frac{\partial I}{\partial R_j} &=& \frac{\partial R_\t}{\partial R_j} X + R_\t \frac{\partial X}{\partial R_j} \nonumber \\
!latex                                           &=& \frac{1}{2} \sum_k \left[ - ( m_j \lambda_k + m_k \lambda_j ) R_k \sin (\alpha_j + \alpha_k )
!latex                                                                         - ( m_j \lambda_k - m_k \lambda_j ) R_k \sin (\alpha_j - \alpha_k ) \right]\\
!latex           \frac{\partial I}{\partial Z_j} &=& \frac{\partial Z_\t}{\partial R_j} Y + Z_\t \frac{\partial Y}{\partial R_j} \nonumber \\
!latex                                           &=& \frac{1}{2} \sum_k \left[ + ( m_j \lambda_k + m_k \lambda_j ) Z_k \sin (\alpha_j + \alpha_k )
!latex                                                                         - ( m_j \lambda_k - m_k \lambda_j ) Z_k \sin (\alpha_j - \alpha_k ) \right].
!latex       \ee
        
! this could possibly come out of do iocons loop;
        
        if( Lgeometry.ge.4 .and. innout.eq.1 .and. iocons.eq.1 ) then ! include derivatives of spectral constraints wrt interface Fourier harmonics;
         
         DspectralDrz(lvol,idof,2:mn) = zero ! initialize summation;
         
         mmi = im(imn) ; nni = in(imn) ! associated with derivative degree-of-freedom;
         
         do jmn = 2, mn ; mmj = im(jmn) ; nnj = in(jmn) ! constraint harmonic;
          
          do kmn = 2, mn ; mmk = im(kmn) ; nnk = in(kmn) ! summation loop; THIS SHOULD PROBABLY BE CONSTRUCTED USING FFTs; at least should be consistent with above;
           
           if( irz.eq.0 ) then ! R;
            if( mmi+mmk.eq.mmj .and. nni+nnk.eq.nnj ) DspectralDrz(lvol,idof,jmn) = DspectralDrz(lvol,idof,jmn) - (mmi*mpnq(kmn)+mmk*mpnq(imn)) * iRbc(lvol,kmn)
            if( mmi-mmk.eq.mmj .and. nni-nnk.eq.nnj ) DspectralDrz(lvol,idof,jmn) = DspectralDrz(lvol,idof,jmn) - (mmi*mpnq(kmn)-mmk*mpnq(imn)) * iRbc(lvol,kmn)
            if( mmk-mmi.eq.mmj .and. nnk-nni.eq.nnj ) DspectralDrz(lvol,idof,jmn) = DspectralDrz(lvol,idof,jmn) + (mmi*mpnq(kmn)-mmk*mpnq(imn)) * iRbc(lvol,kmn)
           endif
           
           if( irz.eq.1 )then ! Z;
            if( mmi+mmk.eq.mmj .and. nni+nnk.eq.nnj ) DspectralDrz(lvol,idof,jmn) = DspectralDrz(lvol,idof,jmn) + (mmi*mpnq(kmn)+mmk*mpnq(imn)) * iZbs(lvol,kmn)
            if( mmi-mmk.eq.mmj .and. nni-nnk.eq.nnj ) DspectralDrz(lvol,idof,jmn) = DspectralDrz(lvol,idof,jmn) - (mmi*mpnq(kmn)-mmk*mpnq(imn)) * iZbs(lvol,kmn)
            if( mmk-mmi.eq.mmj .and. nnk-nni.eq.nnj ) DspectralDrz(lvol,idof,jmn) = DspectralDrz(lvol,idof,jmn) + (mmi*mpnq(kmn)-mmk*mpnq(imn)) * iZbs(lvol,kmn)
           endif
           
          enddo ! end of do kmn;
          
         enddo ! end of do jmn;
          
         DspectralDrz(lvol,idof,2:mn) = half * DspectralDrz(lvol,idof,2:mn) * nflux(lvol)**(im(imn)*half)
         
        endif ! end of if( Lgeometry.ge.4 .and. iocons.eq.1 ) ;
        
       enddo ! end of do iocons; inside do lvol; inside do imn; inside do irz; inside do innout;

      enddo ! end of do innout; inside do lvol; inside do imn; inside do irz;
      

      LforcebalDrz(lvol,idof) = .true. ! derivatives have been calculated;
     !write(ounit,'("fc02aa : " 10x " : myid="i3" ; lvol="i3" ; imn="i3" ; (im,in)=("2i3") ; irz="i2" ; idof="i4" ; LforcebalDrz="L2" ;")')&
     !                                  myid       ,lvol       ,imn       , im(imn),in(imn), irz       ,idof       ,LforcebalDrz(lvol,idof)


     enddo ! end of do irz; inside do lvol; inside do imn;
     
    enddo ! end of do imn; inside do lvol;
    
0800 format("fc02aa : "f10.2" : myid="i3" ; lvol="i3" ;    linear solution     ; if04asf="i2"       " 8x  "       "9x" ; time="f9.6" ; "a17)

0820 format("fc02aa : "f10.2" : myid="i3" ; lvol="i3" ; Cholesky  solution     ;         "2x" ; err="es8.0"       "9x" ; time="f9.6" ; ")

   endif ! end of if( Lderiv) ;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

   dAd%L = .true. ; dAd%innout = -1 ; dAd%irz = -1 ; dAd%imn = -1 ! will allocate, but not compute, derivatives;
   
   WCALL(fc02aa,ma00ab,( 'D', lvol )) ! deallocate volume integrated metric elements;
   
!  DEALLOCATE( iMB)
   DEALLOCATE(  MB)
   DEALLOCATE(  MR)
!  DEALLOCATE(duMB)
!  DEALLOCATE(duMR)
   if( Lderiv) then
   DEALLOCATE(dxMB)
   DEALLOCATE(dxMR)
   endif
  !DEALLOCATE(matrix)
   DEALLOCATE(vector)

   DEALLOCATE(osolution) ! original  solution;
   DEALLOCATE(psolution) ! perturbed solution;

!  if( Lcompare) then
!  DEALLOCATE(workvectora)
!  DEALLOCATE(workvectorb)
!  endif

  !DEALLOCATE(cholesky)
   DEALLOCATE(reciprocals)
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  enddo ! end of do lvol = 1, Nvol ;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  DEALLOCATE(dAt)
  DEALLOCATE(dAz)

  DEALLOCATE( Bsqd)
  DEALLOCATE(dBsqd)

  DEALLOCATE(Roij)
  DEALLOCATE(Zoij)

  DEALLOCATE(II)
  DEALLOCATE(dIdR)
  DEALLOCATE(dIdZ)

  dAd%L = .false. ; dAd%innout = -1 ; dAd%irz = -1 ; dAd%imn = -1

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
 
  do lvol = 1, Nvol ! broadcast;
 
  WCALL(fc02aa,bc00aa,( lvol )) ! loop over volumes; ll is global parameter; cannot parallelize this as data is to be broadcast; 

  enddo
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  Energy = sum( lBBintegral(1:Nvol) ) ! should also compute beta;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  WCALL(fc02aa,fv00aa,( Ngdof, force(1:Ngdof), Lderiv )) ! pieces together DforcebalDrz, DspectralDrz to create hessian;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  RETURN(fc02aa)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
end subroutine fc02aa
 
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
