!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Calls a NAG routine (either C05NDF or C05PDF) to find a zero of the force-balance vector.

!latex \subsubheading{construction of force-balance vector}

!latex \item The force-balance vector is a combination of the Fourier harmonics of the pressure imbalance,
!latex \be F_j \equiv [[p+B^2/2]]_j
!latex \ee
!latex and the spectral condensation constraints,
!latex \be F_j \equiv (R_\t X + Z_\t Y)_j.
!latex \ee

!latex \subsubheading{relevant input parameters}

!latex \item \verb+Lfindzero+ : \begin{itemize} \item If \verb+Lfindzero.eq.1+, then \verb+C05NDF+ is used. This routine uses function values only.
!latex                                          \item If \verb+Lfindzero.eq.2+, then \verb+C05PDF+ is used. This routine uses function values and user supplied derivatives.
!latex                          \end{itemize}
!latex \item \verb+forcetol+  : \begin{itemize} \item The accuracy in force-balance to which the solution is required.
!latex                          \end{itemize}
!latex \item \verb+c05xtol+   : \begin{itemize} \item The accuracy in position to which the solution is required. See NAG document for more detail.
!latex                          \end{itemize}
!latex \item \verb+c05factor+ : \begin{itemize} \item A quantity to be used in determining the initial step bound. See NAG document for more detail.
!latex                          \end{itemize}
!latex \item \verb+LreadGF+   : \begin{itemize} \item If \verb+LreadGF.eq.T+, then the derivative matrix will be read from the file \verb+.GF+.
!latex                          \end{itemize}

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine jk03aa( Ngdof, position, ic05pxf )
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  use constants, only : zero, ten

  use numerical, only : sqrtmachprec

  use fileunits, only : ounit, dunit

  use inputlist, only : Wjk03aa, ext, Lgeometry, Nvol,                    &
                        Lfindzero, forcetol, c05xtol, c05factor, LreadGF, &
                        forceerr

  use cputiming, only : Tjk03aa

  use allglobal, only : writin, myid, ncpu, cpus,                            &
                        mn, im, in, iRbc, iZbs,                              &
                        mBomn, mIomn,                                        &
                        nlocaldof, DforcebalDrz, DspectralDrz, LforcebalDrz, &
                        hessian, Lhessianallocated 
!                                                   naisseh,                 &


!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  LOCALS

  INTEGER, intent(in)    :: Ngdof
  REAL   , intent(inout) :: position(1:Ngdof)
  INTEGER, intent(out)   :: ic05pxf

  LOGICAL                :: Lderiv
  INTEGER                :: nFunctionIterations, nDerivativeIterations, idof, jdof, ijdof, wflag, iflag, ireadhessian
  REAL                   :: rflag
  CHARACTER              :: packorunpack

  INTEGER                :: localdof, imn, irz, lvol

  INTEGER                :: irevcm, mode, Ldfjac, LR
  REAL                   :: xtol, epsfcn, factor
  REAL                   :: diag(1:Ngdof), RR(1:Ngdof*(Ngdof+1)/2), QTF(1:Ngdof), workspace(1:Ngdof,1:4)
  
  REAL                   :: force(1:Ngdof), gradient(1:Ngdof), fjac(1:Ngdof,1:Ngdof)

  INTEGER                :: ML, MU ! required for only Lc05ndf;

  LOGICAL                :: Lcheckandexit = .true. ! perhaps this could be made user input;

  BEGIN(jk03aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
  if( myid.eq.0 ) then
   cput = MPI_WTIME()
   write(ounit,'("jk03aa : " 10x " : ")')
   write(ounit,'("jk03aa : "f10.2" : Lfindzero="i2" ; forcetol="es13.5" ; c05xtol="es13.5" ; c05factor="es13.5" ; LreadGF="L2" ; Ngdof="i6" ;")')&
                         cput-cpus,  Lfindzero,       forcetol,           c05xtol,           c05factor,           LreadGF,       Ngdof
   write(ounit,'("jk03aa : " 10x " : ")')
  endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  xtol = c05xtol ! tolerance in position; supplied to NAG;
  
  Ldfjac = Ngdof ; LR = Ngdof * (Ngdof+1) / 2 ! supplied to NAG;
  
  mode = 0 ! if mode=2, multiplicative scale factors need to be provided in diag; if mode=0, scaling factors computed internally; ! supplied to NAG;
  
  factor = c05factor ! used to determine initial step bound; ! supplied to NAG;
    
  select case( Lfindzero )
  case( 1 )    ; ML = Ngdof-1 ; MU = Ngdof-1 ; epsfcn = sqrtmachprec ! only required for C05NDF; ! supplied to NAG;
  case( 2 )    ;
  case default
   FATALMESS(jk03aa,.true.,invalid Lfindzero)
  end select
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  nFunctionIterations = 0 ; nDerivativeIterations= 0 ! counters;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  if( Lcheckandexit ) then
   
   Lderiv= .false.
   WCALL(jk03aa,fc02aa,( Ngdof, position(1:Ngdof), force(1:Ngdof), Lderiv )) ! calculate the force-imbalance;
   
   if( myid.eq.0 ) then
    cput = MPI_WTIME()
    ;                    write(ounit,'("jk03aa : "f10.2" : "2i4" |f|="es12.5" : |B|="999f6.2)')cput-cpus,nFunctionIterations,nDerivativeIterations,forceerr,mBomn(1:Nvol-1)
    if( Lgeometry.ge.4 ) write(ounit,'("jk03aa : " 10x " : "8x "     "  12x " : |I|="999f6.2)')                                                             mIomn(1:Nvol-1)
   endif
   
   if( forceerr.lt.forcetol ) then ; ic05pxf = 0 ; goto 9999 ! force-balance is satisfied;  
   endif
   
  endif
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
  irevcm = 0 ; ic05pxf = 1 ! required for initial entry; herefater unchanged by user;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  RALLOCATE( DforcebalDrz,( 1:Nvol, 0:1, 1:nlocaldof, 0:1, 1:mn )) ! derivatives of BB                   wrt geometry;
  LALLOCATE( LforcebalDrz,( 1:Nvol,      1:nlocaldof            ))
  if( Lgeometry.ge.4 ) then
  RALLOCATE( DspectralDrz,( 1:Nvol,      1:nlocaldof,      2:mn )) ! derivatives of spectral constraints wrt geometry;
  endif
  RALLOCATE(hessian,(1:Ngdof,1:Ngdof))
 !RALLOCATE(naisseh,(1:Ngdof,1:Ngdof))
  Lhessianallocated = .true.
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  do ! reverse communication loop; controlled by irevcm;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
   select case( Lfindzero )
    
   case( 1 ) ! use function values                               to find x st f(x)=0, where x is the geometry of the interfaces, and f is the force;
    
    call C05NDF( irevcm, Ngdof, position(1:Ngdof), force(1:Ngdof),                                                                                &
  xtol, ML, MU, epsfcn, diag(1:Ngdof), mode, factor, fjac(1:Ldfjac,1:Ngdof), Ldfjac, RR(1:LR), LR, QTF(1:Ngdof), workspace(1:Ngdof,1:4), ic05pxf )
    
   case( 2 ) ! use function values and user-supplied derivatives to find x st f(x)=0, where x is the geometry of the interfaces, and f is the force;
    
    call C05PDF( irevcm, Ngdof, position(1:Ngdof), force(1:Ngdof),                                                                                &
                                                     fjac(1:Ldfjac,1:Ngdof), Ldfjac,                                                              &
  xtol                , diag(1:Ngdof), mode, factor                                , RR(1:LR), LR, QTF(1:Ngdof), workspace(1:Ngdof,1:4), ic05pxf )
    
   end select

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

   select case( irevcm )

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
    
   case( 0 ) ! final exit;

    if( myid.eq.0 ) then
     cput = MPI_WTIME()
     select case(ic05pxf)
     case( 0   )  ; write(ounit,'("jk03aa : "f10.2" : success         ; ic05p*f="i2" ; its="i5" ,"i5" ;")') cput-cpus,ic05pxf,nFunctionIterations,nDerivativeIterations
     case( 1   )  ; write(ounit,'("jk03aa : "f10.2" : input error     ; ic05p*f="i2" ; its="i5" ,"i5" ;")') cput-cpus,ic05pxf,nFunctionIterations,nDerivativeIterations
     case( 2   )  ; write(ounit,'("jk03aa : "f10.2" : irevcm error    ; ic05p*f="i2" ; its="i5" ,"i5" ;")') cput-cpus,ic05pxf,nFunctionIterations,nDerivativeIterations
     case( 3   )  ; write(ounit,'("jk03aa : "f10.2" : xtol too small  ; ic05p*f="i2" ; its="i5" ,"i5" ;")') cput-cpus,ic05pxf,nFunctionIterations,nDerivativeIterations
     case( 4:5 )  ; write(ounit,'("jk03aa : "f10.2" : bad progress    ; ic05p*f="i2" ; its="i5" ,"i5" ;")') cput-cpus,ic05pxf,nFunctionIterations,nDerivativeIterations
     case default ; write(ounit,'("jk03aa : "f10.2" : illegal ifail   ; ic05p*f="i2" ; its="i5" ,"i5" ;")') cput-cpus,ic05pxf,nFunctionIterations,nDerivativeIterations
     end select
    endif ! end of if( myid.eq.0 ) then;
    
    exit ! escape from infinite do loop;
    
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
    
   case( 1 ) ! indicates start of new iteration; no action is required; position and force available for printing; force must not be changed;
    
    packorunpack = 'U' ; WCALL(jk03aa,gf00aa,( Ngdof, position(1:Ngdof), Nvol, mn, iRbc(1:Nvol-1,1:mn), iZbs(1:Nvol-1,1:mn), packorunpack ))
    
    if( myid.eq.0 ) then
     
     cput = MPI_WTIME()
     ;                    write(ounit,'("jk03aa : "f10.2" : "2i4" |f|="es12.5" : |B|="999f6.2)')cput-cpus,nFunctionIterations,nDerivativeIterations,forceerr,mBomn(1:Nvol-1)
     if( Lgeometry.ge.4 ) write(ounit,'("jk03aa : " 10x " : "8x "     "  12x " : |I|="999f6.2)')                                                             mIomn(1:Nvol-1)
     
     wflag=2 ; iflag=nDerivativeIterations ; rflag=forceerr
     WCALL(jk03aa,writin,(wflag,iflag,rflag)) ! write restart file; save geometry to ext.end; iRbc, iZbs consistent with position;
     
    endif ! end of if( myid.eq.0 );
    
    if( forceerr.lt.forcetol ) then
     if( myid.eq.0 ) write(ounit,'("jk03aa : " 10x " : user terminated ;         "2x" ; its="i5" ,"i5" ;")') nFunctionIterations,nDerivativeIterations
     exit ! escape from infinite do loop; IT IS NOT CLEAR IF RR and FJAC are valid!!!
    endif
    
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
    
   case( 2 ) ! before re-entry to C05NDF / C05PDF, force must contain the function values;
    
    nFunctionIterations = nFunctionIterations + 1
    
    Lderiv = .false.
    WCALL(jk03aa,fc02aa,( Ngdof, position(1:Ngdof), force(1:Ngdof), Lderiv )) ! calculate the force-imbalance;
    
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
    
   case( 3 ) ! before re-entry to          C05PDF, fjac must contain the derivatives;
    
    nDerivativeIterations = nDerivativeIterations + 1
    
    if( LreadGF .and. nDerivativeIterations.eq.1 ) then ! this is the first iteration; will check to see if derivative matrix alread exists in file .GF;
     
     if( myid.eq.0 ) call writereadgf( 'R', Ngdof, ireadhessian ) ! reads derivatives matrix = hessian from file;
     
     RlBCAST( DforcebalDrz(1:Nvol,0:1,1:nlocaldof,0:1,1:mn),Nvol*2*nlocaldof*2* mn   , 0 )
     LlBCAST( LforcebalDrz(1:Nvol    ,1:nlocaldof         ),Nvol*  nlocaldof         , 0 ) ! this indicates which derivatives are saved in file;
     if( Lgeometry.ge.4 ) then
      RlBCAST( DspectralDrz(1:Nvol,    1:nlocaldof    ,2:mn),Nvol*  nlocaldof  *(mn-1), 0 )
     endif
     
     IlBCAST( ireadhessian, 1, 0 )
     if( ireadhessian.eq.1 ) then ! hessian has been read from file;
      RlBCAST( hessian(1:Ngdof,1:Ngdof),Ngdof*Ngdof,0)
     endif

    else ! matches if( LreadGF .and. nDerivativeIterations.eq.1 ) then;
     
     LforcebalDrz(1:Nvol,1:nlocaldof) = .false. ! this instructs fc02aa to compute all derivatives;
     ireadhessian = 0 ! hessian has not been read from file;

    endif ! end of if( LreadGF .and. nDerivativeIterations.eq.1 ) then;
    
    if( ireadhessian.eq.0 ) then
     
     Lderiv = .true.
     WCALL(jk03aa,fc02aa,( Ngdof, position(1:Ngdof), gradient(1:Ngdof), Lderiv )) ! calculate the force-imbalance; this calls bc00aa to broadcast;
     
    endif
    
    fjac(1:Ngdof,1:Ngdof) = hessian(1:Ngdof,1:Ngdof) ! hessian is passed through global;
    
    if( myid.eq.0 ) call writereadgf( 'W', Ngdof, ireadhessian ) ! will always save derivative matrix;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

   case default
    
    FATALMESS(jk03aa,.true.,illegal irevcm : C05PDF error)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
    
   end select ! end of select case(irevcm);

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
  enddo ! end of infinite reverse-communication do loop;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  if( myid.eq.0 .and. irevcm.eq.0 ) then ! will save derivative matrix for future use;
   hessian(1:Ngdof,1:Ngdof) = zero
   ijdof = 0
   do idof = 1, Ngdof
    do jdof = idof, Ngdof ; ijdof = ijdof + 1 ; hessian(idof,jdof) = RR(ijdof) ! un-pack R matrix;
    enddo
   enddo
   hessian(1:Ngdof,1:Ngdof) = matmul( fjac(1:Ngdof,1:Ngdof), hessian(1:Ngdof,1:Ngdof) ) ! reconstruct derivative matrix = Q R; see NAG document for C05PDF for more details;
   call writereadgf( 'W', Ngdof, ireadhessian ) ! write derivative matrix, hessian, to file;
  endif ! end of if( myid.eq.0 ) then;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  DEALLOCATE( DforcebalDrz)
  DEALLOCATE( LforcebalDrz)
  if( Lgeometry.ge.4 ) then
   DEALLOCATE( DspectralDrz)
  endif
  DEALLOCATE(hessian)
! DEALLOCATE(naisseh)
  Lhessianallocated = .false.
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  RETURN(jk03aa)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
end subroutine jk03aa

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!


!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine writereadgf( readorwrite, Ngdof , ireadhessian )
  
  use constants, only :

  use numerical, only :

  use fileunits, only : ounit, dunit

  use inputlist, only : Wjk03aa, ext, Nvol, Mpol, Ntor, Lgeometry

  use cputiming, only : Tjk03aa

  use allglobal, only : myid, cpus, mn, im, in, nlocaldof, DforcebalDrz, DspectralDrz, LforcebalDrz, hessian, Lhessianallocated
  
  LOCALS
  
  CHARACTER, intent(in) :: readorwrite
  LOGICAL               :: exist
  INTEGER, intent(in)   :: Ngdof
  INTEGER, intent(out)  :: ireadhessian
  
  INTEGER               :: lNvol, lMpol, lNtor, lNgdof
  
  FATALMESS(jk03aa,.not.Lhessianallocated,error)
  
  ireadhessian = 0 ! set default intent out;
  
  select case( readorwrite ) 
   
  case( 'W' ) ! will write derivative matrix to file;
   
   open(dunit,file="."//trim(ext)//".GF",status="replace",form="unformatted",iostat=ios) ! save derivative matrix to file;
   if( ios.ne.0 ) write(ounit,'("jk03aa : "10x  " : error opening (saving) derivative matrix file;")')
   
   if( ios.eq.0 ) then
    
    write(dunit  ,iostat=ios) Nvol, Mpol, Ntor, Ngdof ! enable resolution consistency check;
    if( ios.ne.0 ) write(ounit,'("jk03aa : "10x  " : error writing Nvol, Mpol, Ntor, Ngdof ;")')
    
    write(dunit  ,iostat=ios)DforcebalDrz( 1:Nvol, 0:1, 1:nlocaldof, 0:1, 1:mn ) ! derivatives of BB                   wrt geometry;
    if( ios.ne.0 ) write(ounit,'("jk03aa : "10x  " : error writing DforcebalDrz ;")')
    
    write(dunit  ,iostat=ios)LforcebalDrz( 1:Nvol,      1:nlocaldof            )
    if( ios.ne.0 ) write(ounit,'("jk03aa : "10x  " : error writing LforcebalDrz ;")')
    
    if( Lgeometry.ge.4 ) then
     
     write(dunit  ,iostat=ios)DspectralDrz( 1:Nvol,      1:nlocaldof,      2:mn ) ! derivatives of spectral constraints wrt geometry;
     if( ios.ne.0 ) write(ounit,'("jk03aa : "10x  " : error writing DspectralDrz ;")')
     
    endif ! end of if( Lgeometry.ge.4 ) ;

    write(dunit  ,iostat=ios)hessian( 1:Ngdof,1:Ngdof)

    if( ios.ne.0 ) write(ounit,'("jk03aa : "10x  " : error writing hessian ;")')
    
   endif ! end of if( ios.eq.0 ) ;

   close(dunit,iostat=ios)

   if( ios.ne.0 ) write(ounit,'("jk03aa : "10x  " : error closing (saving) derivative matrix file;")')
   
   return
   
  case( 'R' )
   
   cput = MPI_WTIME()
   
   inquire(file="."//trim(ext)//".GF",exist=exist) ! the derivative matrix;
   
   if( exist ) then
    open(dunit,file="."//trim(ext)//".GF",status="old",form="unformatted",iostat=ios)
   else
    inquire(file=".GF",exist=exist) ! the derivative matrix;
    if( .not.exist ) then ; write(ounit,1000)cput-cpus,myid,"does not exist ;                     " ; goto 9999
    endif
    open(dunit,file=".GF",status="old",form="unformatted",iostat=ios)
   endif

   if( ios.ne.0 ) then ; write(ounit,1000)cput-cpus,myid,"error opening file ;                 " ; goto 9999
   endif
   
   read(dunit,iostat=ios) lNvol, lMpol, lNtor, lNgdof ! resolution consistency check;

   if( ios.ne.0        ) then ; write(ounit,1000)cput-cpus,myid,"error reading Nvol, Mpol, Ntor, Ngdof" ; goto 9998
   endif
   if( lNvol.ne.Nvol   ) then ; write(ounit,1000)cput-cpus,myid,"inconsistent resolution ; Nvol ;     " ; goto 9998
   endif
   if( lMpol.ne.Mpol   ) then ; write(ounit,1000)cput-cpus,myid,"inconsistent resolution ; Mpol ;     " ; goto 9998
   endif
   if( lNtor.ne.Ntor   ) then ; write(ounit,1000)cput-cpus,myid,"inconsistent resolution ; Ntor ;     " ; goto 9998
   endif
   
   read(dunit  ,iostat=ios)DforcebalDrz( 1:Nvol, 0:1, 1:nlocaldof, 0:1, 1:mn ) ! derivatives of BB                   wrt geometry;
   if( ios.ne.0 )        then ; write(ounit,1000)cput-cpus,myid,"error reading DforcebalDrz ;         " ; goto 9998
   endif
   
   read(dunit  ,iostat=ios)LforcebalDrz( 1:Nvol,      1:nlocaldof            )
   if( ios.ne.0 )        then ; write(ounit,1000)cput-cpus,myid,"error reading LforcebalDrz ;         " ; goto 9998
   endif
   
   if( Lgeometry.ge.4 ) then
    read(dunit  ,iostat=ios)DspectralDrz( 1:Nvol,      1:nlocaldof,      2:mn ) ! 
    if( ios.ne.0 )        then ; write(ounit,1000)cput-cpus,myid,"error reading DspectralDrz ;         " ; goto 9998
    endif
    
   endif ! if( Lgeometry.ge.4 );

   if( lNgdof.eq.Ngdof ) then
    
    read(dunit  ,iostat=ios)hessian(1:Ngdof,1:Ngdof)
    if( ios.ne.0 )        then ; write(ounit,1000)cput-cpus,myid,"error reading hessian ;              " ; goto 9998
    endif
    
    ireadhessian = 1
    
   endif ! end of if( lNgdof.eq.Ngdof );
   
   ;                            write(ounit,1000)cput-cpus,myid,"ok ;                                 "
   
9998 close(dunit,iostat=ios)
   
  case default
   
   FATALMESS(jk03aa,.true.,invalid readorwrite)
   
  end select
  
9999 return
  
1000 format("jk03aa : "f10.2" : myid="i3" ; .GF file (contains derivative matrix) ; "a37)

end subroutine writereadgf

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
