!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Adjusts helicity-multipler and poloidal flux to solve zero, one or two constraints.

!latex \end{enumerate} \subsubsection{rotational transform constraint} \begin{enumerate}

!latex \item In each annulus, considering the enclosed toroidal flux, $\delta \psi_t$, to be given, there are two degrees of freedom in the Beltrami fields
!latex       that are not constrained by the gauge or the interface condition ${\bf B}\cdot\nabla s=0$.
!latex \item These two degrees of freedom are the helicity multiplier, $\mu$, and the enclosed poloidal flux, $\delta \psi_p$.
!latex \item In each annulus there are two constraints, namely that the rotational transform match prescribed values.
!latex \item This routine adjusts ${\bf x}\equiv(\mu,\delta \psi_p)$ so that ${\bf f}({\bf x})\equiv (\iotabar_{inn}-\iotabar_{l-1}, \iotabar_{out}-\iotabar_{l})=0$,
!latex       where $\iotabar_{inn}$and $\iotabar_{out}$ are the ``measured'' rotational-transforms on the inner and outer interfaces,
!latex       and $\iotabar_l$ are the rotational-transforms prescribed on input, \mbox{$\iotabar_l\equiv(p_l+\gamma p_r)/(q_l+\gamma q_r)$}, where $\gamma$ is the golden mean.
!latex \item The ``measured'' rotational-transform on the interfaces is determined, given the Beltrami field for a given $\mu$ and $\delta \psi_p$,
!latex       by performing a transformation to the straight-field line angle on the interfaces.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine ma02aa( lvol, NBdof )
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  use constants, only : zero, half, one, two, ten, goldmean

  use numerical, only : sqrtmachprec, small

  use fileunits, only : ounit

  use inputlist, only : Wma02aa, Nvol, Lgeometry, Lconstraint, mu, mupftol, mupfits, helicity, Ni, iota, Lfindzero, Lcheck

  use cputiming

  use allglobal, only : myid, cpus, mn, im, in,                            &
                        iMB, MB, MR, duMB, duMR,                           &
                        dpflux,                                            &
                        doAt, doAz,                                        &
                        LbeltramiLinear, LbeltramiNewton, LbeltramiSeQuad, &
                        ivol,                                              &
                        mns, diota, dsflangle,                             &
                        xoffset,                                           &
                        Lcoordinatesingularity, Louterboundarynormal

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  LOCALS

  INTEGER, intent(in)  :: lvol, NBdof

  INTEGER              :: iflag, Nmudp, nfev, irevcm, ii, Ldo, imaxloc(1), idof, imn, ip, iilow
  REAL                 :: tol, multiplier, lmu
  REAL                 :: Xmudp(1:2), Fmudp(1:2)
  REAL                 :: Xmuaa(0:NBdof), Fmuaa(0:NBdof)
  CHARACTER            :: packorunpack

! required for finite element check on derivative calculation; only used if Lcheck.eq.1;
  INTEGER              :: imu, idp, jfinite
  REAL                 :: lfdiff, dFmudp(-1:1,-1:1,1:2)

! required for C05NDF; required for Linear method (and old Newton method);
  INTEGER              :: ic05ndf, Ldfjac, mldiag, mudiag, mode, Lrr, pNBdof, nprint, maxfev, njev, ic05pcf, MLow, MUpp, ic05ncf
  REAL                 :: epsfcn, factor
  REAL                 :: diag(1:2), Dmudp(1:2,1:2), rr(1:2*(2+1)/2), qtf(1:2), wk(1:2,1:4)
  REAL                 :: Bdiag(0:NBdof), Bfjac(0:NBdof,0:NBdof), Brr(0:(NBdof+1)*(NBdof+1+1)/2-1), Bqtf(0:NBdof), Bwk(0:NBdof,1:4)
  external             :: mp00ac

!required for C05PBF;
  INTEGER              :: ic05pbf, Ldfmuaa, lengthwork
  REAL                 :: DFmuaa(0:NBdof,0:NBdof), work(1:(1+NBdof)*(1+NBdof+13)/2), error
  external             :: df00aa, dfcalc ! subroutine that provides volume integrals and derivatives;

! required for E04UFF;
  INTEGER              :: NCLIN, NCNLN, LDA, LDCJ, LDR, iterations, LIWk, LRWk, ie04uff
  INTEGER, allocatable :: Istate(:), NEEDC(:), IWk(:)
  REAL                 :: OBJF
  REAL   , allocatable :: Alc(:,:), BL(:), BU(:), Con(:), dCon(:,:), Clambda(:), OBJGRD(:), RS(:,:), RWk(:)
  CHARACTER            :: optionalparameter*33

  LOGICAL              :: Lold = .false.

  BEGIN(ma02aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  ivol = lvol ! the subroutines mp00ac/df00aa, which are called below, requires volume identification, but the argument list is fixed by NAG;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  pNBdof = NBdof + 1 ; Ldfmuaa = pNBdof

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  if( LbeltramiSeQuad .or. LbeltramiNewton ) then
   WCALL(ma02aa,dfalloc,( pNBdof, Xmuaa(0:NBdof), Fmuaa(0:NBdof), DFmuaa(0:NBdof,0:NBdof), Ldfmuaa ))
  endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  if( LbeltramiSeQuad ) then ! sequential quadratic programming (SQP); construct minimum energy with constrained helicity;
   
   NCLIN = 0 ! no linear constraints;
   
   NCNLN = 1 ! single non-linear constraint = conserved helicity;
   
   LDA = max(1,NCLIN)
   
   LDCJ = max(1,NCNLN)
   
   LDR = NBdof
   
   RALLOCATE(Alc,(1:LDA,1:1)) ! linear constraint matrix;
   
   RALLOCATE(BL,(1:NBdof+NCLIN+NCNLN)) ! lower bounds on variables, linear constraints and non-linear constraints;
   RALLOCATE(BU,(1:NBdof+NCLIN+NCNLN)) ! upper bounds on variables, linear constraints and non-linear constraints;
   
   BL(             1 : NBdof             ) = -1.0E+21       !   variable constraints; no constraint;
   BU(             1 : NBdof             ) = +1.0E+21       !
   BL( NBdof+      1 : NBdof+NCLIN       ) = -1.0E+21       !     linear constraints; no constraint;
   BU( NBdof+      1 : NBdof+NCLIN       ) = +1.0E+21       !
   BL( NBdof+NCLIN+1 : NBdof+NCLIN+NCNLN ) = helicity(lvol) ! non-linear constraints; enforce helicity constraint;
   BU( NBdof+NCLIN+1 : NBdof+NCLIN+NCNLN ) = helicity(lvol) !
   
   iterations = 0 ! iteration counter;
   
   IALLOCATE(Istate,(1:NBdof+NCLIN+NCNLN))
   
   RALLOCATE(Con,(1:NCNLN)) ! constraint functions;
   
   RALLOCATE(dCon,(1:LDCJ,1:NBdof)) ! derivatives of constraint functions;
   
   RALLOCATE(Clambda,(1:NBdof+NCLIN+NCNLN)) ! Lagrange multipliers ?;
   
   OBJF = zero ! objective function;
   
   RALLOCATE(OBJGRD,(1:NBdof)) ! derivatives of objective function;
   
   RALLOCATE(RS,(1:LDR,1:NBdof))
   
   Ldo = 0 ; packorunpack = 'P'
  !WCALL(ma02aa,up00aa,( packorunpack, lvol, NBdof, Xmuaa(1:NBdof), dpflux(lvol), Ldo )) ! pack initial guess with previous solution;
   call up00aa( packorunpack, lvol, NBdof, Xmuaa(1:NBdof), dpflux(lvol), Ldo ) ! pack initial guess with previous solution;
   
   IALLOCATE(NEEDC,(1:NCNLN))
   
   LIWk = 3*NBdof + NCLIN + 2*NCNLN ! workspace;
   IALLOCATE(IWk,(1:LIWk))          ! workspace;
   
   LRWk = 2*NBdof**2 + NBdof * NCLIN + 2 * NBdof * NCNLN + 21 * NBdof + 11 * NCLIN + 22 * NCNLN + 1 ! workspace;
   RALLOCATE(RWk,(1:LRWk))                                                                          ! workspace;

   
   irevcm = 0 ; ie04uff = 1 ! reverse communication loop control; ifail error flag;

   
! supply optional parameters to E04UFF;

   call E04UEF('Nolist')                    ! turn of screen output;
   call E04UEF('Print Level = 0')           ! turn of screen output;
   call E04UEF('Derivative Level = 3')      ! assume all derivatives are provided by user;
   call E04UEF('Verify Level = -1')         ! do not verify derivatives using finite-differences; default is Verify Level = 0, which does verify gradients;
   write(optionalparameter,'("Major Iteration Limit = "i9)') 2**2 * max( 50, 3 * ( NBdof + NCLIN ) + 10 * NCNLN ) ! ; write(*,*)optionalparameter
   call E04UEF(optionalparameter)


   do ! reverse communication loop;
    
    call E04UFF( irevcm, &                                                           ! reverse communication control;
                 NBdof, NCLIN, NCNLN, LDA, LDCJ, LDR, &                              ! integers: number of degrees-of-freedom, array dimensions;
                 Alc(1:LDA,1:1), BL(1:NBdof+NCLIN+NCNLN), BU(1:NBdof+NCLIN+NCNLN), & ! linear constraint matrix and lower/upper bounds;
                 iterations, Istate(1:NBdof+NCLIN+NCNLN), &                          ! iteration counter;
                 Con(1:NCNLN), dCon(1:LDCJ,1:NBdof), Clambda(1:NBdof+NCLIN+NCNLN), & ! constraint functions, constraint derivatives, Lagrange multipliers;
                 OBJF, OBJGRD(1:NBdof), &                                            ! objective function and gradient;
                 RS(1:LDR,1:NBdof), &                                                ! 
                 Xmuaa(1:NBdof), &                                                   ! degrees-of-freedom (in vector potential);
                 NEEDC(1:NCNLN), IWk(1:LIWk), LIWk, RWk(1:LRWk), LRWk, ie04uff )        ! workspace, error flag;

    mu(lvol) = two * Clambda( NBdof + NCLIN + NCNLN )    

    Ldo = 0 ; packorunpack = 'U'
   !WCALL(ma02aa,up00aa,( packorunpack, lvol, NBdof, Xmuaa(1:NBdof), dpflux(lvol), Ldo )) ! unpack solution; 
    call up00aa( packorunpack, lvol, NBdof, Xmuaa(1:NBdof), dpflux(lvol), Ldo ) ! unpack solution; 
    
    if( irevcm.eq.0                  ) then ! final exit;
     cput = MPI_WTIME()
     select case(ie04uff)
     case(:-1)    ;               write(ounit,1011)cput-cpus,myid,lvol,ie04uff,iterations,helicity(lvol),mu(lvol),"user enforced termination         "
     case(  0)    ; if( Wma02aa ) write(ounit,1011)cput-cpus,myid,lvol,ie04uff,iterations,helicity(lvol),mu(lvol),"success                           "
     case(  1)    ;               write(ounit,1011)cput-cpus,myid,lvol,ie04uff,iterations,helicity(lvol),mu(lvol),"not converged, but satisfactory   "
     case(  2)    ;               write(ounit,1011)cput-cpus,myid,lvol,ie04uff,iterations,helicity(lvol),mu(lvol),"constraints infeasible (   linear)"
     case(  3)    ;               write(ounit,1011)cput-cpus,myid,lvol,ie04uff,iterations,helicity(lvol),mu(lvol),"constraints infeasible (nonlinear)"
     case(  4)    ;               write(ounit,1011)cput-cpus,myid,lvol,ie04uff,iterations,helicity(lvol),mu(lvol),"major iteration limit reached     "
     case(  6)    ;               write(ounit,1011)cput-cpus,myid,lvol,ie04uff,iterations,helicity(lvol),mu(lvol),"error                             "
     case(  7)    ;               write(ounit,1011)cput-cpus,myid,lvol,ie04uff,iterations,helicity(lvol),mu(lvol),"derivatives appear incorrect      "
     case(  9)    ;               write(ounit,1011)cput-cpus,myid,lvol,ie04uff,iterations,helicity(lvol),mu(lvol),"input error                       "
     case default ; FATALMESS(ma02aa,.true.,illegal value of ie04uff = IFAIL returned by E04UFF)
     end select

1011 format("ma02aa : "f10.2" : myid="i3" ; lvol="i3" ; SQP : ie04uff="i2" ; iter="i5" ; hel="es13.5" ; mu="es13.5" ; "a34" ;")
     
     exit ! finished;

    endif ! end of if( irevcm.eq.0 ) then;
    
    FATALMESS(ma02aa,irevcm.lt.1 .or. irevcm.gt.6,invalid)
    
    iflag = -irevcm ! controls operation of df00aa;

    Xmuaa(0) = zero ! this, generally, contains the helicity multiplier; the helicity multiplier is not required in this case;
    
    if( Lold ) then
     call df00aa( pNBdof, Xmuaa(0:NBdof), Fmuaa(0:NBdof), DFmuaa(0:NBdof,0:NBdof), Ldfmuaa, iflag ) ! compute volume integrals and derivatives;
    else
     call dfcalc( pNBdof, Xmuaa(0:NBdof), Fmuaa(0:NBdof), DFmuaa(0:NBdof,0:NBdof), Ldfmuaa, iflag ) ! compute volume integrals and derivatives;
    endif
    
    if( irevcm.eq.1 .or. irevcm.eq.3 ) OBJF              = Fmuaa( 0       ) 
    if( irevcm.eq.2 .or. irevcm.eq.3 ) OBJGRD( 1:NBdof ) = Fmuaa( 1:NBdof )
    
    if( irevcm.eq.4 .or. irevcm.eq.6 .and. NEEDC(1).gt.0 )  Con( 1         ) = Fmuaa( 0       ) 
    if( irevcm.eq.5 .or. irevcm.eq.6 .and. NEEDC(1).gt.0 ) dCon( 1,1:NBdof ) = Fmuaa( 1:NBdof )
    
   enddo ! end of do ! reverse communication loop;

   DEALLOCATE(RWk)
   DEALLOCATE(IWk)
   DEALLOCATE(NEEDC)
   DEALLOCATE(RS)
   DEALLOCATE(OBJGRD)
   DEALLOCATE(Clambda)
   DEALLOCATE(dCon)
   DEALLOCATE(Con)
   DEALLOCATE(Istate)
   DEALLOCATE(BL)
   DEALLOCATE(BU)
   DEALLOCATE(Alc)

  endif ! end of if( LbeltramiSeQuad ) then;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  if( Lbeltramilinear ) then
   
   LALLOCATE( iMB,(    0:NBdof,0:NBdof)) ! this is just book-keeping; simple way to record non-zero elements of Beltrami linear matrix for sparse methods;   
!  RALLOCATE(  MB,(    0:NBdof,0:NBdof)) ! matrix          of linear system; this is now allocated in fc01aa/fc02aa;
!  RALLOCATE(  MR,(    0:NBdof        )) ! right hand side of linear system; this is now allocated in fc01aa/fc02aa;
   RALLOCATE(duMB,(1:2,0:NBdof,0:NBdof)) ! matrix          of linear system; derivatives wrt mu & pflux; really only required if Ldmupf.eq.2;
   RALLOCATE(duMR,(1:2,0:NBdof        )) ! right hand side of linear system; derivatives wrt mu & pflux; really only required if Ldmupf.eq.2;
   
  endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  if( Lcheck.eq.1 ) then ! perform finite difference check on derivative of rotational-transform wrt helicity multiplier and enclosed poloidal flux;
   
   select case( Lconstraint ) ! need to set Nmudp = degrees of freedom;
   case( 0 )    ;                                                               Nmudp = 1 ! multiplier &  poloidal flux NOT varied                               ; dummy value;
   case( 1 )    ; if( Lcoordinatesingularity .or. Louterboundarynormal ) then ; Nmudp = 1 ! multiplier or poloidal flux IS  varied to match       transform      ;
    ;             else                                                        ; Nmudp = 2 ! multiplier &  poloidal flux ARE varied to match inner/outer transform;
    ;             endif                                                                   !
   case( 2 )    ;                                                               Nmudp = 1 ! multiplier                 IS  varied to match             helicity ;
   case default
    FATALMESS(ma02aa,.true.,the value of Lconstraint is not valid)
   end select

   ldfjac = Nmudp ; dFmudp(-1:1,-1:1,1:2) = zero
   
   ;                          Xmudp(1:2) = xoffset + (/ mu(lvol), dpflux(lvol) /) ! initial guess for degrees of freedom; offset from zero so that relative error is small;
   if( Louterboundarynormal ) Xmudp(1:2) = xoffset + (/ dpflux(lvol), mu(lvol) /) ! only dpflux is required;
   
   iflag = 2 ! derivatives are required;
   
   call mp00ac( Nmudp, Xmudp(1:Nmudp), dFmudp( 0 , 0 ,1:Nmudp), Dmudp(1:ldfjac,1:Nmudp), ldfjac, iflag ) ! compute "exact" derivatives using matrix perturbation theory;
   
   jfinite = 0
   write(ounit,2000) myid, lvol, jfinite, Dmudp(1:ldfjac,1:Nmudp)
   
2000 format("ma02aa : " 10x " :":" myid="i3" : lvol="i3" ; jj="i3" ; DF="4es23.15)
   
   do jfinite = -6,-2,+2
    
    lfdiff = ten**jfinite
    
    do imu = -1, +1 ! centered differences;
     do idp = -1, +1 ! centered differences;
      
      if( Lcoordinatesingularity .and. idp.ne.0 ) cycle ! Beltrami field in volume with coordinate singularity does not depend on enclosed poloidal flux;
      if( Louterboundarynormal   .and. imu.ne.0 ) cycle ! Beltrami field in outermost volume does depend on mu, but constraint mu = zero is understood;
       
      if( imu.ne.0 .and. idp.ne.0 ) cycle ! centered finite differences; off-diagonal is not required;
      if( imu.eq.0 .and. idp.eq.0 ) cycle ! centered finite differences;    center    is not required;
      
      ;                          Xmudp(1:2) = xoffset + (/ mu(lvol), dpflux(lvol) /) + (/ imu, idp /) * lfdiff * half 
      if( Louterboundarynormal ) Xmudp(1:2) = xoffset + (/ dpflux(lvol), mu(lvol) /) + (/ idp, imu /) * lfdiff * half 
       
      iflag = 1 ! derivatives are not required;
      
      call mp00ac( Nmudp, Xmudp(1:Nmudp), dFmudp(imu,idp,1:Nmudp), Dmudp(1:ldfjac,1:Nmudp), ldfjac, iflag ) ! compute function values only;
      
     enddo ! end of do idp;
    enddo ! end of do imu;
     
    ;                            Dmudp(1:Nmudp,1) = ( dFmudp( 1, 0, 1:Nmudp) - dFmudp(-1, 0, 1:Nmudp) ) / lfdiff ! derivative wrt helicity multiplier   ;
    if( Nmudp.eq.2             ) Dmudp(1:Nmudp,2) = ( dFmudp( 0, 1, 1:Nmudp) - dFmudp( 0,-1, 1:Nmudp) ) / lfdiff ! derivative wrt enclosed poloidal flux;
    if( Louterboundarynormal   ) Dmudp(1:Nmudp,1) = ( dFmudp( 0, 1, 1:Nmudp) - dFmudp( 0,-1, 1:Nmudp) ) / lfdiff ! derivative wrt enclosed poloidal flux;

    write(ounit,2000) myid, lvol, jfinite, Dmudp(1:ldfjac,1:Nmudp)
    
   enddo ! end of do jfinite;

   write(ounit,2000)

  endif ! end of if( Lcheck.eq.1 ) ;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  if( LbeltramiLinear ) then ! assume Beltrami field is parameterized by helicity multiplier (and poloidal flux);
   
   ;                          Xmudp(1:2) = xoffset + (/ mu(lvol), dpflux(lvol) /) ! initial guess for degrees of freedom; offset from zero so that relative error is small;
   if( Louterboundarynormal ) Xmudp(1:2) = xoffset + (/ dpflux(lvol), mu(lvol) /) ! only dpflux is required;
   
   select case( Lconstraint ) ! need to set Nmudp = degrees of freedom;
   case( 0 )    ;                                                               Nmudp = 1 ! multiplier &  poloidal flux NOT varied                               ; dummy value;
   case( 1 )    ; if( Lcoordinatesingularity .or. Louterboundarynormal ) then ; Nmudp = 1 ! multiplier OR poloidal flux IS  varied to match       transform      ;
    ;             else                                                        ; Nmudp = 2 ! multiplier &  poloidal flux ARE varied to match inner/outer transform;
    ;             endif                                                                   !
   case( 2 )    ;                                                               Nmudp = 1 ! multiplier                 IS  varied to match             helicity ;
   case default  
    FATALMESS(ma02aa,.true.,the value of Lconstraint is not valid)
   end select
   
!latex \end{enumerate} \subsubsection{logical control / constraints / degrees of freedom} \begin{enumerate}
!latex \item The input variable \verb+Lconstraint+ controls operation as follows:
!latex \begin{itemize}
   
! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! !
   
   if( Lconstraint.eq.0 ) then
    
!latex \item \verb+Lconstraint.eq.0+ : neither $\mu$ nor $\delta \psi_p$ are varied in any volume;
!latex in this case the Betrami field for the given $(\mu,\delta \psi_p)$ is constructed.
    
    iflag = 1 ; ldfjac = Nmudp
    WCALL(ma02aa,mp00ac,( Nmudp, Xmudp(1:Nmudp), Fmudp(1:Nmudp), Dmudp(1:ldfjac,1:Nmudp), ldfjac, iflag )) ! calculate Beltrami field;
    
   else ! matches if( Lconstraint.eq.0 ) then;
    
!latex \item \verb+Lconstraint.eq.1+ : both $\mu$ and $\delta \psi_p$ are varied in the annular volumes to match the inner and outer transform constraints, and
!latex                                 in the simple torus, $\mu$ is varied to match the outer transform constraint;
!latex \item \verb+Lconstraint.eq.2+ : $\mu$ is varied in all volumes to match the helicity constraint; \verb+dpflux+$\equiv\delta \psi_p$ is held constant;
    
!latex \item if \verb+Lconstraint.eq.1+ or \verb+Lconstraint.eq.2+, then the NAG routine \verb+C05NDF+ is employed to find the solution. 
!latex The following parameters are required on input:
    
    tol = mupftol ; Ldfjac = Nmudp ; Lrr = Nmudp * (Nmudp+1)/2 ; mode = 0 ; diag(1:2) = zero ; factor = 1.0e-02
    
    FATALMESS(ma02aa,mupfits.le.0,must give C05PCF/C05NCF a postive integer value for the maximum iterations = mupfits given on input)
    
    maxfev = mupfits ; nprint = 0 ; nfev = 0 ; njev = 0
    
! HOW DOES C05PCF KNOW IF mp00ac FAILED?
 
    ic05pcf = 1
    call C05PCF( mp00ac, Nmudp, Xmudp(1:Nmudp), Fmudp(1:Nmudp), Dmudp(1:Ldfjac,1:Nmudp), Ldfjac, tol, maxfev, diag(1:Nmudp), mode, factor, nprint, &
  nfev, njev, RR(1:LRR), LRR, QTF(1:Nmudp), WK(1:Nmudp,1:4), ic05pcf )
    
    if( Louterboundarynormal ) then
     dpflux(lvol) = Xmudp(1) - xoffset
!    ;   mu(lvol) = Xmudp(2) - xoffset
    else
     ;   mu(lvol) = Xmudp(1) - xoffset
     dpflux(lvol) = Xmudp(2) - xoffset ! if Nmudp.lt.2, then this value should not be changed;
    endif
    
    cput = MPI_WTIME()   !                                                                                                   1     
    select case(ic05pcf) !                                                                                          1234567890123456
    case(    0   )  ; if( Wma02aa ) write(ounit,1020)cput-cpus,myid,lvol,mu(lvol),dpflux(lvol),helicity(lvol),ic05pcf,"success         ",nfev,njev,Fmudp(1:Nmudp)
    case(  :-1   )  ;               write(ounit,1020)cput-cpus,myid,lvol,mu(lvol),dpflux(lvol),helicity(lvol),ic05pcf,"user terminated ",nfev,njev!Fmudp(1:Nmudp)
    case(    1   )  ;               write(ounit,1020)cput-cpus,myid,lvol,mu(lvol),dpflux(lvol),helicity(lvol),ic05pcf,"input error     ",nfev,njev,Fmudp(1:Nmudp)
    case(    2   )  ;               write(ounit,1020)cput-cpus,myid,lvol,mu(lvol),dpflux(lvol),helicity(lvol),ic05pcf,"consider restart",nfev,njev,Fmudp(1:Nmudp)
    case(    3   )  ;               write(ounit,1020)cput-cpus,myid,lvol,mu(lvol),dpflux(lvol),helicity(lvol),ic05pcf,"xtol too small  ",nfev,njev,Fmudp(1:Nmudp)
    case(    4:5 )  ;               write(ounit,1020)cput-cpus,myid,lvol,mu(lvol),dpflux(lvol),helicity(lvol),ic05pcf,"bad progress    ",nfev,njev,Fmudp(1:Nmudp)
    case default    ;               write(ounit,1020)cput-cpus,myid,lvol,mu(lvol),dpflux(lvol),helicity(lvol),ic05pcf,"illegal ifail   ",nfev,njev,Fmudp(1:Nmudp)
    end select
    
1020 format("ma02aa : "f10.2" : myid="i3" ; lvol="i3" ; (mu,dp)=("es23.15" ,"es23.15" ) ; hel="es13.5" ; ic05pcf="i3" ; "a16" ; nfev,njev=("i4" ,"i2" ) ;":" F="2es13.5)

   endif ! end of if( Lconstraint.eq.0 ) ;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \end{itemize}

   DEALLOCATE( iMB)
!  DEALLOCATE(  MB) ! this is now allocated in fc01aa/fc02aa;
!  DEALLOCATE(  MR) ! this is now allocated in fc01aa/fc02aa;
   DEALLOCATE(duMB)
   DEALLOCATE(duMR)
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  endif ! end of if( LbeltramiLinear ) then;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!


!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  if( LbeltramiNewton ) then ! helicity is given by helicity(ivol) used in df00aa; poloidal flux is given by dpflux(lvol), and dpflux(ivol) used in df00aa; 
   
   FATALMESS(ma02aa,Louterboundarynormal,need to revise logic -- helicity multiplier is not a independent paramter but enclosed poloidal flux is)

   Xmuaa(0) = mu(lvol) ! initialize; helicity multiplier;

   Ldo = 0 ; packorunpack = 'P'
  !WCALL(ma02aa,up00aa,( packorunpack, lvol, NBdof, Xmuaa(1:NBdof), dpflux(lvol), Ldo )) ! pack initial guess with previous solution, which may have been calculated above;
   call up00aa( packorunpack, lvol, NBdof, Xmuaa(1:NBdof), dpflux(lvol), Ldo ) ! pack initial guess with previous solution, which may have been calculated above;
   
   tol = mupftol ; lengthwork = pNBdof * (pNBdof+13) / 2
   
   
   select case( Lconstraint )
    
   case( 2 ) ! Lconstraint=2; do not adjust helicity or poloidal flux; see df00aa for construction of function to be zero-ed;
    
    ic05pbf = 1
    
    if( Lold ) then
     call C05PBF( df00aa, pNBdof, Xmuaa(0:NBdof), Fmuaa(0:NBdof), DFmuaa(0:NBdof,0:NBdof), Ldfmuaa, tol, work(1:lengthwork), lengthwork, ic05pbf )
    else
     call C05PBF( dfcalc, pNBdof, Xmuaa(0:NBdof), Fmuaa(0:NBdof), DFmuaa(0:NBdof,0:NBdof), Ldfmuaa, tol, work(1:lengthwork), lengthwork, ic05pbf )
    endif
    
    error = maxval( abs( Fmuaa(0:NBdof) ) )
    
   case( 1 ) ! Lconstraint=1; adjust helicity and poloidal flux to match transform constraints;
    
    if( Lgeometry.eq.1 .or. Lgeometry.eq.4 .or. lvol.gt.1 ) then ; Nmudp = 2 ! annular region; vary multiplier & poloidal flux to match inner & outer transform;
    else                                                         ; Nmudp = 1 ! "simple" torus; vary multiplier                 to match         outer transform;
    endif
    
! I don't really know what happens here if Louterboundarynormal = T;

    Xmudp(1:2) = xoffset + (/ helicity(lvol), dpflux(lvol) /) ! independent degrees of freedom, helicity and enclosed poloidal flux;

    tol = mupftol ; Ldfjac = Nmudp ; Lrr = Nmudp * (Nmudp+1) / 2 ; mldiag = Nmudp-1 ; mudiag = Nmudp-1 ; mode = 0 ; diag(1:2) = zero ; epsfcn = sqrtmachprec ; factor = 1.0e-02
    
    nfev = 0

    irevcm = 0 ; ic05ndf = 1 ! required for initial entry; herefater unchanged by user;
    
    do ! reverse communication loop;
     
     FATALMESS(ma02aa,Lfindzero.eq.1,cannot use C05NDF inside C05NDF loop)
     
     WCALL(ma02aa,C05NDF,( irevcm, Nmudp, Xmudp(1:Nmudp), Fmudp(1:Nmudp), tol, &
   mldiag, mudiag, epsfcn, diag(1:Nmudp), mode, factor, Dmudp(1:Ldfjac,1:Nmudp), Ldfjac, rr(1:Lrr), Lrr, qtf(1:Nmudp), wk(1:Nmudp,1:4), ic05ndf ))
     
     select case( irevcm )
      
     case( 0 ) ! irevcm=0; final exit;
      
      cput = MPI_WTIME()
      select case(ic05ndf)  
      case( 0   )  ; if( Wma02aa ) write(ounit,2021)cput-cpus,myid,lvol,ic05ndf,nfev
      case( 1   )  ;               write(ounit,2021)cput-cpus,myid,lvol,ic05ndf,nfev
      case( 2   )  ;               write(ounit,2021)cput-cpus,myid,lvol,ic05ndf,nfev
      case( 3   )  ;               write(ounit,2021)cput-cpus,myid,lvol,ic05ndf,nfev
      case( 4:5 )  ;               write(ounit,2021)cput-cpus,myid,lvol,ic05ndf,nfev
      case default
       FATALMESS(ma02aa,.true.,illegal ic05ndf on final exit)
      end select
2021  format("ma02aa : "f10.2" : myid="i3" ; lvol="i3" ; Newton : ic05ndf="i2" ; iter="i5" ;":" hel="es13.5" ; mu="es13.5" ; "a34" ;")
      
!     FATALMESS(ma02aa,.true.,mu is not an independent degree of freedom)
!     mu(lvol) = Xmudp(1)-xoffset ; dpflux(lvol) = Xmudp(2)-xoffset 
      
      FATALMESS(ma02aa,ic05ndf.ne.0,failed to satisfy transform constraints)

      exit ! escape from infinite do loop; note that "solution" vector is "unpacked" by mp00ab;
      
     case( 1 ) ! irevcm=1; indicates start of new iteration; no action is required; xtry and fvec available for printing; fvec must not be changed;
      
     case( 2 ) ! irevcm=2; before re-entry to C05NDF / C05PDF, fvec must contain the function values;
      
      nfev = nfev + 1

                       helicity(lvol) = Xmudp(1) - xoffset
      if( Nmudp.eq.2 )   dpflux(lvol) = Xmudp(2) - xoffset
      
      ic05pbf = 1
      
      if( Lold ) then
       call C05PBF( df00aa, pNBdof, Xmuaa(0:NBdof), Fmuaa(0:NBdof), DFmuaa(0:NBdof,0:NBdof), Ldfmuaa, tol, work(1:lengthwork), lengthwork, ic05pbf )
      else
       call C05PBF( dfcalc, pNBdof, Xmuaa(0:NBdof), Fmuaa(0:NBdof), DFmuaa(0:NBdof,0:NBdof), Ldfmuaa, tol, work(1:lengthwork), lengthwork, ic05pbf )
      endif
      
      error = maxval( abs( Fmuaa(0:NBdof) ) )

      mu(lvol) = Xmuaa(0)
      Ldo = 0 ; packorunpack = 'U'
     !WCALL(ma02aa,up00aa,( packorunpack, lvol, NBdof, Xmuaa(1:NBdof), dpflux(lvol), Ldo )) ! unpack solution;
      call up00aa( packorunpack, lvol, NBdof, Xmuaa(1:NBdof), dpflux(lvol), Ldo ) ! unpack solution;

      iflag = 1
      WCALL(ma02aa,tr00ab,(lvol,Nvol,mns,diota(lvol,0:2,0:1),iflag)) ! given vector potential, computes transform on interfaces;
      
      if( Nmudp.eq.1 ) then ; Fmudp(1  ) = diota(lvol,0,  1) - (                iota(lvol)  ) ! provided returned function;
      else                  ; Fmudp(1:2) = diota(lvol,0,0:1) - (/ iota(lvol-1), iota(lvol) /)
      endif

      if( Wma02aa ) then 
       cput = MPI_WTIME()
       write(ounit,'("ma02aa : "f10.2" : myid="i3" ; lvol="i3" ; transform="2es13.5" ;")')cput-cpus,myid,lvol,diota(lvol,0,0:1)
      endif
      
      if( Wma02aa ) then 
       cput = MPI_WTIME()
       write(ounit,'("ma02aa : "f10.2" : myid="i3" ; lvol="i3" ; "i3" ; X="2es13.5" F="2es13.5" ;")')cput-cpus,myid,lvol,nfev,Xmudp(1:2)-xoffset,Fmudp(1:Nmudp)
      endif

     case( 3 ) ! irevcm=3; before re-entry to          C05PDF, He must contain the derivatives;
      
      FATALMESS(ma02aa,.true.,have not yet implemented the derivatives)

     case default ! irevcm default;
      
      FATALMESS(ma02aa,.true.,illegal irevcm : C05 error)
      
     end select ! end of select case( irevcm );
     
    enddo ! end of infinite reverse-communication do loop;
    
   case default    

    FATALMESS(ma02aa,.true.,invalid Lconstraint supplied to Newton method)
    
   end select ! end select case( Lconstraint )
   
   cput = MPI_WTIME()
   select case(ic05pbf)
   case(-1   ) ;               write(ounit,1041) cput-cpus,myid,lvol,ic05pbf,error,"user termination                  "
   case( 0   ) ; if( Wma02aa ) write(ounit,1041) cput-cpus,myid,lvol,ic05pbf,error,"success                           "
   case( 1   ) ;               write(ounit,1041) cput-cpus,myid,lvol,ic05pbf,error,"input error                       "
   case( 2   ) ;               write(ounit,1041) cput-cpus,myid,lvol,ic05pbf,error,"exceeded max. evaluations         "
   case( 3   ) ;               write(ounit,1041) cput-cpus,myid,lvol,ic05pbf,error,"xtol too small                    "
   case( 4   ) ;               write(ounit,1041) cput-cpus,myid,lvol,ic05pbf,error,"bad progress                      "
   case default
    FATALMESS(ma02aa,.true.,illegal ic05 on final exit)     
   end select
1041 format("ma02aa : "f10.2" : myid="i3" ; lvol="i3" ; Newton : ic05pbf="i2" ; error="es13.5" ; "a34" ;")
   

   mu(lvol) = Xmuaa(0)
   Ldo = 0 ; packorunpack = 'U'
  !WCALL(ma02aa,up00aa,( packorunpack, lvol, NBdof, Xmuaa(1:NBdof), dpflux(lvol), Ldo )) ! unpack solution;
   call up00aa( packorunpack, lvol, NBdof, Xmuaa(1:NBdof), dpflux(lvol), Ldo ) ! unpack solution;

  endif ! end of if( LbeltramiNewton ) then
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  if( LbeltramiSeQuad .or. LbeltramiNewton ) then
   call dfstop ()
  endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  RETURN(ma02aa)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
end subroutine ma02aa

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
