!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Use preconditioned conjugate gradient method (proviced by the NAG routine E04DGF) to find minimum of energy functional.

!latex \item The energy functional is described in \verb+pc00ab+.

!latex \subsubheading{relevant input variables}
!latex \item The following input variables control the operation of \verb+E04DGF+
!latex \begin{itemize}
!latex \item \verb+epsilon+ : weighting of ``spectral energy'';
!latex                        see \verb+pc00ab+;
!latex \item \verb+maxstep+ : this is given to \verb+E04DGF+ for the \verb+Maximum Step Length+;
!latex                       for further details see the NAG documentation for \verb+E04DGF+;
!latex \item \verb+maxiter+ : upper limit on derivative calculations used in the conjugate gradient iterations;
!latex \item \verb+verify+ : if \verb+verify+$=1$, then \verb+E04DGF+ will confirm user supplied gradients (provided by \verb+pc00ab+) are correct;
!latex                       this is primarily used for debugging;
!latex                       for further details see the NAG documentation for \verb+E04DGF+;
!latex \end{itemize}

!latex \item Unfortunately, \verb+E04DGF+ seems to compute approximately $3 N$ function evaluations before proceeding to minimize the energy functional,
!latex where there are $N$ degrees of freedom.
!latex I don't know how to turn this off!

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!


!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine pc00aa( Nvol, mn, Ngdof, position, ie04dgf ) ! argument list is optional;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  use constants, only : zero, ten
  use numerical, only :
  use fileunits, only : ounit
  use inputlist, only : Wpc00aa, Lgeometry, Lconstraint, verify, maxstep, maxiter, epsilon, forcetol, forceerr
  use cputiming, only : Tpc00aa
  use allglobal, only : ncpu, myid, cpus, mBomn, mIomn
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  LOCALS

  INTEGER, intent(in)    :: Nvol, mn, Ngdof
  REAL   , intent(inout) :: position(1:Ngdof)
  INTEGER                :: ie04dgf

  LOGICAL                :: Lderiv
  INTEGER                :: niterations, Iwork(1:Ngdof+1), Iuser(1:4)
  REAL                   :: Energy, Gradient(1:Ngdof), work(1:13*Ngdof), Ruser(0:2)
  CHARACTER              :: smaxstep*34

  external               :: pc00ab

  BEGIN(pc00aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  if( myid.eq.0 ) then ; cput = MPI_WTIME() ; write(ounit,'("pc00aa : " 10x " :")') ; write(ounit,1000)cput-cpus,myid,Ngdof,epsilon,maxstep,maxiter,verify
  endif
  
1000 format("pc00aa : "f10.2" : myid="i3" ; calling E04DGF : Ngdof="i6" ; epsilon="es13.5" ; maxstep="es10.2" ; maxiter="i9" ; verify="i3" ;")

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  Lderiv= .false.
  WCALL(pc00aa,fc02aa,( Ngdof, position(1:Ngdof), Gradient(1:Ngdof), Lderiv ))


  if( myid.eq.0 ) then
   cput = MPI_WTIME()
                        write(ounit,'("pc00aa : "f10.2" : |f|="es12.5" : |B|="999f6.2)')cput-cpus,forceerr,mBomn(1:Nvol-1)
   if( Lgeometry.ge.4 ) write(ounit,'("pc00aa : " 10x " :     "  12x " : |I|="999f6.2)')                   mIomn(1:Nvol-1)
  endif
    
  if( forceerr.lt.forcetol ) then ; ie04dgf=0 ; goto 9999 ! force-balance is satisfied;  
  endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  select case( verify ) ! supply optional parameters to E04DGF;
   
  case( -1 ) ! no checks; no screen output;
   call E04DKF('Nolist')                          !; write(ounit,'("Nolist")')
   call E04DKF('Print Level = 0')                 !; write(ounit,'("Print Level = 0")')
   call E04DKF('Verify = -1')                     !; write(ounit,'("Verify = -1")')
  case(  0 ) ! simple check;
  !call E04DKF('Nolist')
  !call E04DKF('Print Level = 0')
   call E04DKF('Verify =  0') ! simple check
  case(  1 ) ! extensive test;
   call E04DKF('Verify =  1') ! extensive test;   
  case default
   FATALMESS(pc00aa,.true.,invalid verify : please set verify = -1 or verify = 0 or verify = +1)
  end select
  
  call E04DKF('Iteration Limit = 99999999')
  write(smaxstep,'("Maximum Step Length ="es13.5)')maxstep
  call E04DKF(smaxstep)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  Iuser(1:4) = (/ mn, Nvol, 0, 0 /) ! this is only used to pass information through to pc00ab; some resolution settings & iteration counter;
  Ruser(0:2) = zero                 ! this will be assigned in first call to pc00ab;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  ie04dgf = 1
  
  call E04DGF( Ngdof, pc00ab, niterations, Energy, Gradient(1:Ngdof), position(1:Ngdof), Iwork(1:Ngdof+1), work(1:13*Ngdof), Iuser(1:4), Ruser(0:2), ie04dgf )
  
  cput = MPI_WTIME()

  select case( ie04dgf )    
   case(:-1)    ; if( myid.eq.0 ) write(ounit,'("pc00aa : "f10.2" : user requested termination      ; ie04dgf="i3" ;")')cput-cpus,ie04dgf
   case(  0)    ; if( myid.eq.0 ) write(ounit,'("pc00aa : "f10.2" : success                         ; ie04dgf="i3" ;")')cput-cpus,ie04dgf
   case(  3)    ; if( myid.eq.0 ) write(ounit,'("pc00aa : "f10.2" : iteration limit reached         ; ie04dgf="i3" ;")')cput-cpus,ie04dgf
   case(  4)    ; if( myid.eq.0 ) write(ounit,'("pc00aa : "f10.2" : step length too small           ; ie04dgf="i3" ;")')cput-cpus,ie04dgf
   case(  6)    ; if( myid.eq.0 ) write(ounit,'("pc00aa : "f10.2" : not all minimum conditions met  ; ie04dgf="i3" ;")')cput-cpus,ie04dgf
   case(  7)    ; if( myid.eq.0 ) write(ounit,'("pc00aa : "f10.2" : user supplied derivatives error ; ie04dgf="i3" ;")')cput-cpus,ie04dgf
   case(  8)    ; if( myid.eq.0 ) write(ounit,'("pc00aa : "f10.2" : initial gradient too small      ; ie04dgf="i3" ;")')cput-cpus,ie04dgf
   case(  9)    ; if( myid.eq.0 ) write(ounit,'("pc00aa : "f10.2" : input error                     ; ie04dgf="i3" ;")')cput-cpus,ie04dgf
   case default ; FATALMESS(pc00aa,.true.,E04DGF ifail error)
   end select

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  RETURN(pc00aa)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

end subroutine pc00aa

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
