!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Fits Fourier harmonics to field line. Redundant;

!latex \item Given a set of points, $R_j$ and $Z_j$, this routines finds the values of $\iotabar$, $R_{m,n}$ and $Z_{m,n}$ that minimizes the sum of squares
!latex \be f \equiv \sum_j \left[ (R_j-\bar R_j)^2+ (Z_j-\bar Z_j)^2\right]
!latex \ee
!latex where
!latex \be \bar R_j & = & \sum_{m,n} R_{m,n} \cos( m \t_j - n \z_j),\\
!latex     \bar Z_j & = & \sum_{m,n} Z_{m,n} \sin( m \t_j - n \z_j),
!latex \ee
!latex and $\t_j = \iotabar \z_j$.

!latex \item The \verb+NAG+ routine \verb+E04HYF+ is used.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine pp20aa( Nj , Rj , Zj , mn , ii ) !, estiota , Rmn , Zmn , ifail )
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  use constants, only : zero, one, goldmean
  use numerical, only : small
  use fileunits, only : ounit, bunit
  use inputlist, only : Wpp20aa, Nvol, Nfp, iota
  use cputiming, only : Tpp20aa
  use allglobal, only : myid, ncpu, cpus, iRbc, iZbs, im, in
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  LOCALS
  
  INTEGER, intent(in)  :: Nj, mn, ii
  REAL   , intent(in)  :: Rj(0:Nj), Zj(0:Nj)
  
  INTEGER              :: jj
  
  INTEGER              :: Mresiduals , Nxdof , lengthwork , ifail, iuser(1:3), imn, LJC, idof, jres, iloop
  REAL                 :: fsumsq , xdof(1:1+mn+mn-1), pdof(1:1+mn+mn-1), user(1:2*Nj+2), FVECC(1:2*Nj+2), FVECO(1:2*Nj+2), FJACC(1:2*Nj+2,1+mn+mn-1), FJACO(1:2*Nj+2,1+mn+mn-1)
  REAL                 :: error
  REAL   , allocatable :: realwork(:)
  CHARACTER            :: suffix*4

  external             :: FITSURFACE!, LSHES2
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  BEGIN(pp20aa)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  Mresiduals = 2 * (Nj+1)          ! number of `residuals';
  
  Nxdof = 1 + mn + mn-1            ! number of variables : x degrees-of-freedom = {iota, Rmn, Zmn};
  
  xdof(1:Nxdof) = zero             ! a better initial guess will probably need to be provided;

  xdof(1)                = iota(Nvol)     ! + small ! initial guess for rotational-transform;
  xdof(2:mn+1)           = iRbc(Nvol,1:mn) ! + small ! initial guess for R harmonics;
  xdof(1+mn+1:1+mn+mn-1) = iZbs(Nvol,2:mn) ! + small;
  
  lengthwork = 8 * Nxdof + 2 * Nxdof * Nxdof + 2 * Mresiduals * Nxdof + 3 * Mresiduals ! length of work array;
  
  RALLOCATE(realwork,(1:lengthwork)) ! real work array;
  
  iuser(1:3) = (/ 0 , Nj , mn /) ! iteration counter; dimension checks;
  
  user(     1:     Nj+1) = Rj(0:Nj)  ! pass target values through to FITSURFACE and LSHES2;
  user(Nj+1+1:Nj+1+Nj+1) = Zj(0:Nj)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

! check derivatives;
!  LJC = Mresiduals
!  call FITSURFACE( Mresiduals , Nxdof , xdof(1:Nxdof) , FVECO(1:Mresiduals) , FJACO(1:LJC,1:Nxdof) , LJC , iuser(1:3) , user(1:2*Nj+2) )
!  
!  do idof = 1,Nxdof ! perturbed degree of freedom;
!
!   pdof(1:Nxdof) = xdof(1:Nxdof) ; pdof(idof) = xdof(idof) + small
!
!   call FITSURFACE( Mresiduals , Nxdof , pdof(1:Nxdof) , FVECC(1:Mresiduals) , FJACC(1:LJC,1:Nxdof) , LJC , iuser(1:3) , user(1:2*Nj+2) )
!
!   do jres = 1,Mresiduals
!    error = FJACO(jres,idof) - ( FVECC(jres)-FVECO(jres) ) / small
!    write(ounit,'(2i6,es23.15" error=",es10.2)')idof,jres,FJACO(jres,idof),error
!    if( abs(error).gt.1.0e-03) pause
!   enddo
!
!  enddo ! end of do idof;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  cput = MPI_WTIME()
  write(ounit,1001)cput-cpus,"calling",Mresiduals,Nxdof
  
  ifail = 1 ; fsumsq = -one
  call E04GZF( Mresiduals , Nxdof , FITSURFACE, xdof(1:Nxdof) , fsumsq , realwork(1:lengthwork) , lengthwork , iuser(1:3) , user(1:Mresiduals) , ifail )
! call E04HYF( Mresiduals , Nxdof , FITSURFACE, LSHES2, xdof(1:Nxdof) , fsumsq , realwork(1:lengthwork) , lengthwork , iuser(1:3) , user(1:Mresiduals) , ifail )
  
  cput = MPI_WTIME()
  
  select case(ifail)                                                                                        !1234567890123456789012345678
  case(0  )    ; write(ounit,1001)cput-cpus,"called ",Mresiduals,Nxdof,fsumsq/Nxdof,xdof(1),iuser(1),ifail,"success ;                   "
  case(1  )    ; write(ounit,1001)cput-cpus,"called ",Mresiduals,Nxdof,fsumsq/Nxdof,xdof(1),iuser(1),ifail,"input error ;               "
  case(2  )    ; write(ounit,1001)cput-cpus,"called ",Mresiduals,Nxdof,fsumsq/Nxdof,xdof(1),iuser(1),ifail,"50*N calls ;                "
  case(3  )    ; write(ounit,1001)cput-cpus,"called ",Mresiduals,Nxdof,fsumsq/Nxdof,xdof(1),iuser(1),ifail,"final point is not minimum ;"
  case(4  )    ; write(ounit,1001)cput-cpus,"called ",Mresiduals,Nxdof,fsumsq/Nxdof,xdof(1),iuser(1),ifail,"auxillary SVD failed ;      "
  case(5:8)    ; write(ounit,1001)cput-cpus,"called ",Mresiduals,Nxdof,fsumsq/Nxdof,xdof(1),iuser(1),ifail,"some doubt ;                "
  case(9  )    ; write(ounit,1001)cput-cpus,"called ",Mresiduals,Nxdof,fsumsq/Nxdof,xdof(1),iuser(1),ifail,"error forming derivatives ; "
  case default ; write(ounit,1001)cput-cpus,"called ",Mresiduals,Nxdof,fsumsq/Nxdof,xdof(1),iuser(1),ifail,"illegal ifail on exit ;     "
  end select
  
1001 format("pp20aa : "f10.2" : "a6" E04GZF : Mresiduals="i9" ; Nxdof="i6" ;":" fsumsq/N="es12.5" ; iota="es23.15" ; ncalls="i9" ; ifail="i3" ; ":,a28)

  DEALLOCATE(realwork)

 !FATALMESS(pp20aa,ifail.ne.0,failed)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  if( ifail.eq.0 ) then
   write(suffix,'(i4.4)')ii
   write(ounit,1002)cput-cpus,ii
   open(bunit,file="vacuum."//suffix//".rbczbs",status="unknown")
   do imn = 1,mn
    if( imn.eq.1) write(bunit,'(2i6,2es23.15)') in(imn)/Nfp, im(imn), xdof(1+imn), zero
    if( imn.gt.1) write(bunit,'(2i6,2es23.15)') in(imn)/Nfp, im(imn), xdof(1+imn), xdof(1+mn+imn-1)
   enddo
   close(bunit)
  endif
  
1002 format("pp20aa : "f10.2" : writing surface harmonics to vacuum."i4.4".rbczbs ;")

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  RETURN(pp20aa)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
end subroutine pp20aa

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!


!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

SUBROUTINE FITSURFACE( Mresiduals , Nxdof , XC , FVECC , FJACC , LJC , iuser , user )
  
  use constants, only : zero
  use numerical, only : 
  use fileunits, only : ounit
  use inputlist, only : Wpp20aa
  use cputiming, only : Tpp20aa
  use allglobal, only : myid, ncpu, cpus, Nz, mn, im, in, pi2nfp
  
  LOCALS
  
  INTEGER :: Mresiduals ! residuals;
  INTEGER :: Nxdof ! variables;
  
  INTEGER :: LJC, iuser(1:3)
  
  REAL    :: XC(1:Nxdof), FVECC(1:Mresiduals), FJACC(1:LJC,1:Nxdof), user(1:Mresiduals)
  
  INTEGER :: Nj, jj, imn
  REAL    :: iota, Rmn(1:Nxdof), Zmn(1:Nxdof), teta, zeta, arg, carg, sarg
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  Nj = ( Mresiduals / 2 ) - 1

  iuser(1) = iuser(1) + 1 ! counter;

  FATALMESS(pp20aa,iuser(2).ne.Nj,dimension check failed)
  FATALMESS(pp20aa,iuser(3).ne.mn,dimension check failed)

  FJACC(1:LJC,1:Nxdof) = zero ! set default;
    
  iota      = XC(1)
  Rmn(1:mn) = XC(1+1:1+mn)
  Zmn(1)    = zero
  Zmn(2:mn) = XC(1+mn+1:1+mn+mn-1)
                 
!  write(ounit,'("FITSURFACE : "10x" : iota="es23.15" ;")')iota
!  write(ounit,'("FITSURFACE : "10x" : im ="999i10)')im(1:mn)
!  write(ounit,'("FITSURFACE : "10x" : in ="999i10)')in(1:mn)
!  write(ounit,'("FITSURFACE : "10x" : Rmn="    999es10.02)')Rmn(1:mn)
!  write(ounit,'("FITSURFACE : "10x" : Zmn="10x,998es10.02)')Zmn(2:mn)  
  
  FVECC(1:2*Nj+2) = -user(1:2*Nj+2) ! R_j and Z_j; initialize summation;

  do jj = 0 , Nj ! loop over residuals;
   
   zeta = jj * pi2nfp / Nz ; teta = iota * zeta
   
   do imn = 1 , mn ! loop over variables = Fourier hamonics;
    
    arg = im(imn)*teta - in(imn)*zeta ; carg = cos(arg) ; sarg = sin(arg)
    
    FVECC(     1+jj) = FVECC(     1+jj) + Rmn(imn) * carg ! R_j ;
    FVECC(Nj+1+1+jj) = FVECC(Nj+1+1+jj) + Zmn(imn) * sarg ! Z_j ;
    
    FJACC(     1+jj,1) = FJACC(     1+jj,1) + Rmn(imn) * sarg * ( - im(imn) * zeta ) ! d R_j / d iota;
    FJACC(Nj+1+1+jj,1) = FJACC(Nj+1+1+jj,1) + Zmn(imn) * carg * ( + im(imn) * zeta ) ! d Z_j / d iota;
    
    if( imn.ge.0 ) FJACC(     1+jj,   1+imn  ) = carg ! d R_j / d R_mn
    if( imn.gt.1 ) FJACC(Nj+1+1+jj,mn+1+imn-1) = sarg ! d Z_j / d Z_mn ; does not depend on Zmn_{0,0};
    
   enddo ! end of do imn;

  enddo ! end of do jj;

  return
  
end SUBROUTINE FITSURFACE

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!


!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!SUBROUTINE LSHES2( Mresiduals , Nxdof , FVECC , XC , B , LB , iuser , user )
!  
!  use constants, only : zero
!  use numerical, only : 
!  use fileunits, only : ounit
!  use inputlist, only : Wpp20aa
!  use cputiming, only : Tpp20aa
!  use allglobal, only : myid, ncpu, cpus
!  
!  LOCALS
!  
!  INTEGER :: Mresiduals, Nxdof, LB, iuser(1:3)
!  REAL    :: FVECC(1:Mresiduals), XC, B(1:LB), user(1:Nxdof)
!  
!  FATALMESS(LSHES2,.true.,not yet implemented)
!
!  return
!  
!end SUBROUTINE LSHES2

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
