!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Compute spectral-width extremizing condition.

!latex \subheading{spectral width}

!latex \item The geometry of an interface is described by two functions, $R=\sum_j R_j \cos(m_j \t-n_j \z)$ and $Z=\sum_j Z_j \sin(m_j \t-n_j \z)$.
!latex (See \verb+global+ and \verb+co01a+ for more details.)
!latex \item The spectral width is defined
!latex \be M = \frac{1}{2}\sum_j (m_j^p+n_j^q) \left(R_j^2+Z_j^2\right).\ee
!latex where $p\equiv$ \verb+pwidth+, $q\equiv$ \verb+qwidth+ are positive integers given on input, and $m_j^p=0$ for $m_j=0$, $n_j^q=0$ for $n_j=0$.

!latex \subheading{tangential variations}

!latex \item We seek to extremize the spectral width without changing the geometry of the interface.
!latex Accordingly, we restrict attention to tangential variations, i.e. variations of the form
!latex \be \delta R &=& R_\t \; \delta u,\\
!latex     \delta Z &=& Z_\t \; \delta u .
!latex \ee
!latex \item To preserve stellarator symmetry, we consider $\delta u = \sum_k u_k \sin(m_k\t-n_k\z)$.
!latex \item The variations in the Fourier harmonics of $R$ and $Z$ are given by
!latex \be \delta R_j &=& \ooint R_\t \; \delta u \; \cos(m_j \t - n_j \z), \\
!latex     \delta Z_j &=& \ooint Z_\t \; \delta u \; \sin(m_j \t - n_j \z), \ee
!latex \item The first variation in $M$ as
!latex \be \delta M = \ooint \left(R_\t \; X +  Z_\t \; Y \right) \delta u,
!latex \ee
!latex where $X = \sum_j (m_j^p+n_j^q)R_{j} \cos(m_j\t-n_j\z)$ and $Y = \sum_j (m_j^p+n_j^q)Z_{j} \sin(m_j\t-n_j\z)$

!latex \subheading{extremizing condition}

!latex \item The condition that $\delta M = 0$ for arbitrary $\delta u$ is 
!latex \be I \equiv R_\t \; X + Z_\t \; Y = 0. \ee
!latex \item The derivatives of $M$ with respect to the $u_k$ are given
!latex \be \frac{\partial M}{\partial u_k} = \ooint (R_\t X + Z_\t Y) \sin(m_k\t-n_k\z).\ee
!latex These quantities are provided by an fast Fourier transform of $I$, which is computed in \verb+cb02aa+.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine sw02aa( Ntz, lvol, II, mn, dIdR, dIdZ ) ! depends on iRbc,iZbs;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  use constants, only : zero, one

  use numerical, only : vsmall

  use fileunits, only : ounit

  use inputlist, only : Wsw02aa

  use cputiming, only : Tsw02aa

  use allglobal, only : myid, cpus, iRbc, iZbs, im, in, mpnq, Nt, Nz, isr, trigm, trign, trigwk, cosj, sinj

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  LOCALS
  
  INTEGER, intent(in)  :: Ntz, lvol, mn
  REAL   , intent(out) :: II(1:Ntz)
  
  REAL   , intent(out) :: dIdR(1:mn,1:Ntz), dIdZ(1:mn,1:Ntz)

  INTEGER              :: imn
  REAL                 :: efmn(1:mn), ofmn(1:mn), cfmn(1:mn), sfmn(1:mn), ijreal(1:Ntz), ijimag(1:Ntz), jireal(1:Ntz), jiimag(1:Ntz) ! workspace;

  BEGIN(sw02aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  efmn(1:mn) = zero ; sfmn(1:mn) = zero !                                                            ijreal = R_\t; ijimag = Z_\t;
  call invfft( mn, im, in, efmn(1:mn), -im*iRbc(lvol,1:mn), +im*iZbs(lvol,1:mn), sfmn(1:mn), Nt, Nz, ijreal(1:Ntz), ijimag(1:Ntz), isr, trigm, trign, trigwk )
  
  efmn(1:mn) = zero
  sfmn(1:mn) = zero ! Fourier harmonics; dummy workspace;

  do imn = 2, mn ; efmn(imn) = mpnq(imn) * iRbc(lvol,imn) ! X_j;
                   sfmn(imn) = mpnq(imn) * iZbs(lvol,imn) ! Y_j;
  enddo
  
  ofmn(1:mn) = zero ; cfmn(1:mn) = zero !                                          jireal = X   ; jiimag = Y   ;
  call invfft( mn, im, in, efmn(1:mn), ofmn(1:mn), cfmn(1:mn), sfmn(1:mn), Nt, Nz, jireal(1:Ntz), jiimag(1:Ntz), isr, trigm, trign, trigwk )


!  I        = R_\t            X             + Z_\t            Y            
  II(1:Ntz) = ijreal(1:Ntz) * jireal(1:Ntz) + ijimag(1:Ntz) * jiimag(1:Ntz)
  
  dIdR(1,1:Ntz) = zero
  dIdZ(1,1:Ntz) = zero

  do imn = 2, mn

   dIdR(imn,1:Ntz) = -im(imn) * sinj(imn,1:Ntz) * jireal(1:Ntz) + ijreal(1:Ntz) * mpnq(imn) * cosj(imn,1:Ntz) ! derivative of I wrt R_j;
   dIdZ(imn,1:Ntz) = +im(imn) * cosj(imn,1:Ntz) * jiimag(1:Ntz) + ijimag(1:Ntz) * mpnq(imn) * sinj(imn,1:Ntz) ! derivative of I wrt Z_j;

  enddo

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  RETURN(sw02aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

end subroutine sw02aa

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \subheading{comments}

!latex \item For \verb+pwidth+$=2$, and ignoring the $n^q$ term, we see \cite{Hirshman_Breslau_98} that $X\equiv - R_{\t\t}$ and $Y\equiv - Z_{\t\t}$,
!latex and the extremizing condition reduces to $R_\t R_{\t\t} + Z_\t Z_{\t\t}=0$,
!latex which is equivalent to the equal arc length condition, $R_\t^2 + Z_\t^2 = const.$

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

