!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Given the Fourier harmonics of the interface geometry, return spectral constraints.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \subsubheading{spectral width}

!latex \item The geometry of an interface is described by two functions, $R=\sum_j R_j \cos(m_j \t-n_j \z)$ and $Z=\sum_j Z_j \sin(m_j \t-n_j \z)$.
!latex (See \verb+global+ and \verb+co01a+ for more details.)
!latex \item The spectral width is defined
!latex \be M = \frac{1}{2}\sum_j (m_j^p+n_j^q) \left(R_j^2+Z_j^2\right).\ee
!latex where $p\equiv$ \verb+pwidth+, $q\equiv$ \verb+qwidth+ are positive integers given on input, and $m_j^p=0$ for $m_j=0$, $n_j^q=0$ for $n_j=0$.

!latex \subsubheading{tangential variations}

!latex \item We seek to extremize the spectral width without changing the geometry of the interface.
!latex Accordingly, we restrict attention to tangential variations, i.e. variations of the form
!latex \be \delta R &=& R_\t \; \delta u ,\\
!latex     \delta Z &=& Z_\t \; \delta u .
!latex \ee
!latex \item To preserve stellarator symmetry, we consider $\delta u = \sum_k u_k \sin(m_k\t-n_k\z)$.
!latex \item The variations in the Fourier harmonics of $R$ and $Z$ are given by
!latex \be \delta R_j &=& \ooint R_\t \; \delta u \; \cos(m_j \t - n_j \z), \\
!latex     \delta Z_j &=& \ooint Z_\t \; \delta u \; \sin(m_j \t - n_j \z), \ee
!latex \item The first variation in $M$ as
!latex \be \delta M = \ooint \left(R_\t \; X +  Z_\t \; Y \right) \delta u,
!latex \ee
!latex where $X = \sum_j (m_j^p+n_j^q)R_{j} \cos(m_j\t-n_j\z)$ and $Y = \sum_j (m_j^p+n_j^q)Z_{j} \sin(m_j\t-n_j\z)$

!latex \subsubheading{extremizing condition}

!latex \item The condition that $\delta M = 0$ for arbitrary $\delta u$ is 
!latex \be I \equiv R_\t \; X + Z_\t \; Y = 0. \ee
!latex \item The derivatives of $M$ with respect to the $u_k$ are given
!latex \be \frac{\partial M}{\partial u_k} = \ooint (R_\t X + Z_\t Y) \sin(m_k\t-n_k\z).\ee

!latex \subsubheading{comments}

!latex \item For \verb+pwidth+$=2$, and ignoring the $n^q$ term, we see \cite{Hirshman_Breslau_98} that $X\equiv - R_{\t\t}$ and $Y\equiv - Z_{\t\t}$,
!latex and the extremizing condition reduces to $R_\t R_{\t\t} + Z_\t Z_{\t\t}=0$,
!latex which is equivalent to the equal arc length condition, $R_\t^2 + Z_\t^2 = const.$

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine sw03aa( mn, iRbc, iZbs, Iomn )
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  use constants, only : zero, half, one, two
  use numerical, only : 
  use fileunits, only : ounit
  use inputlist, only : Wsw03aa, ext 
  use cputiming, only : Tsw03aa  
  use allglobal, only : ncpu, myid, cpus, im, in, mpnq
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  LOCALS
  
  INTEGER, intent(in)  :: mn
  REAL,    intent(in)  :: iRbc(1:mn), iZbs(1:mn)
  REAL,    intent(out) :: Iomn(1:mn)

  INTEGER              :: imn, jmn, kmn, mi, ni, mj, nj, mk, nk
  REAL                 :: RmnsqdplusZmnsqd, SpectralWidthNum, SpectralWidthDen, SpectralWidth

  BEGIN(sw03aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
! do imn = 1, mn ; RmnsqdplusZmnsqd = iRbc(imn)**2 + iZbs(imn)**2
!  SpectralWidthNum = SpectralWidthNum + mpnq(imn) * RmnsqdplusZmnsqd
!  SpectralWidthDen = SpectralWidthDen +             RmnsqdplusZmnsqd
   SpectralWidthDen = one                                             ! need to check this; I think that derivatives of spectral width computed in fc02aa ignore normalization;
! enddo
!  SpectralWidth    = SpectralWidthNum / SpectralWidthDen
   SpectralWidth    = zero                                            ! need to check this; I think that derivatives of spectral width computed in fc02aa ignore normalization;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  Iomn(1:mn) = zero ! initialize summation variable for spectral constraint harmonics; all harmonics;
  
  do jmn = 2, mn ; mj = im(jmn) ; nj = in(jmn) ! odd symmetry guarantees that I_i=0 for m_i=0, n_i=0;
   
   do imn = 1, mn ; mi = im(imn) ; ni = in(imn) ! with some effort, and if required, could eliminate loops for efficiency;
    do kmn = 2, mn ; mk = im(kmn) ; nk = in(kmn)
     
     if( mi+mk.eq. mj .and. ni+nk.eq. nj ) Iomn(jmn) = Iomn(jmn) + mi * ( mpnq(kmn) - SpectralWidth ) * ( iZbs(imn)*iZbs(kmn) - iRbc(imn)*iRbc(kmn) )
     if( mi-mk.eq. mj .and. ni-nk.eq. nj ) Iomn(jmn) = Iomn(jmn) - mi * ( mpnq(kmn) - SpectralWidth ) * ( iZbs(imn)*iZbs(kmn) + iRbc(imn)*iRbc(kmn) )

    !if( mk-mi.eq. mj .and. nk-ni.eq. nj ) Iomn(jmn) = Iomn(jmn) + mi * ( mpnq(kmn) - SpectralWidth ) * ( iZbs(imn)*iZbs(kmn) + iRbc(imn)*iRbc(kmn) )
     if( mi-mk.eq.-mj .and. ni-nk.eq.-nj ) Iomn(jmn) = Iomn(jmn) + mi * ( mpnq(kmn) - SpectralWidth ) * ( iZbs(imn)*iZbs(kmn) + iRbc(imn)*iRbc(kmn) )
     
    enddo ! end of do kmn;
   enddo ! end of do imn;
   
  enddo ! end of do jmn = 1, mn loop;
  
  Iomn(1:mn) = half * Iomn(1:mn) / SpectralWidthDen
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item The derivatives of the spectral constraints, $I =R_\t X + Z_\t Y$,  are derived using $\sin(\alpha+\beta)=\left[\sin(\alpha+\beta)+\sin(\alpha-\beta)\right]/2$
!latex to give
!latex \be \frac{\partial I}{\partial R_j} &=& \frac{\partial R_\t}{\partial R_j} X + R_\t \frac{\partial X}{\partial R_j} \nonumber \\
!latex                                     &=& \frac{1}{2} \sum_k \left[ - ( m_j \lambda_k + m_k \lambda_j ) R_k \sin (\alpha_j + \alpha_k )
!latex                                                                   - ( m_j \lambda_k - m_k \lambda_j ) R_k \sin (\alpha_j - \alpha_k ) \right]\\
!latex     \frac{\partial I}{\partial Z_j} &=& \frac{\partial Z_\t}{\partial R_j} Y + Z_\t \frac{\partial Y}{\partial R_j} \nonumber \\
!latex                                     &=& \frac{1}{2} \sum_k \left[ + ( m_j \lambda_k + m_k \lambda_j ) Z_k \sin (\alpha_j + \alpha_k )
!latex                                                                   - ( m_j \lambda_k - m_k \lambda_j ) Z_k \sin (\alpha_j - \alpha_k ) \right].
!latex \ee

!  DspectralDrz(lvol,lgdof,2:mn) = zero ! initialize summation;
!  
!  mi = im(imn) ; ni = in(imn)
!  
!  do jmn = 2, mn ; mj = im(jmn) ; nj = in(jmn)
!   
!   do kmn = 2, mn ; mk = im(kmn) ; nk = in(kmn)
!    
!    if( irz.eq.0 ) then 
!     if( mi+mk.eq.mj .and. ni+nk.eq.nj ) DspectralDrz(lvol,lgdof,jmn) = DspectralDrz(lvol,lgdof,jmn) - (mi*mpnq(kmn)+mk*mpnq(imn)) * iRbc(lvol,kmn)
!     if( mi-mk.eq.mj .and. ni-nk.eq.nj ) DspectralDrz(lvol,lgdof,jmn) = DspectralDrz(lvol,lgdof,jmn) - (mi*mpnq(kmn)-mk*mpnq(imn)) * iRbc(lvol,kmn)
!     if( mk-mi.eq.mj .and. nk-ni.eq.nj ) DspectralDrz(lvol,lgdof,jmn) = DspectralDrz(lvol,lgdof,jmn) + (mi*mpnq(kmn)-mk*mpnq(imn)) * iRbc(lvol,kmn)
!    endif
!    
!    if(irz.eq.1)then
!     if( mi+mk.eq.mj .and. ni+nk.eq.nj ) DspectralDrz(lvol,lgdof,jmn) = DspectralDrz(lvol,lgdof,jmn) + (mi*mpnq(kmn)+mk*mpnq(imn)) * iZbs(lvol,kmn)
!     if( mi-mk.eq.mj .and. ni-nk.eq.nj ) DspectralDrz(lvol,lgdof,jmn) = DspectralDrz(lvol,lgdof,jmn) - (mi*mpnq(kmn)-mk*mpnq(imn)) * iZbs(lvol,kmn)
!     if( mk-mi.eq.mj .and. nk-ni.eq.nj ) DspectralDrz(lvol,lgdof,jmn) = DspectralDrz(lvol,lgdof,jmn) + (mi*mpnq(kmn)-mk*mpnq(imn)) * iZbs(lvol,kmn)
!    endif
!    
!   enddo ! end of do kmn;
!  enddo ! end of do jmn;
!  
!  DspectralDrz(lvol,lgdof,2:mn) = half * DspectralDrz(lvol,lgdof,2:mn) ! derivatives of angle constraints wrt Rj and Zj;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  RETURN(sw03aa)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
end subroutine sw03aa
 
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
