!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Constructs straight field line coordinates on interfaces, 
!latex and calculates rotational transform for an arbitrary tangential field.

!latex \end{enumerate} \subsubsection{constructing straight field line angle on interfaces} \begin{enumerate}

!latex \item The algorithm stems from introducing a straight field line angle $\theta_s=\theta+\lambda(\theta,\zeta)$, 
!latex       where $\lambda=\sum_i \lambda_{i}\sin(m_i\theta-n_i\zeta)$ and insisting that
!latex       \be
!latex       \iotabar \equiv \frac{{\bf B}\cdot \nabla \theta_s}{{\bf B}\cdot \nabla \zeta} = \dot \theta ( 1 + \lambda_\theta) + \lambda_\zeta = const.
!latex       \ee

!latex \item Writing $\dot \theta = - \partial_s A_\zeta / \partial_s A_\theta$, we have
!latex       \be \label{eq:sfla} 
!latex       \partial_s A_\theta \, \iotabar + \partial_s A_\zeta \, \lambda_\theta - \partial_s A_\theta \, \lambda_\zeta = - \partial_s A_\zeta.
!latex       \ee
!latex       By equating coefficients of the Fourier harmonics, an equation of the form ${\bf A} \cdot {\bf x} = {\bf b}$ is obtained,
!latex       where ${\bf x}=(\iotabar,\lambda_2, \lambda_3,\dots)^T$.
!latex \item The Fourier harmonics of \Eqn{sfla} are computed by double angle formulae.
!latex \item The solution to the linear system is provided by NAG \verb+f04atf+.

!latex \item Note : the Fourier resolution of the angle transformation defined by $\lambda$ may need to be quite large.
!latex       The resolution of the angle transformation is defined by \verb+iMpol+ and \verb+iNtor+ which are given on input.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!


!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine tr00ab( lvol, Nvol, mns, ldiota, iflag ) ! construct straight-field line magnetic coordinates;

  use constants, only : zero, half, one

  use numerical, only : vsmall, small

  use fileunits, only : ounit

  use inputlist, only : Wtr00ab, Lgeometry, Ni, Nofe

  use cputiming, only : Ttr00ab

  use allglobal, only : cpus, myid, mn, im, in, interfacelabel, ims, ins, &
                        doAtmn, doAzmn, dsflangle, doAt, doAz, &
                        pi2nfp, Nz, doubleim, doubleip, &
                        Liotasparse, Liotastandard, LiotaSVD, &
                        AngleLogical, AngleNZ, AngleLIRN, AngleLICN, AngleIRN, AngleICN, AngleIKEEP, AngleSparseMatrix, AngleIVECT, AngleJVECT, AngleIDISP,&
                        doubleex, doubleie, doubleip, doubleim, doublein, & 
                        Lcoordinatesingularity, Louterboundarynormal

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  LOCALS 

  INTEGER, intent(in)  :: lvol, Nvol, mns, iflag ! note that these are really global quantities, but included in argument list to remove allocations;

  REAL, intent(out)    :: ldiota(0:2,0:1) ! extra memory is included for the derivatives; first argument indicates derivative; second argument indicates inner/outer interface;

  INTEGER              :: innout, Nio, Lg, NN, MM, imp, mj, nj, Ldo
  REAL                 :: lss, error

! variables required for constructing matrix that defines transformation to straight-field-line angle on interfaces;
  INTEGER              :: imn, jmn, kmn, madd, nadd, msub, nsub
  REAL                 :: omatrix(-1:2,1:mns,1:mns), orhs(-1:2,1:mns) ! matrix and right hand side; for original;

  LOGICAL              :: lblock, grow, abort(1:4)
  INTEGER              :: LNZ, if01brf, if01bsf, Iwork(1:8*mns), mtype
  REAL                 :: pivot, resid, eta, rpmin, Rwork(1:mns)

! variables required for standard linear solver, F04ATF, or singular value decomposition routine, F04jgf;
  LOGICAL              :: SVDlogical
  INTEGER              :: if04atf, IA, IMat, if04jgf, NRA, IRANK, lwkc
  REAL                 :: svdtol, SIGMA
  REAL                 :: matrix(1:mns,1:mns), rhs(1:mns)
  REAL                 :: workmat(1:mns,1:mns), wka(1:mns), wkb(1:mns), wkc(1:4*mns) ! dummy workspace required for NAG routines

  REAL                 :: cpuConstruct(0:1), cpuSparse(0:2), cpuStandard, cpuSVD

  LOGICAL              :: Lfirstcall = .true.

  BEGIN(tr00ab)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Note that the radial derivatives of the vector potential are required.
!latex Presently, the routine only works if \verb+Nofe.gt.0+, so that cubic or higher order radial basis functions are used to describe the vector potential.
!latex However, for the linear case (where radial gradients are not really defined on the sub-radial grid) one could take the `inside limit'.

  FATALMESS(tr00ab,Nofe.lt.1,derivatives of vector potential are required to have continuous magnetic field)

  FATALMESS(tr00ab,mns.le.0,no degrees of freedom in angle transformation)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  ldiota(0:2,0:1) = zero ! set default intent out;

  NN = mns ; MM = mns ; IA = NN ; IMat = IA ; NRA = MM ; svdtol = small ; lwkc = 4 * NN

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  do innout = 0, 1 ! loop over inner and outer interfaces;

   if( innout.eq.0 ) then ; Nio = 0
   else                   ; Nio = Ni(lvol)
   endif
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

   if( Lcoordinatesingularity .and. innout.eq.0 ) cycle ! iota on axis     not required;
   if( Louterboundarynormal   .and. innout.eq.1 ) cycle ! iota on boundary not required;
    
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
   FATALMESS(tr00ab,lvol-1+innout.lt.0 .or. lvol-1+innout.gt.Nvol,invalid interface label)
   
   lss = interfacelabel(lvol-1+innout) ! local radial coordinate;
   
   do Ldo = 0, 2
    WCALL(tr00ab,sa00aa,( lvol, lss, Ldo )) ! interpolate Fourier harmonics of vector potential; returns doAtmn & doAzmn;
   enddo

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
! construct matrix that defines transformation to straight field line angle;
   
   cpuConstruct(0) = MPI_WTIME()
   
  !omatrix(0:Ldmupf,1:mns,1:mns) = zero ; orhs(0:Ldmupf,1:mns) = zero ! initialize summation of linear system that defines angle transformation; these should neve be corrupted;
   omatrix(0:2     ,1:mns,1:mns) = zero ; orhs(0:2     ,1:mns) = zero ! initialize summation of linear system that defines angle transformation; these should neve be corrupted;
   
!   do kmn = 1, mns
!    
!    do imn = 1, mn
!     
!     if( im(imn).eq.ims(kmn) .and. in(imn).eq.ins(kmn) ) then
!          orhs(0:Ldmupf,kmn  ) =    orhs(0:Ldmupf,kmn  ) - doAzmn(0:Ldmupf,1,imn)
!       omatrix(0:Ldmupf,kmn,1) = omatrix(0:Ldmupf,kmn,1) + doAtmn(0:Ldmupf,1,imn)
!     endif
!     
!    enddo ! end of do imn = 1,mn;
!    
!    do jmn = 2, mns ! really need only go from jmn = 2,.. as lambda is odd, so the first harmonic is zero;
!     do imn = 1, mn
!      
!      madd = im(imn)+ims(jmn) ; nadd = in(imn) + ins(jmn)
!      msub = im(imn)-ims(jmn) ; nsub = in(imn) - ins(jmn)
!      
!      if(                 madd .eq. ims(kmn) .and. nadd .eq. ins(kmn) ) then
!       omatrix(0:Ldmupf,kmn,jmn) = omatrix(0:Ldmupf,kmn,jmn) + half * ( doAzmn(0:Ldmupf,1,imn)          * ims(jmn) + doAtmn(0:Ldmupf,1,imn)          * ins(jmn) )
!      endif
!      
!      if(                 msub .eq. ims(kmn) .and. nsub .eq. ins(kmn) ) then
!       omatrix(0:Ldmupf,kmn,jmn) = omatrix(0:Ldmupf,kmn,jmn) + half * ( doAzmn(0:Ldmupf,1,imn)          * ims(jmn) + doAtmn(0:Ldmupf,1,imn)          * ins(jmn) )
!      endif
!
!      if( msub.lt.0 .and. msub .eq.-ims(kmn) .and. nsub .eq.-ins(kmn) ) then
!       omatrix(0:Ldmupf,kmn,jmn) = omatrix(0:Ldmupf,kmn,jmn) + half * ( doAzmn(0:Ldmupf,1,imn)          * ims(jmn) + doAtmn(0:Ldmupf,1,imn)          * ins(jmn) )
!      endif
!      
!     enddo ! end of do imn;
!     
!    enddo ! end of do jmn;
!    
!   enddo ! end of do kmn;
!   
!   cpuConstruct(1) = MPI_WTIME()
!   
!   omatrix(-1,1:mns,1:mns) = zero ; orhs(-1,1:mns) = zero ! initialize summation of linear system that defines angle transformation;
   
   do kmn = 1, mns

!   omatrix(0:Ldmupf,kmn,1) =  doAtmn(0:Ldmupf,1,doubleie(kmn,1))
!      orhs(0:Ldmupf,kmn  ) = -doAzmn(0:Ldmupf,1,doubleie(kmn,1))
    omatrix(0:2     ,kmn,1) =  doAtmn(0:2     ,1,doubleie(kmn,1))
       orhs(0:2     ,kmn  ) = -doAzmn(0:2     ,1,doubleie(kmn,1))

    do jmn = 2, mns

     if( .not.doubleex(kmn,jmn) ) cycle

     mj = ims(jmn) ; nj = ins(jmn)

!    omatrix(0:Ldmupf,kmn,jmn) = ( mj * ( doAzmn(0:Ldmupf,1,doubleim(kmn,jmn)) + doAzmn(0:Ldmupf,1,doubleip(kmn,jmn)) ) &
!                                + nj * ( doAtmn(0:Ldmupf,1,doubleim(kmn,jmn)) + doAtmn(0:Ldmupf,1,doubleip(kmn,jmn)) ) &
!                                + mj *   doAzmn(0:Ldmupf,1,doublein(kmn,jmn))                                          &
!                                + nj *   doAtmn(0:Ldmupf,1,doublein(kmn,jmn))                                          &
!                                 ) * half

     omatrix(0:2     ,kmn,jmn) = ( mj * ( doAzmn(0:2     ,1,doubleim(kmn,jmn)) + doAzmn(0:2     ,1,doubleip(kmn,jmn)) ) &
                                 + nj * ( doAtmn(0:2     ,1,doubleim(kmn,jmn)) + doAtmn(0:2     ,1,doubleip(kmn,jmn)) ) &
                                 + mj *   doAzmn(0:2     ,1,doublein(kmn,jmn))                                          &
                                 + nj *   doAtmn(0:2     ,1,doublein(kmn,jmn))                                          &
                                  ) * half
    enddo

   enddo

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
   if( Liotasparse ) then
    
    cpuSparse(0) = MPI_WTIME()
    
    LNZ = 0
    do kmn = 1, mns
     do jmn = 1, mns
      if( AngleLogical(kmn,jmn) ) then ; LNZ = LNZ + 1 ; AngleIVECT(LNZ) = kmn ; AngleJVECT(LNZ) = jmn ; AngleSparseMatrix(LNZ) = omatrix(0,kmn,jmn)
      endif
     enddo
    enddo
    FATALMESS(tr00ab,LNZ.ne.AngleNZ,need to check sparsity pattern more closely)
    
    cpuSparse(1) = MPI_WTIME()
    
    if( Lfirstcall ) then
     
     pivot = 0.1 ; lblock = .true. ; grow = .false. ; abort(1:4) = (/ .true., .true., .false., .true. /) ; if01brf = 1
     call F01BRF( NN, AngleNZ, AngleSparseMatrix, AngleLICN, AngleIRN, AngleLIRN, AngleICN, pivot, AngleIKEEP, Iwork, Rwork, lblock, grow, abort(1:4), AngleIDISP(1:10), if01brf )
     
     cput = MPI_WTIME()
     select case( if01brf ) 
     case( -2    ) ;               write(ounit,'("tr00ab : "f10.2" : if01brf="i2" ; numerically singular ; ok ;    ")')cput-cpus,if01brf
     case( -1    ) ;               write(ounit,'("tr00ab : "f10.2" : if01brf="i2" ; structurally singular ; ok ;   ")')cput-cpus,if01brf
     case(  0    ) ; if( Wtr00ab ) write(ounit,'("tr00ab : "f10.2" : if01brf="i2" ; success ;                      ")')cput-cpus,if01brf
     case(  1    ) ;               write(ounit,'("tr00ab : "f10.2" : if01brf="i2" ; structurally singular; fail ;  ")')cput-cpus,if01brf
     case(  2    ) ;               write(ounit,'("tr00ab : "f10.2" : if01brf="i2" ; numerically singular ; fail ;  ")')cput-cpus,if01brf
     case(  3    ) ;               write(ounit,'("tr00ab : "f10.2" : if01brf="i2" ; LIRN is too small ; try again ;")')cput-cpus,if01brf
     case(  4    ) ;               write(ounit,'("tr00ab : "f10.2" : if01brf="i2" ; LICN is much too small ;       ")')cput-cpus,if01brf
     case(  5    ) ;               write(ounit,'("tr00ab : "f10.2" : if01brf="i2" ; some LU factors discarded ;    ")')cput-cpus,if01brf
     case(  6    ) ;               write(ounit,'("tr00ab : "f10.2" : if01brf="i2" ; LIRN & LICN too small ;        ")')cput-cpus,if01brf
     case(  7    ) ;               write(ounit,'("tr00ab : "f10.2" : if01brf="i2" ; LICN too small ;               ")')cput-cpus,if01brf
     case(  8:13 ) ;               write(ounit,'("tr00ab : "f10.2" : if01brf="i2" ; input error ;                  ")')cput-cpus,if01brf
     case default  ;               write(ounit,'("tr00ab : "f10.2" : if01brf="i2" ; invalid ifail ;                ")')cput-cpus,if01brf
     end select

     FATALMESS(tr00ab,if01brf.ne.0,failed to construct LU factorization of sparse matrix)
     
    else ! this is not the first call ;
     
     grow = .false. ; eta = one ; if01bsf = 1
     call F01BSF( NN, AngleNZ, AngleSparseMatrix, AngleLICN, AngleIVECT, AngleJVECT, AngleICN, AngleIKEEP, Iwork, Rwork, grow, eta, rpmin, abort(1), AngleIDISP(1:2), if01bsf )
     
     cput = MPI_WTIME()
     select case( if01bsf ) 
     case( 0   )  ; if( Wtr00ab ) write(ounit,'("tr00ab : "f10.2" : myid="i3" ; lvol="i3" ; if01bsf="i2" ; success ;                 ")')cput-cpus,myid,lvol,if01bsf
     case( 1:4 )  ;               write(ounit,'("tr00ab : "f10.2" : myid="i3" ; lvol="i3" ; if01bsf="i2" ; input error ;             ")')cput-cpus,myid,lvol,if01bsf
     case(   5 )  ;               write(ounit,'("tr00ab : "f10.2" : myid="i3" ; lvol="i3" ; if01bsf="i2" ; inconsistent with F01BRF ;")')cput-cpus,myid,lvol,if01bsf
     case(   6 )  ;               write(ounit,'("tr00ab : "f10.2" : myid="i3" ; lvol="i3" ; if01bsf="i2" ; numerically singular ;    ")')cput-cpus,myid,lvol,if01bsf
     case(   7 )  ; if( Wtr00ab ) write(ounit,'("tr00ab : "f10.2" : myid="i3" ; lvol="i3" ; if01bsf="i2" ; very small pivot ;        ")')cput-cpus,myid,lvol,if01bsf
     case(   8 )  ;               write(ounit,'("tr00ab : "f10.2" : myid="i3" ; lvol="i3" ; if01bsf="i2" ; duplicate entries ;       ")')cput-cpus,myid,lvol,if01bsf
     case default ;               write(ounit,'("tr00ab : "f10.2" : myid="i3" ; lvol="i3" ; if01bsf="i2" ; invalid ifail ;           ")')cput-cpus,myid,lvol,if01bsf
     end select
     
    endif ! end of if( Lfirstcall ) ; recall that F01BSF (faster) can only be called after F01BRF (slow);
    
    cpuSparse(2) = MPI_WTIME()
    
    Lfirstcall = .false.
    
    do imp = 0, 2 !Ldmupf ! loop over derivatives wrt mu and poloidal flux;
     
     if( iflag.eq.1 .and. imp.gt.0 ) cycle ! do not need derivatives; this matches logic in mp00ac;
     
    !if( Lcoordinatesingularity .and. imp.eq.2 ) cycle ! do not need derivatives of coordinate singularity field wrt enclosed poloidal flux;
    !if( Louterboundarynormal   .and. imp.eq.1 ) cycle ! do not need derivatives of outermost volume       field wrt helicity multiplier   ;

     if( imp.eq.0 ) dsflangle(imp,lvol,innout,1:mns) = orhs(imp,1:mns)
     if( imp.gt.0 ) dsflangle(imp,lvol,innout,1:mns) = orhs(imp,1:mns) - matmul( omatrix(imp,1:mns,1:mns), dsflangle(0,lvol,innout,1:mns) ) ! matrix perturbation;
     
     mtype = 1 ! recall that AngleSparseMatrix, AngleICN, AngleIKEEP, AngleIDISP(1:2) must be unchanged;
     call F04AXF( NN, AngleSparseMatrix, AngleLICN, AngleICN, AngleIKEEP, dsflangle(imp,lvol,innout,1:mns), Rwork, mtype, AngleIDISP(1:2), resid )
     
     ldiota(imp,innout) = dsflangle(imp,lvol,innout,1)

     if( Wtr00ab .or. Liotastandard .or. Liotasvd ) then
      cput = MPI_WTIME()
      write(ounit,1000)cput-cpus,myid,lvol,innout," sparse ",imp,ldiota(imp,innout),cput-cpuSparse(0),cpuSparse(1)-cpuSparse(0), cpuSparse(2)-cpuSparse(1), cput-cpuSparse(2)
     endif
     
    enddo ! end of do imp = 0, Ldmupf ;

   endif ! end of if( Liotasparse );

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

1000 format("tr00ab : "f10.2" : myid="i3" ; lvol="i3" ; innout="i2" ; "a8" : diota("i1")="es23.15" ; time="f9.4" ;":" construct="f9.4" ; factorize="f9.4" ; solve="f9.4" ;")

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
   if( Liotastandard .or. LiotaSVD ) then
    
    do imp = 0, 2 !Ldmupf
     
     if( iflag.eq.1 .and. imp.gt.0 ) cycle ! do not need derivatives; this matches logic in mp00ac;

     matrix(1:NN,1:NN) = omatrix(0,1:mns,1:mns)
     
     if( imp.eq.0 ) rhs(1:NN) = orhs(imp,1:mns)
     if( imp.gt.0 ) rhs(1:NN) = orhs(imp,1:mns) - matmul( omatrix(imp,1:mns,1:mns), dsflangle(0,lvol,innout,1:mns) ) ! matrix perturbation;

     if( Liotastandard ) then
      
      cpuStandard = MPI_WTIME()
      
      if04atf = 0 ; call F04ATF( matrix(1:mns,1:mns), IA, rhs(1:NN), NN, dsflangle(imp,lvol,innout,1:mns), workmat(1:mns,1:mns), IMat, wka(1:mns), wkb(1:mns), if04atf )
      
      cput = MPI_WTIME()
      select case( if04atf ) 
      case( 0 )  ; if( Wtr00ab ) write(ounit,'("tr00ab : "f10.2" : F04ATF ; if04atf="i2" ; success ;        ")')cput-cpus,if04atf
      case( 1 )  ;               write(ounit,'("tr00ab : "f10.2" : F04ATF ; if04atf="i2" ; singular ;       ")')cput-cpus,if04atf
      case( 2 )  ;               write(ounit,'("tr00ab : "f10.2" : F04ATF ; if04atf="i2" ; ill-conditioned ;")')cput-cpus,if04atf
      case( 3 )  ;               write(ounit,'("tr00ab : "f10.2" : F04ATF ; if04atf="i2" ; input error ;    ")')cput-cpus,if04atf
      case default ;             write(ounit,'("tr00ab : "f10.2" : F04ATF ; if04atf="i2" ; invalid if04atf ;")')cput-cpus,if04atf
      end select
      
      ldiota(imp,innout) = dsflangle(imp,lvol,innout,1)
      
      if( Wtr00ab .or. Liotasparse .or. Liotasvd ) then
       cput = MPI_WTIME()
       write(ounit,1000)cput-cpus,myid,lvol,innout,"standard",imp,ldiota(imp,innout),cput-cpuStandard
      endif
      
     endif ! end of if( Liotastandard) ;
     
     
     if( LiotaSVD ) then

!latex \item If the transform is rational, then the straight-field line angle is not unique.
!latex To accomodate this possibility, the NAG routine \verb+F04JGF+ is available.
      
      cpuSVD = MPI_WTIME()
      
      dsflangle(imp,lvol,innout,1:mns) = rhs(1:mns)
      
      if04jgf = 1 ; call F04JGF( MM, NN, matrix(1:mns,1:mns), NRA, dsflangle(imp,lvol,innout,1:mns), svdtol, SVDlogical, SIGMA, IRANK, wkc(1:lwkc), lwkc, if04jgf )
      
      cput = MPI_WTIME()
      select case( if04jgf ) 
      case( 0 )  ; if( Wtr00ab ) write(ounit,'("tr00ab : "f10.2" : F04JGF ; if04jgf="i2" ; success ;        ")')cput-cpus,if04jgf
      case( 1 )  ;               write(ounit,'("tr00ab : "f10.2" : F04JGF ; if04jgf="i2" ; input error ;    ")')cput-cpus,if04jgf
      case( 2 )  ;               write(ounit,'("tr00ab : "f10.2" : F04JGF ; if04jgf="i2" ; QR fail ;        ")')cput-cpus,if04jgf
      case default ;             write(ounit,'("tr00ab : "f10.2" : F04JGF ; if04jgf="i2" ; invalid if04jgf ;")')cput-cpus,if04jgf
      end select
      
      ldiota(imp,innout) = dsflangle(imp,lvol,innout,1)
      
      if( Wtr00ab .or. Liotasparse .or. Liotastandard ) then
       cput = MPI_WTIME()
       write(ounit,1000)cput-cpus,myid,lvol,innout,"  svd   ",imp,ldiota(imp,innout),cput-cpuSVD
      endif
      
     endif ! end of if( LiotaSVD) ;
     
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
     
    enddo ! end of do imp = 0, Ldmupf;
    
   endif ! end of if( Liotastandard .or. LiotaSVD ) 
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
  enddo ! end of do innout = 0, 1 ! inner and outer interfaces;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  RETURN(tr00ab)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

end subroutine tr00ab

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

