!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Packs, and unpacks, Beltrami field solution vector into vector potential.

!latex \subsubheading{construction of ``vector'' of independent degrees of freedom}

!latex \item Numerical routines for solving linear equations, etc. typically require the unknown, independent degrees of freedom,
!latex which in this case are the covariant components of the vector potential, $A_{\t,l,j,p,i}$ and $A_{\z,l,j,p,i}$,
!latex to be ``packed'' into a vector, ${\bf x}$.

!latex \item The location of $A_{\t,l,j,p,i}$ and $A_{\z,l,j,p,i}$ in this vector are given by \verb+doAt(0,l,j,p)%i(i)+ and \verb+doAz(0,l,j,p)%i(i)+,
!latex which are assigned in \verb+al00aa+.

!latex \item Note that not all of the $A_{\t,l,j,p,i}$ and $A_{\z,l,j,p,i}$ are independent.
!latex \item The boundary condition at the outer interface, \mbox{$\partial_{\t} A_\z - \partial_{\z}A_{\t} =0$},
!latex is satisfied by writing $A_\t = \partial_\t f$ and $A_\z = \partial_\z f$, where $f$ is independent.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine up00aa( packorunpack, lvol, lNBdof, xi, dpf, Ldo ) 

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  use constants, only : zero, one

  use numerical, only :

  use fileunits, only : ounit

  use inputlist, only : Wup00aa, Nvol, Ni, Nofe

  use cputiming, only : Tup00aa

  use allglobal, only : myid, cpus, mn, im, in, dtflux, iBns, iBnc, doAt, doAz, Louterboundarynormal

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  LOCALS

  CHARACTER, intent(in) :: packorunpack
  INTEGER, intent(in)   :: lvol, lNBdof, Ldo
  REAL   , intent(in)   :: dpf
  REAL                  :: xi(1:lNBdof)

  INTEGER               :: ii, ij, ip, iq, mm, nn, mrho, nrho

  BEGIN(up00aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  select case( packorunpack )
  case( 'P' )  ; !   pack vector potential into vector;
  case( 'U' )  ; ! unpack vector potential from vector;       add toroidal and poloidal fluxes;
  case( 'u' )  ; ! unpack vector potential from vector; don't add toroidal and poloidal fluxes;
  case default ; FATALMESS(up00aa,.true.,invalid argument)
  end select

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  if( packorunpack.eq.'P' ) xi(1:lNBdof) = zero ! packing;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  do ii = 0, Ni(lvol)
   do ij = 1, mn ; mm = im(ij) ; nn = in(ij)
    do ip = 0, Nofe
     
     mrho = 1 ; nrho = 1 ! mode multiplication factors associated with outer interface boundary condition;

     if( packorunpack.eq.'u' .or. packorunpack.eq.'U' ) then ; doAt(Ldo,lvol,ij,ip)%s(ii) = zero ! unpacking;
      ;                                                        doAz(Ldo,lvol,ij,ip)%s(ii) = zero
     endif
     
     if( doAt(0,lvol,ij,ip)%i(ii).ne.0 ) then
      
      if( ii.eq.Ni(lvol) .and. ij.gt.1 .and. ip.eq.0 ) mrho =  mm ! not an independent degree of freedom;
      
      FATALMESS(up00aa,doAt(0,lvol,ij,ip)%i(ii).lt.1 .or. doAt(0,lvol,ij,ip)%i(ii).gt.lNBdof,need to check dimension of xi)
      
      if( packorunpack.eq.'u' .or. packorunpack.eq.'U' ) then  ! unpacking;
       ;               doAt(Ldo,lvol,ij,ip)%s(ii)     = xi( doAt(0,lvol,ij,ip)%i(ii) ) * mrho ! unpacking degrees of freedom;
      else
       if( mrho.ne.0 ) xi( doAt(0,lvol,ij,ip)%i(ii) ) = doAt(Ldo,lvol,ij,ip)%s(ii)     / mrho
      endif
      
     endif ! end of if( doAt(0,lvol,ij,ip)%i(ii).ne.0 ) then;

          
     if( doAz(0,lvol,ij,ip)%i(ii).ne.0 ) then
      
      if( ii.eq.Ni(lvol) .and. ij.gt.1 .and. ip.eq.0 ) nrho = -nn ! not an independent degree of freedom;
      
      FATALMESS(up00aa,doAz(0,lvol,ij,ip)%i(ii).lt.1 .or. doAz(0,lvol,ij,ip)%i(ii).gt.lNBdof,need to check dimension of xi)
      
      if( packorunpack.eq.'u' .or. packorunpack.eq.'U' ) then ! unpacking;
       ;               doAz(Ldo,lvol,ij,ip)%s(ii)     = xi( doAz(0,lvol,ij,ip)%i(ii) ) * nrho ! unpacking degrees of freedom;
      else
       if( nrho.ne.0 ) xi( doAz(0,lvol,ij,ip)%i(ii) ) = doAz(Ldo,lvol,ij,ip)%s(ii)     / nrho
      endif
      
     endif ! end of if( doAz(0,lvol,ij,ip)%i(ii).ne.0 ) then;

     
    enddo ! end of do ip;
   enddo ! end of do ij;
  enddo ! end of do ii;
  
  
  if( packorunpack.eq.'U' ) then ! unpacking; adding inhomogeneous toroidal and poloidal flux;
   
   ii = Ni(lvol) ; ip = 0 ! outermost sub-grid; zero-th derivative;
   
   ij = 1 ! constant harmonic; always included; inhomogeneous term arising from enclosed toroidal and poloidal flux;
   
   doAt(Ldo,lvol,ij,ip)%s(ii) = dtflux(lvol)
   doAz(Ldo,lvol,ij,ip)%s(ii) = dpf
   
   if( Louterboundarynormal ) then ! include inhomogeneous terms that produce non-zero normal field;
    
    ii = Ni(lvol) ; ip = 0 ! outermost sub-grid; zero-th derivative;
    
    do ij = 2, mn
     
     if( in(ij).ne.0 .and. im(ij).eq.0 ) doAt(Ldo,lvol,ij,ip)%s(ii) = doAt(Ldo,lvol,ij,ip)%s(ii) - iBns(ij) / in(ij) ! include non-zero normal field; see ma01ag;
     if( im(ij).ne.0                   ) doAz(Ldo,lvol,ij,ip)%s(ii) = doAz(Ldo,lvol,ij,ip)%s(ii) - iBns(ij) / im(ij) ! include non-zero normal field; see ma01ag;
     
    enddo
    
   endif ! end of if( Louterboundarynormal ) then;
   
  endif ! end of if( packorunpack.eq.'U' ) then;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  RETURN(up00aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

end subroutine up00aa

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
