!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Computes volume and, if required, the derivatives of the volume with respect to the interface geometry.

!latex \end{enumerate} \subsubsection{volume} \begin{enumerate}
!latex \item The volume enclosed by $(l-1)$ and $l$-th interfaces is given by the integral
!latex \be V_l = \int_{s_{l-1}}^{s_{l}} \!\!\! ds \int_{0}^{2\pi} \!\!\! d\t \int_{0}^{2\pi/N} \!\!\! d\z \;\;\; \sqrt g,
!latex \ee
!latex where $\sqrt g$ is the Jacobian, which depends on the coordinate functions, $R(\s,\t,\z)$ and $Z(\s,\t,\z)$.
!latex \item In the following, the coordinate functions, 
!latex \be R&=&\sum_i R_i(s) \cos(m_i \t - n_i \z),\\
!latex     Z&=&\sum_i Z_i(s) \sin(m_i \t - n_i \z),
!latex \ee
!latex are constructed from a linear interpolation of the interface harmonics, so that
!latex \be R_i &=& R_{l-1,i} + \Delta R_i \; \bar s,\\
!latex     Z_i &=& Z_{l-1,i} + \Delta Z_i \; \bar s,
!latex \ee
!latex where $\bar s \equiv (s-s_{l-1})/\Delta s$, $\Delta s \equiv s_{l}-s_{l-1}$, $\Delta R_i \equiv R_{l,i}-R_{l-1,i}$ and $\Delta Z_i \equiv Z_{l,i}-Z_{l-1,i}$.
!latex \item In the innermost volume for the cylindrical and toroidal case, however, we use 
!latex \be R_i&=&R_{1,i}s^{m_i/2}/s_{1}^{m_i/2}, \\
!latex     Z_i&=&Z_{1,i}s^{m_i/2}/s_{1}^{m_i/2}.
!latex \ee
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine vo00aa( lvol )

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  use constants, only : zero, half, third, quarter, sixth, pi2 

  use numerical, only : vsmall

  use fileunits, only : ounit

  use inputlist, only : Wvo00aa, Itoroidal, Lgeometry, Nvol, Nfp

  use cputiming

  use allglobal, only : myid, cpus, mn, im, in, iRbc, iZbs, interfacelabel, aijk, bijk, lvolume, dAd, dvolume, pi2pi2nfpquarter, pi2nfp

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  LOCALS

  INTEGER, intent(in) :: lvol

  INTEGER             :: imn, jmn, kmn, mm, nn, pp, jj, kk, mi, ni, mj, nj, mk, nk, imjmkm

  REAL                :: Rim(0:3), Rjn(0:3), Zkp(0:3)

  REAL                :: dRi, dRj, dRk, Rli, Rlj, Rlk
  REAL                :: dZi, dZj, dZk, Zli, Zlj, Zlk

  REAL                :: iRbcl, iRbcu

  BEGIN(vo00aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  FATALMESS(vo00aa,lvol.lt.1.or.lvol.gt.Nvol,invalid interface label)
  
              lvolume(lvol) = zero ! initialize;
  if( dAd%L ) dvolume       = zero ! derivatives of volume wrt interface inner/outer, R/Z harmonic;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!latex \end{enumerate} \subsubsection{geometry} 
!latex The geometry is controlled by the input parameter \verb+Lgeometry+ as follows: 

  select case( Lgeometry ) !latex \begin{enumerate}
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
  case( 1 ) !latex \item \verb+Lgeometry.eq.1+ : Cartesian : $\sqrt g = R_s$.
   
    lvolume(lvol) =   pi2*pi2nfp * ( iRbc(lvol,1) - iRbc(lvol-1,1) )
   
   if( dAd%L .and. dAd%innout.eq.0 .and. dAd%irz.eq.0 .and. dAd%imn.eq.1 ) then
    dvolume       = - pi2*pi2nfp
   endif
   
   if( dAd%L .and. dAd%innout.eq.1 .and. dAd%irz.eq.0 .and. dAd%imn.eq.1 ) then
    dvolume       = + pi2*pi2nfp
   endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
  case( 2 ) !latex \item \verb+Lgeometry.eq.2+ : cylindrical : $\sqrt g = R R_s$.
   
   do jj = 1, mn ; mj = im(jj) ; nj = in(jj)
    do kk = 1, mn ; mk = im(kk) ; nk = in(kk)
     
     if( mj+mk.eq.0 .and. nj+nk.eq.0 ) lvolume(lvol) = lvolume(lvol) + iRbc(lvol  ,jj) * iRbc(lvol  ,kk)
     if( mj-mk.eq.0 .and. nj-nk.eq.0 ) lvolume(lvol) = lvolume(lvol) + iRbc(lvol  ,jj) * iRbc(lvol  ,kk)
     
     if( lvol.gt.1 ) then
     if( mj+mk.eq.0 .and. nj+nk.eq.0 ) lvolume(lvol) = lvolume(lvol) - iRbc(lvol-1,jj) * iRbc(lvol-1,kk) 
     if( mj-mk.eq.0 .and. nj-nk.eq.0 ) lvolume(lvol) = lvolume(lvol) - iRbc(lvol-1,jj) * iRbc(lvol-1,kk)
     endif

    enddo
   enddo
   
   lvolume(lvol) = lvolume(lvol) * pi2pi2nfpquarter !pi2/Nfp * quarter

   FATALMESS(vo00aa,dAd%L,have not yet implemented derivatives)
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
  case( 3 ) !latex \item \verb+Lgeometry.eq.3+ : toroidal : see mathematica notebook for jacobian etc.
   
   do imn = 1, mn ; mi = im(imn) ; ni = in(imn)
    do jmn = 1, mn ; mj = im(jmn) ; nj = in(jmn)
     
                      iRbcu = iRbc(lvol  ,imn) * iRbc(lvol  ,jmn)
      if( lvol.gt.1 ) iRbcl = iRbc(lvol-1,imn) * iRbc(lvol-1,jmn)
      if( lvol.eq.1 ) iRbcl = zero
     
      if( mi+mj     .eq.0 .and. ni+nj   .eq.0) lvolume(lvol) = lvolume(lvol) + ( iRbcu - iRbcl ) * iRbc(0,1)
      if( mi-mj     .eq.0 .and. ni-nj   .eq.0) lvolume(lvol) = lvolume(lvol) + ( iRbcu - iRbcl ) * iRbc(0,1)
     
     do kmn = 1, mn ; mk = im(kmn) ; nk = in(kmn) 

                      iRbcu = iRbc(lvol  ,imn) * iRbc(lvol  ,jmn) * iRbc(lvol  ,kmn)
      if( lvol.gt.1 ) iRbcl = iRbc(lvol-1,imn) * iRbc(lvol-1,jmn) * iRbc(lvol-1,kmn)
      if( lvol.eq.1 ) iRbcl = zero
      
      if( mi+mj+mk+1.eq.0 .and. ni+nj+nk.eq.0) lvolume(lvol) = lvolume(lvol) + ( iRbcu - iRbcl ) * sixth
      if(-mi+mj+mk+1.eq.0 .and.-ni+nj+nk.eq.0) lvolume(lvol) = lvolume(lvol) + ( iRbcu - iRbcl ) * sixth
      if( mi-mj+mk+1.eq.0 .and. ni-nj+nk.eq.0) lvolume(lvol) = lvolume(lvol) + ( iRbcu - iRbcl ) * sixth
      if(-mi-mj+mk+1.eq.0 .and.-ni-nj+nk.eq.0) lvolume(lvol) = lvolume(lvol) + ( iRbcu - iRbcl ) * sixth
      if( mi+mj-mk+1.eq.0 .and. ni+nj-nk.eq.0) lvolume(lvol) = lvolume(lvol) + ( iRbcu - iRbcl ) * sixth
      if(-mi+mj-mk+1.eq.0 .and.-ni+nj-nk.eq.0) lvolume(lvol) = lvolume(lvol) + ( iRbcu - iRbcl ) * sixth
      if( mi-mj-mk+1.eq.0 .and. ni-nj-nk.eq.0) lvolume(lvol) = lvolume(lvol) + ( iRbcu - iRbcl ) * sixth
      if(-mi-mj-mk+1.eq.0 .and.-ni-nj-nk.eq.0) lvolume(lvol) = lvolume(lvol) + ( iRbcu - iRbcl ) * sixth
      
     enddo
     
    enddo
   enddo
   
   lvolume(lvol) = lvolume(lvol) * pi2pi2nfpquarter
   
   FATALMESS(vo00aa,dAd%L,have not yet implemented derivatives)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
  case( 5 ) !latex \item \verb+Lgeometry.eq.5+ : cylindrical : $\sqrt g = R_s Z_\t - R_\t Z_s$.
   
!latex \be V_l &=& \sum_{i,j} m_j \; \Delta R_i \; \int_{0}^{1} d \bar s \; Z_j(\bar s) \ooint \cos \alpha_i \cos \alpha_j\\
!latex         &+& \sum_{i,j} m_i \; \Delta Z_j \; \int_{0}^{1} d \bar s \; R_i(\bar s) \ooint \sin \alpha_i \sin \alpha_j
!latex \ee
   
   if( lvol.eq.1 ) then
    
    do imn = 1, mn ; mi = im(imn) ; ni = in(imn)
     do jmn = 1, mn ; mj = im(jmn) ; nj = in(jmn)
      
! This first term is zero, so I've commented it out.
!     if ( mi+mj.eq.0 .and. ni+nj.eq.0 ) volume(0,lvol)=volume(0,lvol) + mj*mi*iRbc(lvol,imn)*iZbs(lvol,jmn)/(mi+mj) - mj*mi*iRbc(lvol,imn)*iZbs(lvol,jmn)/(mi+mj)

      if ( mi-mj.eq.0 .and. ni-nj.eq.0 .and. (mi+mj).gt.0) lvolume(lvol) = lvolume(lvol) + 2.0*mi*mj*iRbc(lvol,imn)*iZbs(lvol,jmn)/(mi+mj)
      
     enddo
    enddo
    
   else
    
    do imn = 1, mn ; mi=im(imn); ni=in(imn); dRi=iRbc(lvol,imn)-iRbc(lvol-1,imn); Rli=iRbc(lvol-1,imn) ; dZi=iZbs(lvol,imn)-iZbs(lvol-1,imn); Zli=iZbs(lvol-1,imn)
     do jmn = 1, mn ; mj=im(jmn); nj=in(jmn); dRj=iRbc(lvol,jmn)-iRbc(lvol-1,jmn); Rlj=iRbc(lvol-1,jmn) ; dZj=iZbs(lvol,jmn)-iZbs(lvol-1,jmn); Zlj=iZbs(lvol-1,jmn)
      
      if ( mi+mj.eq.0 .and. ni+nj.eq.0 ) lvolume(lvol) = lvolume(lvol) + mj * dRi * (Zlj + half * dZj) - mi * dZj * (Rli + half * dRi)
      if ( mi-mj.eq.0 .and. ni-nj.eq.0 ) lvolume(lvol) = lvolume(lvol) + mj * dRi * (Zlj + half * dZj) + mi * dZj * (Rli + half * dRi)
      
     enddo
    enddo
   endif
   
   lvolume(lvol) = lvolume(lvol) * half * pi2 * pi2nfp !pi2/Nfp

   FATALMESS(vo00aa,dAd%L,have not yet implemented derivatives)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
  case( 6 ) !latex \item \verb+Lgeometry.eq.6+ : toroidal : $\sqrt g = R ( R_\s Z_\t - R_\t Z_\s)$.

!latex \begin{enumerate}
!latex \item An expression for the $l$-th volume is
!latex \be V &=& \sum_{i,j,k} m_k \; \Delta R_j \; \int_0^1 d \bar s \; R_i(\bar s) Z_k(\bar s) \ooint \cos \alpha_i \cos \alpha_j \cos \alpha_k\\
!latex       &+& \sum_{i,j,k} m_j \; \Delta Z_k \; \int_0^1 d \bar s \; R_i(\bar s) R_j(\bar s) \ooint \cos \alpha_i \sin \alpha_j \sin \alpha_k,
!latex \ee
!latex where $\alpha_i \equiv m_i \t - n_i \z$ and the integrals over $\bar s$ are
!latex \be \int_0^1 d \bar s \; R_i(\bar s) Z_k(\bar s) & = & R_{l-1,i} Z_{l-1,k} + ( R_{l-1,i} \Delta Z_{k} + \Delta R_{i} Z_{l-1,k})/2 + \Delta R_{i} \Delta Z_{k} / 3 \\
!latex     \int_0^1 d \bar s \; R_i(\bar s) R_j(\bar s) & = & R_{l-1,i} R_{l-1,j} + ( R_{l-1,i} \Delta R_{j} + \Delta R_{i} R_{l-1,j})/2 + \Delta R_{i} \Delta R_{j} / 3.
!latex \ee
!latex \item The derivatives of the volume integral with respect to the interface Fourier harmonics are
!latex \be
!latex \begin{array}{cccccccccccccccccccccccccccccccccccccccccccccccc}
!latex \ds \frac{\partial V}{\partial R_{l-1,i}} & = & \sum_{j,k} & \Delta R_{j} & m_k & \ds \int_0^1 d \bar s &  (1-\bar s) & Z_k(\bar s) & \ds \ooint c_i c_j c_k \\
!latex                                           & + & \sum_{j,k} & \Delta Z_{k} & m_j & \ds \int_0^1 d \bar s &  (1-\bar s) & R_j(\bar s) & \ds \ooint c_i s_j s_k \\
!latex                                           & - & \sum_{j,k} &              & m_k & \ds \int_0^1 d \bar s & R_j(\bar s) & Z_k(\bar s) & \ds \ooint c_i c_j c_k \\
!latex                                           & + & \sum_{j,k} & \Delta Z_{k} & m_i & \ds \int_0^1 d \bar s & R_j(\bar s) &  (1-\bar s) & \ds \ooint c_j s_i s_k 
!latex \end{array} \\
!latex \begin{array}{cccccccccccccccccccccccccccccccccccccccccccccccc}
!latex \ds \frac{\partial V}{\partial R_{l  ,i}} & = & \sum_{j,k} & \Delta R_{j} & m_k & \ds \int_0^1 d \bar s &     \bar s  & Z_k(\bar s) & \ds \ooint c_i c_j c_k \\
!latex                                           & + & \sum_{j,k} & \Delta Z_{k} & m_j & \ds \int_0^1 d \bar s &     \bar s  & R_j(\bar s) & \ds \ooint c_i s_j s_k \\
!latex                                           & + & \sum_{j,k} &              & m_k & \ds \int_0^1 d \bar s & R_j(\bar s) & Z_k(\bar s) & \ds \ooint c_i c_j c_k \\
!latex                                           & + & \sum_{j,k} & \Delta Z_{k} & m_i & \ds \int_0^1 d \bar s & R_j(\bar s) &     \bar s  & \ds \ooint c_j s_i s_k 
!latex \end{array} \\
!latex \begin{array}{cccccccccccccccccccccccccccccccccccccccccccccccc}
!latex \ds \frac{\partial V}{\partial Z_{l-1,i}} & = & \sum_{j,k} & \Delta R_{k} & m_i & \ds \int_0^1 d \bar s &  (1-\bar s) & R_j(\bar s) & \ds \ooint c_i c_j c_k \\
!latex                                           & - & \sum_{j,k} &              & m_k & \ds \int_0^1 d \bar s & R_j(\bar s) & R_k(\bar s) & \ds \ooint c_j s_i s_k
!latex \end{array} \\
!latex \begin{array}{cccccccccccccccccccccccccccccccccccccccccccccccc}
!latex \ds \frac{\partial V}{\partial Z_{l  ,i}} & = & \sum_{j,k} & \Delta R_{k} & m_i & \ds \int_0^1 d \bar s &     \bar s  & R_j(\bar s) & \ds \ooint c_i c_j c_k \\
!latex                                           & + & \sum_{j,k} &              & m_k & \ds \int_0^1 d \bar s & R_j(\bar s) & R_k(\bar s) & \ds \ooint c_j s_i s_k
!latex \end{array} 
!latex \ee
!latex The integrals over $\bar s$, for $F=R,Z$, are
!latex \be \begin{array}{ccccl} \ds \int_0^1 d \bar s &    \bar s  & F_{k}(\bar s) &=& F_{l-1,k}/2 +  \Delta F_{l-1,k}/3 \\
!latex                          \ds \int_0^1 d \bar s & (1-\bar s) & F_{k}(\bar s) &=& F_{l-1,k} + (\Delta F_{k}-F_{l-1,k})/2-\Delta F_{k}/3.
!latex \end{array} \ee
!latex \end{enumerate}

   if( lvol.eq.1 ) then
    
    do imn = 1, mn ; mi = im(imn)
     do jmn = 1, mn ; mj = im(jmn)
      do kmn = 1, mn ; mk = im(kmn) ; imjmkm = mi + mj + mk
       
       if( aijk(imn,jmn,kmn).ne.0 .and. mj.gt.0 ) lvolume(lvol) = lvolume(lvol) + mj * iRbc(1,imn) * iRbc(1,jmn) * iZbs(1,kmn) / imjmkm * aijk(imn,jmn,kmn)
       if( bijk(imn,jmn,kmn).ne.0 .and. mk.gt.0 ) lvolume(lvol) = lvolume(lvol) + mk * iRbc(1,imn) * iRbc(1,jmn) * iZbs(1,kmn) / imjmkm * bijk(imn,jmn,kmn)
       
      enddo
     enddo
    enddo

    if(  dAd%L ) then
     
     FATALMESS(vo00aa,dAd%imn.lt.0.or.dAd%imn.gt.mn,invalid subscript)
     
    imn = dAd%imn  ; mi = im(imn)
     do jmn = 1, mn ; mj = im(jmn)
      do kmn = 1, mn ; mk = im(kmn) ; imjmkm = mi + mj + mk

       if( dAd%innout.eq.0 .and. dAd%irz.eq.0 ) then ! perturb inner interface Rj; no dependence;
       endif
       
       if( dAd%innout.eq.0 .and. dAd%irz.eq.1 ) then ! perturb inner interface Zj; no dependence;
       endif
       
       if( dAd%innout.eq.1 .and. dAd%irz.eq.0 ) then ! perturb outer interface Rj;
        if( mj.gt.0 ) dvolume       = dvolume       + ( mj * iRbc(1,jmn) * iZbs(1,kmn) ) / imjmkm * aijk(imn,jmn,kmn) ! ci cj ck m_k ;
        if( mk.gt.0 ) dvolume       = dvolume       + ( mk * iRbc(1,jmn) * iZbs(1,kmn) ) / imjmkm * bijk(imn,jmn,kmn) ! ci sj sk m_j ;
        if( mi.gt.0 ) dvolume       = dvolume       + ( mi * iRbc(1,jmn) * iZbs(1,kmn) ) / imjmkm * aijk(imn,jmn,kmn) ! ci cj ck m_k ;
        if( mk.gt.0 ) dvolume       = dvolume       + ( mk * iRbc(1,jmn) * iZbs(1,kmn) ) / imjmkm * bijk(jmn,imn,kmn) ! cj si sk m_i ;
       endif
       
       if( dAd%innout.eq.1 .and. dAd%irz.eq.1 ) then ! perturb outer interface Zj;
        if( mj.gt.0 ) dvolume       = dvolume       + ( mj * iRbc(1,kmn) * iRbc(1,jmn) ) / imjmkm * aijk(kmn,jmn,imn) ! ci cj ck m_k ;
        if( mi.gt.0 ) dvolume       = dvolume       + ( mi * iRbc(1,kmn) * iRbc(1,jmn) ) / imjmkm * bijk(kmn,jmn,imn) ! ci sj sk m_j ;
       endif

      enddo
     enddo
    endif
       
   else ! matches if( lvol.eq.1 );
    
    do imn = 1, mn ; Rli = iRbc(lvol-1,imn) ; dRi = iRbc(lvol,imn) - Rli ; Zli = iZbs(lvol-1,imn) ; dZi = iZbs(lvol,imn) - Zli
     do jmn = 1, mn ; Rlj = iRbc(lvol-1,jmn) ; dRj = iRbc(lvol,jmn) - Rlj ; Zlj = iZbs(lvol-1,jmn) ; dZj = iZbs(lvol,jmn) - Zlj
      do kmn = 1, mn ; Rlk = iRbc(lvol-1,kmn) ; dRk = iRbc(lvol,kmn) - Rlk ; Zlk = iZbs(lvol-1,kmn) ; dZk = iZbs(lvol,kmn) - Zlk
       
       if( aijk(imn,jmn,kmn).ne.0 ) lvolume(lvol) = lvolume(lvol) + aijk(imn,jmn,kmn) * dRj * ( Rli*Zlk + (Rli*dZk+dRi*Zlk)*half + dRi*dZk*third )
       if( bijk(imn,jmn,kmn).ne.0 ) lvolume(lvol) = lvolume(lvol) + bijk(imn,jmn,kmn) * dZk * ( Rli*Rlj + (Rli*dRj+dRi*Rlj)*half + dRi*dRj*third )
       
      enddo
     enddo
    enddo
    
    if(  dAd%L ) then
    imn = dAd%imn  ; Rli = iRbc(lvol-1,imn) ; dRi = iRbc(lvol,imn) - Rli ; Zli = iZbs(lvol-1,imn) ; dZi = iZbs(lvol,imn) - Zli
     do jmn = 1, mn ; Rlj = iRbc(lvol-1,jmn) ; dRj = iRbc(lvol,jmn) - Rlj ; Zlj = iZbs(lvol-1,jmn) ; dZj = iZbs(lvol,jmn) - Zlj
      do kmn = 1, mn ; Rlk = iRbc(lvol-1,kmn) ; dRk = iRbc(lvol,kmn) - Rlk ; Zlk = iZbs(lvol-1,kmn) ; dZk = iZbs(lvol,kmn) - Zlk

       if( dAd%innout.eq.0 .and. dAd%irz.eq.0 ) then ! perturb inner interface Rj;
        dvolume       = dvolume       + dRj * ( Zlk     + ( dZk - Zlk       )*half - dZk    *third ) * aijk(imn,jmn,kmn) & ! ci cj ck m_k ;
                                      + dZk * ( Rlj     + ( dRj - Rlj       )*half - dRj    *third ) * bijk(imn,jmn,kmn) & ! ci sj sk m_j ;
                                      -       ( Rlj*Zlk + ( Rlj*dZk+dRj*Zlk )*half + dRj*dZk*third ) * aijk(imn,jmn,kmn) & ! ci cj ck m_k ;
                                      + dZk * ( Rlj     + ( dRj - Rlj       )*half - dRj    *third ) * bijk(jmn,imn,kmn)   ! cj si sk m_i ;
       endif

       if( dAd%innout.eq.0 .and. dAd%irz.eq.1 ) then ! perturb inner interface Zj;
        dvolume       = dvolume       + dRk * ( Rlj     + ( dRj - Rlj       )*half - dRj    *third ) * aijk(kmn,jmn,imn) & ! ci cj ck m_i ;
                                      -       ( Rlj*Rlk + ( Rlj*dRk+dRj*Rlk )*half + dRj*dRk*third ) * bijk(jmn,kmn,imn)   ! cj si sk m_k ;
       endif

       if( dAd%innout.eq.1 .and. dAd%irz.eq.0 ) then ! perturb outer interface Rj;
        dvolume       = dvolume       + dRj * (           ( Zlk             )*half + dZk    *third ) * aijk(imn,jmn,kmn) & ! ci cj ck m_k ;
                                      + dZk * (           ( Rlj             )*half + dRj    *third ) * bijk(imn,jmn,kmn) & ! ci sj sk m_j ;
                                      +       ( Rlj*Zlk + ( Rlj*dZk+dRj*Zlk )*half + dRj*dZk*third ) * aijk(imn,jmn,kmn) & ! ci cj ck m_k ;
                                      + dZk * (           ( Rlj             )*half + dRj    *third ) * bijk(jmn,imn,kmn)   ! cj si sk m_i ;
       endif

       if( dAd%innout.eq.1 .and. dAd%irz.eq.1 ) then ! perturb outer interface Zj;
        dvolume       = dvolume       + dRk * (           ( Rlj             )*half + dRj    *third ) * aijk(kmn,jmn,imn) & ! ci cj ck m_i ;
                                      +       ( Rlj*Rlk + ( Rlj*dRk+dRj*Rlk )*half + dRj*dRk*third ) * bijk(jmn,kmn,imn)   ! cj si sk m_k ;
       endif

      enddo ! end of do kmn;
     enddo ! end of do jmn;
    endif
    
   endif ! end of if( lvol.eq.1 );
   
               lvolume(lvol) = - Itoroidal * lvolume(lvol) * pi2pi2nfpquarter ! recall definition of the aijk & bijk;
   if( dAd%L ) dvolume       = - Itoroidal * dvolume       * pi2pi2nfpquarter ! pi2pi2nfpquarter matches definition of aijk & bijk;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
  case default
   
   FATALMESS(vo00aa,.true.,invalid Lgeometry)
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
  end select
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  if( Wvo00aa ) then
   cput = MPI_WTIME() ; write(ounit,'("vo00aa : "f10.2" : myid="i3" ; Lgeometry="i2" ; lvolume("i3" )="es23.15" ;")')cput-cpus,myid,Lgeometry,lvol,lvolume(lvol)
  endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  FATALMESS(vo00aa,lvolume(lvol).lt.vsmall,volume cannot be zero or negative; change Itoroidal;)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  RETURN(vo00aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

end subroutine vo00aa

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
